/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.admin ;

import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.Dimension;
import java.io.Serializable;
import java.util.StringTokenizer;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.border.TitledBorder;

import org.apache.commons.lang.StringUtils;

import com.funambol.framework.engine.source.ContentType;
import com.funambol.framework.engine.source.SyncSourceInfo;
import com.funambol.admin.ui.SourceManagementPanel;
import com.funambol.admin.AdminException;
import com.funambol.admin.mo.SyncSourceManagementObject;


import com.funambol.email.engine.source.EmailSyncSource;
import com.funambol.email.engine.source.ImapEmailSyncSource;
import com.funambol.email.engine.source.PopEmailSyncSource;
import com.funambol.email.items.model.DefaultFolder;
import com.funambol.email.util.Def;


/**
 * This class implements the configuration panel for an EmailSyncSource
 *
 * @version $Id: EmailSyncSourceConfigPanel.java,v 1.25 2006/11/14 15:32:15 gbmiglia Exp $
 */
public class EmailSyncSourceConfigPanel
        extends SourceManagementPanel
        implements Serializable {

    // --------------------------------------------------------------- Constants

    public static final String NAME_ALLOWED_CHARS =
            "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890-_.";
    public static final String PORT_ALLOWED_CHARS = "0123456789";

    // ------------------------------------------------------------- Public Data

    /**
     * label for the panel's name
     */
    public JLabel panelName = new JLabel();

    /**
     * border to evidence the title of the panel
     */
    public TitledBorder titledBorder1;

    public JLabel nameLabel          = new JLabel();
    public JTextField nameValue      = new JTextField();
    public JLabel sourceUriLabel     = new JLabel();
    public JTextField sourceUriValue = new JTextField();

    //--
    public JLabel serverTypeLabel     = new JLabel();
    public JComboBox serverTypeValue  = new JComboBox();
    public JLabel outHostLabel        = new JLabel();
    public JTextField outHostValue    = new JTextField();
    public JLabel outPortLabel        = new JLabel();
    public JTextField outPortValue    = new JTextField();
    public JLabel outAuthLabel        = new JLabel();
    public JCheckBox outAuthValue     = new JCheckBox();
    //--
    public JLabel inHostLabel     = new JLabel();
    public JTextField inHostValue = new JTextField();
    public JLabel inPortLabel     = new JLabel();
    public JTextField inPortValue = new JTextField();

    public JLabel sslCheckLabel               = new JLabel();
    public JCheckBox sslCheckValue            = new JCheckBox();
    public JLabel keystoreLabel               = new JLabel();
    public JTextField keystoreValue           = new JTextField();
    public JLabel keystorePassPhraseLabel     = new JLabel();
    public JTextField keystorePassPhraseValue = new JTextField();

    //--

    public JLabel allmailboxCheckLabel     = new JLabel();
    public JCheckBox allmailboxCheckValue  = new JCheckBox();

    public JLabel inboxLabel               = new JLabel();
    public JTextField inboxValue           = new JTextField();
    public JCheckBox inboxActivationValue  = new JCheckBox();
    public JLabel outboxLabel              = new JLabel();
    public JTextField outboxValue          = new JTextField();
    public JCheckBox outboxActivationValue = new JCheckBox();
    public JLabel sentLabel                = new JLabel();
    public JTextField sentValue            = new JTextField();
    public JCheckBox sentActivationValue   = new JCheckBox();
    public JLabel draftLabel               = new JLabel();
    public JTextField draftValue           = new JTextField();
    public JCheckBox draftActivationValue  = new JCheckBox();
    public JLabel trashLabel               = new JLabel();
    public JTextField trashValue           = new JTextField();
    public JCheckBox trashActivationValue  = new JCheckBox();

    // --

    public JLabel typeLabel             = new JLabel();
    public JTextField typeValue         = new JTextField();
    public JLabel infoTypesLabel        = new JLabel();
    public JTextField infoTypesValue    = new JTextField();
    public JLabel infoVersionsLabel     = new JLabel();
    public JTextField infoVersionsValue = new JTextField();

    public JLabel     dtEncryptLabel    = new JLabel() ;
    public JCheckBox  dtEncryptValue    = new JCheckBox() ;

    public JLabel     softDeleteLabel    = new JLabel() ;
    public JCheckBox  softDeleteValue    = new JCheckBox() ;
    
    
    public JButton confirmButton = new JButton();

    // ------------------------------------------------------------ Constructors

    public EmailSyncSourceConfigPanel() {
        init();
    }

    // ---------------------------------------------------------- Public methods

    /**
     * Create the panel
     */
    public void init() {

        // set layout
        this.setLayout(null);

        // set properties of label, position and border
        //  referred to the title of the panel
        titledBorder1 = new TitledBorder("");

        panelName.setFont(titlePanelFont);
        panelName.setText("Edit Email Connector SyncSource");
        panelName.setBounds(new Rectangle(14, 5, 316, 28));
        panelName.setAlignmentX(SwingConstants.CENTER);
        panelName.setBorder(titledBorder1);

        // --------

        sourceUriLabel.setText("Source URI: ");
        sourceUriLabel.setFont(defaultFont);
        sourceUriLabel.setBounds(new Rectangle(14, 70, 150, 19));
        sourceUriValue.setFont(defaultFont);
        sourceUriValue.setBounds(new Rectangle(170, 70, 350, 19));

        nameLabel.setText("Name: ");
        nameLabel.setFont(defaultFont);
        nameLabel.setBounds(new Rectangle(14, 90, 150, 19));
        nameValue.setFont(defaultFont);
        nameValue.setBounds(new Rectangle(170, 90, 350, 19));

        // -------- server type
        
        serverTypeLabel.setText("Server Type: ");
        serverTypeLabel.setFont(defaultFont);
        serverTypeLabel.setBounds(new Rectangle(14, 120, 150, 19));
        serverTypeValue.setFont(defaultFont);
        serverTypeValue.addItem(Def.SERVER_TYPE_EXCHANGE);
        serverTypeValue.addItem(Def.SERVER_TYPE_COURIER);
        serverTypeValue.addItem(Def.SERVER_TYPE_OTHER);
        serverTypeValue.setBounds(170, 120, 350, 19);
        
        // -------- outgoing
        
        outHostLabel.setText("Outgoing Server : ");
        outHostLabel.setFont(defaultFont);
        outHostLabel.setBounds(new Rectangle(14, 142, 150, 19));
        outHostValue.setFont(defaultFont);
        outHostValue.setBounds(new Rectangle(170, 142, 160, 19));

        outPortLabel.setText("Port: ");
        outPortLabel.setFont(defaultFont);
        outPortLabel.setBounds(new Rectangle(340, 142, 30, 19));
        outPortValue.setFont(defaultFont);
        outPortValue.setBounds(new Rectangle(380, 142, 70, 19));

        outAuthLabel.setText("Auth: ");
        outAuthLabel.setFont(defaultFont);
        outAuthLabel.setBounds(new Rectangle(460, 142, 30, 19));
        outAuthValue.setSelected(false);
        outAuthValue.setBounds(new Rectangle(500, 142, 20, 19));

        // -------- incoming

        inHostLabel.setText("Incoming Server : ");
        inHostLabel.setFont(defaultFont);
        inHostLabel.setBounds(new Rectangle(14, 162, 150, 19));
        inHostValue.setFont(defaultFont);
        inHostValue.setBounds(new Rectangle(170, 162, 160, 19));

        inPortLabel.setText("Port: ");
        inPortLabel.setFont(defaultFont);
        inPortLabel.setBounds(new Rectangle(340, 162, 30, 19));
        inPortValue.setFont(defaultFont);
        inPortValue.setBounds(new Rectangle(380, 162, 70, 19));


        // -------- SSL

        sslCheckLabel.setText("SSL: ");
        sslCheckLabel.setFont(defaultFont);
        sslCheckLabel.setBounds(new Rectangle(14, 182, 150, 19));
        sslCheckValue.setSelected(false);
        sslCheckValue.setBounds(new Rectangle(166, 182, 350, 19));

        sslCheckValue.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (e.getSource().equals(sslCheckValue)) {
                    keystoreLabel.setEnabled(sslCheckValue.isSelected());
                    keystoreValue.setEnabled(sslCheckValue.isSelected());
                    keystorePassPhraseLabel.setEnabled(sslCheckValue.isSelected());
                    keystorePassPhraseValue.setEnabled(sslCheckValue.isSelected());
                }
            }
        });

        keystoreLabel.setText("Keystore Path: ");
        keystoreLabel.setFont(defaultFont);
        keystoreLabel.setBounds(new Rectangle(14, 202, 150, 19));
        keystoreLabel.setEnabled(false);
        keystoreValue.setFont(defaultFont);
        keystoreValue.setBounds(new Rectangle(170, 202, 350, 19));
        keystoreValue.setEnabled(false);

        keystorePassPhraseLabel.setText("Keystore Passphrase: ");
        keystorePassPhraseLabel.setFont(defaultFont);
        keystorePassPhraseLabel.setBounds(new Rectangle(14, 222, 150, 19));
        keystorePassPhraseLabel.setEnabled(false);
        keystorePassPhraseValue.setFont(defaultFont);
        keystorePassPhraseValue.setBounds(new Rectangle(170, 222, 350, 19));
        keystorePassPhraseValue.setEnabled(false);

        // --------

        allmailboxCheckLabel.setText("Complete Mailbox Activation: ");
        allmailboxCheckLabel.setFont(defaultFont);
        allmailboxCheckLabel.setBounds(new Rectangle(14, 242, 150, 19));
        allmailboxCheckValue.setSelected(false);
        allmailboxCheckValue.setBounds(new Rectangle(166, 242, 350, 19));

        inboxLabel.setText("Inbox Folder Name: ");
        inboxLabel.setFont(defaultFont);
        inboxLabel.setBounds(new Rectangle(14, 262, 150, 19));
        inboxValue.setFont(defaultFont);
        inboxValue.setBounds(new Rectangle(170, 262, 250, 19));
        inboxActivationValue.setSelected(true);
        inboxActivationValue.setBounds(new Rectangle(500, 262, 50, 19));

        outboxLabel.setText("Outbox Folder Name: ");
        outboxLabel.setFont(defaultFont);
        outboxLabel.setBounds(new Rectangle(14, 282, 150, 19));
        outboxValue.setFont(defaultFont);
        outboxValue.setBounds(new Rectangle(170, 282, 250, 19));
        outboxActivationValue.setSelected(true);
        outboxActivationValue.setBounds(new Rectangle(500, 282, 50, 19));

        sentLabel.setText("Sent Folder Name: ");
        sentLabel.setFont(defaultFont);
        sentLabel.setBounds(new Rectangle(14, 302, 150, 19));
        sentValue.setFont(defaultFont);
        sentValue.setBounds(new Rectangle(170, 302, 250, 19));
        sentActivationValue.setSelected(true);
        sentActivationValue.setBounds(new Rectangle(500, 302, 50, 19));

        draftLabel.setText("Drafts Folder Name: ");
        draftLabel.setFont(defaultFont);
        draftLabel.setBounds(new Rectangle(14, 322, 150, 19));
        draftValue.setFont(defaultFont);
        draftValue.setBounds(new Rectangle(170, 322, 250, 19));
        draftActivationValue.setSelected(false);
        draftActivationValue.setBounds(new Rectangle(500, 322, 50, 19));

        trashLabel.setText("Trash Folder Name: ");
        trashLabel.setFont(defaultFont);
        trashLabel.setBounds(new Rectangle(14, 342, 150, 19));
        trashValue.setFont(defaultFont);
        trashValue.setBounds(new Rectangle(170, 342, 250, 19));
        trashActivationValue.setSelected(false);
        trashActivationValue.setBounds(new Rectangle(500, 342, 50, 19));

        // --------

        typeLabel.setText("Type: ");
        typeLabel.setFont(defaultFont);
        typeLabel.setBounds(new Rectangle(14, 372, 150, 19));
        typeValue.setFont(defaultFont);
        typeValue.setBounds(new Rectangle(170, 372, 350, 19));
        typeValue.setEnabled(false);

        infoTypesLabel.setText("Supported Types: ");
        infoTypesLabel.setFont(defaultFont);
        infoTypesLabel.setBounds(new Rectangle(14, 392, 150, 19));
        infoTypesValue.setFont(defaultFont);
        infoTypesValue.setBounds(new Rectangle(170, 392, 350, 19));
        infoTypesValue.setEnabled(false);

        infoVersionsLabel.setText("Supported Versions: ");
        infoVersionsLabel.setFont(defaultFont);
        infoVersionsLabel.setBounds(new Rectangle(14, 412, 150, 19));
        infoVersionsValue.setFont(defaultFont);
        infoVersionsValue.setBounds(new Rectangle(170, 412, 350, 19));
        infoVersionsValue.setEnabled(false);

        dtEncryptLabel.setText("Encryption/Encoding: ");
        dtEncryptLabel.setFont(defaultFont);
        dtEncryptLabel.setBounds(new Rectangle(14, 432, 150, 18));
        dtEncryptValue.setSelected(false);
        dtEncryptValue.setBounds(new Rectangle(166, 432, 25, 19));

        softDeleteLabel.setText("Soft Delete: ");
        softDeleteLabel.setFont(defaultFont);
        softDeleteLabel.setBounds(new Rectangle(14, 452, 150, 18));
        softDeleteValue.setSelected(false);
        softDeleteValue.setBounds(new Rectangle(166, 452, 25, 19));

        confirmButton.setFont(defaultFont);
        confirmButton.setText("Add");
        confirmButton.setBounds(170, 492, 70, 25);

        confirmButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                try {
                    validateValues();
                    getValues();
                    if (getState() == STATE_INSERT) {
                        EmailSyncSourceConfigPanel.this.actionPerformed(
                                new ActionEvent(EmailSyncSourceConfigPanel.this,
                                                ACTION_EVENT_INSERT,
                                                event.getActionCommand()));
                    } else {
                        EmailSyncSourceConfigPanel.this.actionPerformed(
                                new ActionEvent(EmailSyncSourceConfigPanel.this,
                                                ACTION_EVENT_UPDATE,
                                                event.getActionCommand()));
                    }
                } catch (Exception e) {
                    notifyError(new AdminException(e.getMessage()));
                }
            }
        });

        // add all components to the panel
        this.add(panelName, null);
        this.add(sourceUriLabel, null);
        this.add(sourceUriValue, null);
        this.add(nameLabel, null);
        this.add(nameValue, null);

        this.add(serverTypeLabel, null);
        this.add(serverTypeValue, null);
        this.add(outHostLabel, null);
        this.add(outHostValue, null);
        this.add(outPortLabel, null);
        this.add(outPortValue, null);
        this.add(outAuthLabel, null);
        this.add(outAuthValue, null);

        this.add(inHostLabel, null);
        this.add(inHostValue, null);
        this.add(inPortLabel, null);
        this.add(inPortValue, null);

        this.add(sslCheckLabel, null);
        this.add(sslCheckValue, null);
        this.add(keystoreLabel, null);
        this.add(keystoreValue, null);
        this.add(keystorePassPhraseLabel, null);
        this.add(keystorePassPhraseValue, null);

        this.add(allmailboxCheckLabel, null);
        this.add(allmailboxCheckValue, null);
        this.add(inboxLabel, null);
        this.add(inboxValue, null);
        this.add(inboxActivationValue, null);
        this.add(outboxLabel, null);
        this.add(outboxValue, null);
        this.add(outboxActivationValue, null);
        this.add(sentLabel, null);
        this.add(sentValue, null);
        this.add(sentActivationValue, null);
        this.add(draftLabel, null);
        this.add(draftValue, null);
        this.add(draftActivationValue, null);
        this.add(trashLabel, null);
        this.add(trashValue, null);
        this.add(trashActivationValue, null);

        this.add(typeLabel, null);
        this.add(typeValue, null);
        this.add(infoTypesLabel, null);
        this.add(infoTypesValue, null);
        this.add(infoVersionsLabel, null);
        this.add(infoVersionsValue, null);
        this.add(dtEncryptLabel, null ) ;
        this.add(dtEncryptValue, null ) ;
        this.add(softDeleteLabel, null ) ;
        this.add(softDeleteValue, null ) ;

        this.add(confirmButton, null);

    }
    
    
    /**
     * Set preferredSize of the panel
     *
     * @return preferredSize of the panel
     */
    public Dimension getPreferredSize() {
        return new Dimension(525, 530);
    }

    /**
     *
     */
    public void updateForm() {

        if (!(getSyncSource() instanceof EmailSyncSource)) {
            error("This is not a EmailSyncSource! Unable to process SyncSource values.");
            return;
        }

        if (getState() == STATE_INSERT) {
            confirmButton.setText("Add");
        } else if (getState() == STATE_UPDATE) {
            confirmButton.setText("Save");
        }

        EmailSyncSource syncSource = (EmailSyncSource) getSyncSource();

        sourceUriValue.setText(syncSource.getSourceURI());

        nameValue.setText(syncSource.getName());

        serverTypeValue.setSelectedItem(syncSource.getServerType());
        
        outHostValue.setText(syncSource.getOutHost());
        outPortValue.setText(syncSource.getOutPort());
        outAuthValue.setSelected(syncSource.isSmtpAuth());

        inHostValue.setText(syncSource.getInHost());
        inPortValue.setText(syncSource.getInPort());

        sslCheckValue.setSelected(syncSource.isSSL());
        keystoreValue.setText(syncSource.getKeystorePath() == null ? "" : syncSource.getKeystorePath());
        keystorePassPhraseValue.setText(syncSource.getKeystorePassphrase() == null ? "" : syncSource.getKeystorePassphrase());
        keystoreLabel.setEnabled(sslCheckValue.isSelected());
        keystoreValue.setEnabled(sslCheckValue.isSelected());
        keystorePassPhraseLabel.setEnabled(sslCheckValue.isSelected());
        keystorePassPhraseValue.setEnabled(sslCheckValue.isSelected());

        if (syncSource instanceof ImapEmailSyncSource) {
            if (syncSource.isAllMailboxActivation()) {
                allmailboxCheckValue.setSelected(true);
                allmailboxCheckValue.setEnabled(false);
            } else {
                allmailboxCheckValue.setSelected(false);
                allmailboxCheckValue.setEnabled(false);
            }
            allmailboxCheckValue.setEnabled(true);
            allmailboxCheckValue.setEnabled(false);
        } else if (syncSource instanceof PopEmailSyncSource) {
            allmailboxCheckValue.setSelected(false);
            allmailboxCheckValue.setEnabled(false);
        }

        inboxValue.setText(syncSource.getDefaultFolder().getInboxName());
        outboxValue.setText(syncSource.getDefaultFolder().getOutboxName());
        sentValue.setText(syncSource.getDefaultFolder().getSentName());
        draftValue.setText(syncSource.getDefaultFolder().getDraftName());
        trashValue.setText(syncSource.getDefaultFolder().getTrashName());

        if (syncSource instanceof ImapEmailSyncSource) {
            inboxValue.setEnabled(false);
            outboxValue.setEnabled(true);
            sentValue.setEnabled(true);
            draftValue.setEnabled(true);
            trashValue.setEnabled(true);
        } else if (syncSource instanceof PopEmailSyncSource) {
            inboxValue.setEnabled(false);
            outboxValue.setEnabled(false);
            sentValue.setEnabled(false);
            draftValue.setEnabled(false);
            trashValue.setEnabled(false);
        }


        inboxActivationValue.setSelected(true);
        outboxActivationValue.setSelected(true);
        sentActivationValue.setSelected(true);
        if (syncSource instanceof ImapEmailSyncSource) {
            draftActivationValue.setSelected(syncSource.isDraftActivation());
            trashActivationValue.setSelected(syncSource.isTrashActivation());
        } else if (syncSource instanceof PopEmailSyncSource) {
            draftActivationValue.setSelected(false);
            trashActivationValue.setSelected(false);
        }

        inboxActivationValue.setEnabled(false);
        outboxActivationValue.setEnabled(false);
        sentActivationValue.setEnabled(false);
        if (syncSource instanceof ImapEmailSyncSource) {
            draftActivationValue.setEnabled(true);
            trashActivationValue.setEnabled(true);
        } else if (syncSource instanceof PopEmailSyncSource) {
            draftActivationValue.setEnabled(false);
            trashActivationValue.setEnabled(false);
        }

        typeValue.setText(syncSource.getType());

        SyncSourceInfo info = syncSource.getInfo();
        if (info != null) {
            ContentType[] types = info.getSupportedTypes();

            StringBuffer typesList = new StringBuffer();
            StringBuffer versionsList = new StringBuffer();

            for (int i = 0; ((types != null) && (i < types.length)); ++i) {

                if (typesList.length() > 0) {
                    typesList.append(',');
                    versionsList.append(',');
                }

                typesList.append(types[i].getType());
                versionsList.append(types[i].getVersion());
            }

            infoTypesValue.setText(typesList.toString());
            infoVersionsValue.setText(versionsList.toString());
        }

        SyncSourceManagementObject mo = (SyncSourceManagementObject)getManagementObject();
        String transformationsRequired = mo.getTransformationsRequired();
        if (transformationsRequired == null){
            dtEncryptValue.setSelected(false);
        } else {
          if (transformationsRequired.equals("des;b64")){
              dtEncryptValue.setSelected(true);
          } else {
              dtEncryptValue.setSelected(false);
          }
        }

        softDeleteValue.setSelected(syncSource.isSoftDelete());
        
        
        if (syncSource.getSourceURI() != null) {
            sourceUriValue.setEditable(false);
        }

    }
    
    
    /**
     * Set syncSource properties with the
     * values provided by the user.
     */
    public void getValues() {

        EmailSyncSource syncSource = (EmailSyncSource) getSyncSource();

        syncSource.setSourceURI(sourceUriValue.getText().trim());

        syncSource.setName(nameValue.getText().trim());

        syncSource.setServerType(serverTypeValue.getSelectedItem().toString().trim());
        
        syncSource.setOutHost(outHostValue.getText().trim());
        syncSource.setOutPort(outPortValue.getText().trim());
        syncSource.setSmtpAuth(outAuthValue.isSelected());

        syncSource.setInHost(inHostValue.getText().trim());
        syncSource.setInPort(inPortValue.getText().trim());

        syncSource.setSSL(sslCheckValue.isSelected());
        syncSource.setKeystorePath(keystoreValue.getText().trim());
        syncSource.setKeystorePassphrase(keystorePassPhraseValue.getText().trim());

        if (allmailboxCheckValue.isSelected()) {
            syncSource.setAllMailboxActivation(true);
        } else {
            syncSource.setAllMailboxActivation(false);
        }

        if (syncSource instanceof ImapEmailSyncSource) {
            syncSource.setProtocol(Def.PROTOCOL_IMAP);
        } else if (syncSource instanceof PopEmailSyncSource) {
            syncSource.setProtocol(Def.PROTOCOL_POP3);
        }

        syncSource.setInboxActivation(true);

        syncSource.setOutboxActivation(true);

        syncSource.setSentActivation(true);

        if (syncSource instanceof ImapEmailSyncSource) {
            if (draftActivationValue.isSelected()) {
                syncSource.setDraftActivation(true);
            } else {
                syncSource.setDraftActivation(false);
            }
            if (trashActivationValue.isSelected()) {
                syncSource.setTrashActivation(true);
            } else {
                syncSource.setTrashActivation(false);
            }
        } else if (syncSource instanceof PopEmailSyncSource) {
            syncSource.setDraftActivation(false);
            syncSource.setTrashActivation(false);
        }


        else if (syncSource instanceof PopEmailSyncSource) {
            syncSource.setDraftActivation(false);
            syncSource.setTrashActivation(false);
        }


        DefaultFolder df = new DefaultFolder();
        df.setInboxName(inboxValue.getText().trim());
        df.setInboxId(Def.FOLDER_INBOX_ID);
        df.setOutboxName(outboxValue.getText().trim());
        df.setOutboxId(Def.FOLDER_OUTBOX_ID);
        df.setSentName(sentValue.getText().trim());
        df.setSentId(Def.FOLDER_SENT_ID);
        df.setDraftName(draftValue.getText().trim());
        df.setDraftId(Def.FOLDER_DRAFTS_ID);
        df.setTrashName(trashValue.getText().trim());
        df.setTrashId(Def.FOLDER_TRASH_ID);
        syncSource.setDefaultFolder(df);


        boolean dtencr = dtEncryptValue.isSelected();
        String transformationsRequired = null;
        if(dtencr){
           transformationsRequired = "des;b64";
        }  else {
           transformationsRequired = null;
        }
        SyncSourceManagementObject mo = (SyncSourceManagementObject)getManagementObject();
        mo.setTransformationsRequired(transformationsRequired);

        syncSource.setSoftDelete(softDeleteValue.isSelected());

        syncSource.setType(typeValue.getText().trim());

        StringTokenizer types    = new StringTokenizer(infoTypesValue.getText()   , ",");
        StringTokenizer versions = new StringTokenizer(infoVersionsValue.getText(), ",");

        ContentType[] contentTypes = new ContentType[types.countTokens()];

        for (int i = 0; i < contentTypes.length; ++i) {
            contentTypes[i] = new ContentType(types.nextToken().trim(),
                    versions.nextToken().trim());
        }

        SyncSourceInfo info = new SyncSourceInfo(contentTypes, 0);

        info.setSupportHierarchicalSync(true);

        syncSource.setInfo(info);

    }
    

    // ----------------------------------------------------------- Private methods

    /**
     * Checks if the values provided by the user are all valid. In caso of errors,
     * a IllegalArgumentException is thrown.
     *
     * @throws IllegalArgumentException if:
     *   <ul>
     *   <li>name, uri, type or directory are empty (null or zero-length)
     *   <li>the types list length does not match the versions list length
     *   <ul>
     */
    public void validateValues() throws IllegalArgumentException {

        String value;

        value = sourceUriValue.getText();
        if (StringUtils.isEmpty(value)) {
            throw new IllegalArgumentException(
                    "Field 'Source URI' cannot be empty. " +
                            "Please provide a SyncSource URI.");
        }

        value = nameValue.getText();
        if (StringUtils.isEmpty(value)) {
            throw new IllegalArgumentException(
                    "Field 'Name' cannot be empty. " +
                            "Please provide a SyncSource name.");
        }
        if (!StringUtils.containsOnly(value, NAME_ALLOWED_CHARS.toCharArray())) {
            throw new IllegalArgumentException(
                    "Only the following characters are " +
                            "allowed for field 'Name': \n" +
                            NAME_ALLOWED_CHARS);
        }

        value = serverTypeValue.getSelectedItem().toString().trim();
        if (StringUtils.isEmpty(value)) {
            throw new IllegalArgumentException("Field 'Server Type' cannot be empty");
        }
        
        value = outHostValue.getText();
        if (StringUtils.isEmpty(value)) {
            throw new IllegalArgumentException(
                    "Field 'Outgoing Server' cannot be empty. " +
                            "Please provide a host.");
        }
        if (!StringUtils.containsOnly(value, NAME_ALLOWED_CHARS.toCharArray())) {
            throw new IllegalArgumentException(
                    "Only the following characters are " +
                            "allowed for field 'Outgoing Server': \n" +
                            NAME_ALLOWED_CHARS);
        }

        value = outPortValue.getText();
        if (StringUtils.isEmpty(value)) {
            throw new IllegalArgumentException(
                    "Field 'Outgoing Server Port' cannot be empty. " +
                            "Please provide Port.");
        }
        if (!StringUtils.containsOnly(value, PORT_ALLOWED_CHARS.toCharArray())) {
            throw new IllegalArgumentException(
                    "Only the following characters are " +
                            "allowed for field 'Mail Server Port': \n" +
                            PORT_ALLOWED_CHARS);
        }

        value = inHostValue.getText();
        if (StringUtils.isEmpty(value)) {
            throw new IllegalArgumentException(
                    "Field 'Incoming Server' cannot be empty. " +
                            "Please provide a host.");
        }
        if (!StringUtils.containsOnly(value, NAME_ALLOWED_CHARS.toCharArray())) {
            throw new IllegalArgumentException(
                    "Only the following characters are " +
                            "allowed for field 'Incoming Server': \n" +
                            NAME_ALLOWED_CHARS);
        }

        value = inPortValue.getText();
        if (StringUtils.isEmpty(value)) {
            throw new IllegalArgumentException(
                    "Field 'Incoming Server Port' cannot be empty. " +
                            "Please provide a SyncSource host.");
        }
        if (!StringUtils.containsOnly(value, PORT_ALLOWED_CHARS.toCharArray())) {
            throw new IllegalArgumentException(
                    "Only the following characters are " +
                            "allowed for field 'Mail Server Port': \n" +
                            PORT_ALLOWED_CHARS);
        }

        value = inboxValue.getText();
        if (StringUtils.isEmpty(value)) {
            throw new IllegalArgumentException(
                    "Field 'Inbox' cannot be empty. Please provide a host.");
        }

        value = outboxValue.getText();
        if (StringUtils.isEmpty(value)) {
            throw new IllegalArgumentException(
                    "Field 'Outbox' cannot be empty. Please provide a host.");
        }

        value = sentValue.getText();
        if (StringUtils.isEmpty(value)) {
            throw new IllegalArgumentException(
                    "Field 'Sent' cannot be empty. Please provide a host.");
        }

        value = draftValue.getText();
        if (StringUtils.isEmpty(value)) {
            throw new IllegalArgumentException(
                    "Field 'Draft' cannot be empty. Please provide a host.");
        }

        value = trashValue.getText();
        if (StringUtils.isEmpty(value)) {
            throw new IllegalArgumentException(
                    "Field 'Trash' cannot be empty. Please provide a host.");
        }

        value = typeValue.getText();
        if (StringUtils.isEmpty(value)) {
            throw new IllegalArgumentException(
                    "Field 'Type' cannot be empty. Please provide a SyncSource type.");
        }

        int typesCount = new StringTokenizer(infoTypesValue.getText(), ",").countTokens();
        if (typesCount == 0) {
            throw new IllegalArgumentException("Field 'Supported types' cannot be empty. Please provide one or more supported types.");
        }
        if (typesCount != new StringTokenizer(infoVersionsValue.getText(), ",").countTokens()) {
            throw new IllegalArgumentException("Number of supported types does not match number of versions");
        }

    }

    /**
     *
     *
     * @param msg String
     */
    protected void error(String msg) {
        JOptionPane.showMessageDialog(null, msg, "Error", JOptionPane.ERROR_MESSAGE);
  }
}
