/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.engine.cache;

import java.io.Serializable;
import java.security.Principal;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.logging.*;

import com.funambol.framework.core.*;
import com.funambol.framework.engine.*;
import com.funambol.framework.engine.source.*;
import com.funambol.framework.tools.beans.*;

import com.funambol.email.admin.EmailConnectorConfig;
import com.funambol.email.exception.EmailAccessException;
import com.funambol.email.exception.EmailConfigException;
import com.funambol.email.exception.EntityException;
import com.funambol.email.items.model.EmailFilter;
import com.funambol.email.transport.IMailServerWrapper;
import com.funambol.email.util.Def;
import com.funambol.email.util.Utility;
import java.util.HashMap;


/**
 * This class represents a SyncSource that read and store the state of the items
 * in a support table.
 *
 *
 */
public abstract class CrcCachedSyncSource
        extends AbstractSyncSource
        implements FilterableSyncSource, Serializable, LazyInitBean {


    // --------------------------------------------------------------- Constants

    private Logger log = Logger.getLogger(Def.LOGGER_NAME);

    public String jndiDataSourceName = null;

    // ------------------------------------------------------------ Private Data

    /**    */
    private CrcItemStateDAO crcItemStateDao = null;

    /**    */
    public Principal principal = null;

    /**    */
    public long principalId ;
    
    /**    */
    public String userName  ;    
    
    /**    */
    public int syncMode;
    
    /**    */
    public EmailFilter filter = new EmailFilter();
    
    /**
     * wrapper for the Mail Server
     */
    public IMailServerWrapper mswf = null;
    
    /**
     * list of CrcSyncItemInfo
     */
    //private CrcSyncItemInfo[] localItems  = null;
    //private CrcSyncItemInfo[] serverItems = null;
    private HashMap localItems  = null;
    private HashMap serverItems = null;

    /*
     *
     */
    private ArrayList newItemKeys     = null;
    private ArrayList deletedItemKeys = null;
    private ArrayList updatedItemKeys = null;

    /**
     * keep the items id and the result of send operation
     */
    public ArrayList sentItems    = new ArrayList();

    /**
     * save the invalid items during the cache updating procedure
     */
    private CrcSyncItemInfo[] invalidItems = null;


    // ---------------------------------------------------------- Public Methods

    /**
     * Init the syncSource
     *
     * @throws BeanInitializationException
     */
    public void init() throws BeanInitializationException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("Init CrcCachedSyncSource with sourceUri: " + sourceURI);
        }

        try {
            jndiDataSourceName  = EmailConnectorConfig.getConfigInstance().getDataSource();
        } catch (EmailConfigException e) {
            log.severe("Error Getting jndi datasource Parameters:: " + e);
            log.throwing(getClass().getName(), "checkMailCredentials", e);
        }

        crcItemStateDao = new CrcItemStateDAO(jndiDataSourceName);
        try {
            crcItemStateDao.init();
        } catch (Exception e) {
            throw new BeanInitializationException("Error initializing CrcItemStateDAO", e);
        }
    }

    /**
     * BeginSync
     *
     * @param context The sync context that includes the principal,
     * sync mode and filter clause.
     * @throws SyncSourceException
     */
    public void beginSync(SyncContext context) throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("CrcCachedSyncSource beginSync [" + sourceURI + "] ");
        }

        try {

            this.serverItems = getAllSyncItemInfo();
            
            if (syncMode == AlertCode.SLOW){
                
                crcItemStateDao.deleteLocalItems(this.sourceURI,principalId);
                crcItemStateDao.deleteSentPop(this.sourceURI,principalId);                
                this.localItems = new HashMap();
                
            } else if (syncMode == AlertCode.REFRESH_FROM_SERVER){
                
                crcItemStateDao.deleteLocalItems(this.sourceURI,principalId);
                crcItemStateDao.deleteSentPop(this.sourceURI,principalId);
                this.localItems = new HashMap();
                this.setItemKeys();                
                
            } else {                
                
                this.localItems = crcItemStateDao.getLocalItems(this.sourceURI, principalId);
                this.setItemKeys();                
                
            }


        } catch (Exception e) {
            try {
                this.mswf.releaseConnection();
            } catch (EmailAccessException e1){
                throw new SyncSourceException("Error closing store connection: " +
                        e1.getMessage(), e1);
            }            
            throw new SyncSourceException("Error handling Items In Cache: " +
                    e.getMessage(), e);
        }

        if (log.isLoggable(Level.FINEST)) {
            String tmp = "";
            if (this.newItemKeys != null){
                log.finest("Crc Cached NEW num:" + this.newItemKeys.size());
                for (int i=0; i<this.newItemKeys.size(); i++){
                    tmp = tmp + " " + (this.newItemKeys.get(i));
                }
                log.finest("NEW List : [" + tmp + "]");
            } else {
                log.finest("Crc Cached NO NEW ITEMS");
            }

            tmp = "";
            if (this.updatedItemKeys != null){
                log.finest("Crc Cached UPDATED num:" + this.updatedItemKeys.size());
                for (int i=0; i<this.updatedItemKeys.size(); i++){
                    tmp = tmp + " " + (this.updatedItemKeys.get(i));
                }
                log.finest("UPDATED List : [" + tmp + "]");
            } else {
                log.finest("Crc Cached NO UPDATED ITEMS");
            }

            tmp = "";
            if (this.deletedItemKeys != null){
                log.finest("Crc Cached DELETED num:" + this.deletedItemKeys.size());
                for (int i=0; i<this.deletedItemKeys.size(); i++){
                    tmp = tmp + " " + (this.deletedItemKeys.get(i));
                }
                log.finest("DELETED List : [" + tmp + "]");
            } else {
                log.finest("Crc Cached NO DELETED ITEMS");
            }
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("CrcCachedSyncSource beginSync done [" + sourceURI + "]");
        }

    }

    /**
     * End Sync Method
     *
     * @throws SyncSourceException
     */
    public void endSync() throws SyncSourceException {


        if (log.isLoggable(Level.FINEST)) {
            log.finest("CrcCachedSyncSource endSync [" + sourceURI + "]"
                    + " principal " + this.principal );
        }

        // refresh the cache
        try {
            
            // we add and remove items from the serverItems hashmap 
            // during the syncsession
            HashMap endServerItems = getAllSyncItemInfo();
            
            invalidItems = crcItemStateDao.getInvalidItems(this.sourceURI, 
                                                           this.principalId);
            
            crcItemStateDao.updateLocalItems(endServerItems, 
                                             this.sentItems,
                                             this.invalidItems, 
                                             this.sourceURI, 
                                             this.principalId);

        } catch (Exception e) {
            throw new SyncSourceException("Error caching items:" +
                    e.getMessage(), e);
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("CrcCachedSyncSource endSync done [" + sourceURI + "]");
        }

    }


    /**
     *
     * @see SyncSource
     *
     * @param since Timestamp
     * @param to Timestamp
     * @return SyncItemKey[]
     * @throws SyncSourceException
     */
    public SyncItemKey[] getUpdatedSyncItemKeys(Timestamp since, Timestamp to)
    throws SyncSourceException {
        return (SyncItemKey[])updatedItemKeys.toArray(new SyncItemKey[0]) ;
    }

    /**
     *
     * @see SyncSource
     *
     * @param since Timestamp
     * @param to Timestamp
     * @return SyncItemKey[]
     * @throws SyncSourceException
     */
    public SyncItemKey[] getNewSyncItemKeys(Timestamp since, Timestamp to)
    throws SyncSourceException {
        return (SyncItemKey[])newItemKeys.toArray(new SyncItemKey[0]);
    }

    /**
     *
     * @see SyncSource
     *
     * @param since Timestamp
     * @param to Timestamp
     * @return SyncItemKey[]
     * @throws SyncSourceException
     */
    public SyncItemKey[] getDeletedSyncItemKeys(Timestamp since, Timestamp to)
    throws SyncSourceException {
        return (SyncItemKey[])deletedItemKeys.toArray(new SyncItemKey[0]) ;
    }

    
    /**
     *
     * @return CrcSyncItemInfo[]
     */
    public HashMap getServerItems() {
        return this.serverItems ;
    }

    /**
     *
     * @return map with all local CrcSyncItemInfo
     */
    public HashMap getLocalItems() {
        return this.localItems ;
    }

    /**
     *
     * @return map with all Mail Server CrcSyncItemInfo
     * @throws SyncSourceException
     */
    public abstract HashMap getAllSyncItemInfo() throws  SyncSourceException;


    /**
     *
     * @throws SyncSourceException
     */
    public abstract void clearServerInfo() throws  SyncSourceException;


    /**
     *
     *
     * @param syncItemKey SyncItemKey
     * @return char
     * @throws SyncSourceException
     */
    public char getSyncItemStateFromId(SyncItemKey syncItemKey)
    throws SyncSourceException {

        char status = SyncItemState.UNKNOWN;

        String key = syncItemKey.getKeyAsString();
        
        status = CrcCachedSyncSourceHelper.getSyncItemStateFromId(key,
                this.syncMode,
                this.newItemKeys,
                this.deletedItemKeys,
                this.updatedItemKeys);

        return status;

    }

    // --------------------------------------------------------- Private Methods
    
    /**
     *
     *
     * @param syncItemKey SyncItemKey
     * @return char
     * @throws SyncSourceException
     */
    private void setItemKeys() throws EntityException {

        if (this.filter.getFilterType().equalsIgnoreCase(Def.FILTER_TYPE_INC)){
            // inclusive
            if (this.filter.getId() != null){
                // id filter
                String idfilter = this.filter.getId();
                this.newItemKeys     = new ArrayList();
                this.updatedItemKeys = CrcCachedSyncSourceHelper.setUpdatedIDFilter(
                                       idfilter, this.serverItems);
                this.deletedItemKeys = new ArrayList();
            } else {
                if (this.filter.getTime() != null) {
                    // time filter
                    java.util.Date timefilter = this.filter.getTime();
                    int comparison = Utility.getComparisonTerm(this.filter.getTimeClause());
                    this.newItemKeys     = new ArrayList();
                    this.updatedItemKeys = CrcCachedSyncSourceHelper.setUpdatedTimeFilter(
                                           this.filter.getTime(), comparison,
                                           this.serverItems);
                    this.deletedItemKeys = new ArrayList();                    
                } else {
                    // no filter
                    this.newItemKeys     = CrcCachedSyncSourceHelper.setNewItemKeys(
                                           this.localItems, this.serverItems);
                    this.updatedItemKeys = CrcCachedSyncSourceHelper.setUpdatedItemKeys(
                                           this.localItems, this.serverItems);
                    this.deletedItemKeys = CrcCachedSyncSourceHelper.setDeletedItemKeys(
                                           this.localItems, this.serverItems);                    
                }
            }  
        } else {
            // exclusive
            if (this.filter.getId() != null){
                // id filter
                String idfilter = this.filter.getId();
                this.newItemKeys     = CrcCachedSyncSourceHelper.setNewItemKeysWithIdFilter(
                                       idfilter, this.localItems, this.serverItems);
                this.updatedItemKeys = CrcCachedSyncSourceHelper.setUpdatedItemKeysWithIdFilter(
                                       idfilter, this.localItems, this.serverItems);
                this.deletedItemKeys = CrcCachedSyncSourceHelper.setDeletedItemKeysWithIdFilter(
                                       idfilter, this.localItems, this.serverItems);                    
            } else {
                if (this.filter.getTime() != null) {
                    // time filter
                    java.util.Date timefilter = this.filter.getTime();
                    int comparison = Utility.getComparisonTerm(this.filter.getTimeClause());
                    this.newItemKeys     = CrcCachedSyncSourceHelper.setNewItemKeysWithTimeFilter(
                                           timefilter, comparison, this.localItems, this.serverItems);
                    this.updatedItemKeys = CrcCachedSyncSourceHelper.setUpdatedItemKeysWithTimeFilter(
                                           timefilter, comparison, this.localItems, this.serverItems);
                    this.deletedItemKeys = CrcCachedSyncSourceHelper.setDeletedItemKeysWithTimeFilter(
                                           timefilter, comparison, this.localItems, this.serverItems);                    
                } else {
                    // no filter
                    this.newItemKeys     = CrcCachedSyncSourceHelper.setNewItemKeys(
                                           this.localItems, this.serverItems);
                    this.updatedItemKeys = CrcCachedSyncSourceHelper.setUpdatedItemKeys(
                                           this.localItems, this.serverItems);
                    this.deletedItemKeys = CrcCachedSyncSourceHelper.setDeletedItemKeys(
                                           this.localItems, this.serverItems);                    
                }
            }              
        }
    }
    
}