/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.engine.cache;

import com.funambol.email.exception.EntityException;
import java.util.ArrayList;
import java.util.logging.*;

import com.funambol.framework.core.*;
import com.funambol.framework.engine.*;
import com.funambol.framework.engine.source.*;
import com.funambol.email.util.Def;
import com.funambol.email.util.Utility;
import java.util.HashMap;
import java.util.Iterator;
import javax.mail.search.ComparisonTerm;


/**
 * This class includes the common methods for the OpenSource CrcCachedSyncSource
 * and the Enterprise CrcCachedSyncSource.
 *
 * @version $Id: CrcCachedSyncSourceHelper.java,v 1.9 2006/11/03 17:28:39 gbmiglia Exp $
 *
 */
public class CrcCachedSyncSourceHelper {

    // --------------------------------------------------------------- Constants

    private Logger log = Logger.getLogger(Def.LOGGER_NAME);

    // ---------------------------------------------------------- Public Methods

    /**
     *
     *
     * @param syncItemKey SyncItemKey
     * @return char
     * @throws SyncSourceException
     */
    public static char getSyncItemStateFromId(String key,
                                              int syncMode,
                                              ArrayList newItemKeys,
                                              ArrayList deletedItemKeys,
                                              ArrayList updatedItemKeys)
    throws SyncSourceException {

        char status       = SyncItemState.UNKNOWN;
        boolean isNew     = false;
        boolean isUpdated = false;
        boolean isDeleted = false;

        //String id = syncItemKey.getKeyAsString();

        //
        if ((syncMode == AlertCode.SLOW) ||
            (syncMode == AlertCode.REFRESH_FROM_SERVER)) {
           return SyncItemState.NEW;
        }

        // search in new items list
        SyncItemKey tmp = null;
        for (int i=0; i<newItemKeys.size(); i++){

            tmp = (SyncItemKey)newItemKeys.get(i);

            if (tmp.getKeyAsString().equals(key)) {
                isNew  = true;
                status = SyncItemState.NEW;
                break;
            }

        }

        if (!isNew){

           // search in updated items list
           for (int i=0; i<updatedItemKeys.size(); i++){

              tmp = (SyncItemKey)updatedItemKeys.get(i);

              if (tmp.getKeyAsString().equals(key)) {
                 isUpdated = true;
                 status    = SyncItemState.UPDATED;
                 break;
              }
           }


           if (!isUpdated){
              // search in deleted items list
              for (int i=0; i<deletedItemKeys.size(); i++){

                 tmp = (SyncItemKey)deletedItemKeys.get(i);

                 if (tmp.getKeyAsString().equals(key)) {
                    isDeleted = true;
                    status    = SyncItemState.DELETED;
                    break;
                 }
              }

              if (!isDeleted){
                 status    = SyncItemState.SYNCHRONIZED;
              }

           }

        }

        return status;

    }

    /**
     * Creates new items id array
     *
     * @param map with all mail server CrcSyncItemInfo
     * @param map with all local CrcSyncItemInfo
     * @return new items id array
     */
    public static ArrayList setNewItemKeys(HashMap localItems,
                                           HashMap serverItems)
     throws EntityException {

        ArrayList       newRows = new ArrayList(); 
        CrcSyncItemInfo scrcii  = null;
        CrcSyncItemInfo lcrcii  = null;
        String          sGUID   = null;
        
        try {
            Iterator values = serverItems.values().iterator();

            while (values.hasNext()) {
               scrcii = (CrcSyncItemInfo)values.next();
               sGUID = scrcii.getKey().getKeyAsString();
               lcrcii = (CrcSyncItemInfo)localItems.get(sGUID);
               if (lcrcii == null){
                  newRows.add(scrcii.getKey());
               }
            }   
        } catch (Exception e){
            throw new EntityException ("Error setting New Item List " + e.getMessage());
        }
        
        return newRows;
        
    }
    
    /**
     * Creates new items id array
     *
     * @param date Filter
     * @param comaprison Ge orr LT
     * @param map with all mail server CrcSyncItemInfo
     * @param map with all local CrcSyncItemInfo
     * @return new items id array
     */
    public static ArrayList setNewItemKeysWithTimeFilter(java.util.Date timefilter,
                                                         int comparison, 
                                                         HashMap localItems,
                                                         HashMap serverItems) 
     throws EntityException {
                                                        
        ArrayList       newRows = new ArrayList(); 
        CrcSyncItemInfo scrcii  = null;
        CrcSyncItemInfo lcrcii  = null;
        String          sGUID   = null;
        java.util.Date  demail  = null;

        try {
            Iterator values = serverItems.values().iterator();

            while (values.hasNext()) {
               scrcii = (CrcSyncItemInfo)values.next();
               sGUID = scrcii.getKey().getKeyAsString();
               lcrcii = (CrcSyncItemInfo)localItems.get(sGUID);
               if (lcrcii == null){
                    if (comparison == ComparisonTerm.GE){
                        demail = scrcii.getHeaderDate();
                        if (demail != null){
                            if (Utility.d1Afterd2(demail, timefilter)){
                                newRows.add(scrcii.getKey());
                            }                        
                        }
                    } else {
                        demail = scrcii.getHeaderDate();
                        if (demail != null){
                            if (Utility.d1Befored2(demail, timefilter)){
                                newRows.add(scrcii.getKey());
                            }                        
                        }
                    }                        
               }
            }   
        } catch (Exception e){
            throw new EntityException ("Error setting NewItem List " + e.getMessage());
        }
        
        
        return newRows;

    }
    
    /**
     *
     * Creates new items id array
     *
     * @param idFilter String
     * @param map with all mail server CrcSyncItemInfo
     * @param map with all local CrcSyncItemInfo
     * @return new items id array
     */
    public static ArrayList setNewItemKeysWithIdFilter(String idfilter,
                                                       HashMap localItems,
                                                       HashMap serverItems) 
     throws EntityException {                                                       
        
        ArrayList       newRows = new ArrayList(); 
        CrcSyncItemInfo scrcii  = null;
        CrcSyncItemInfo lcrcii  = null;
        String          sGUID   = null;

        try {
            Iterator values = serverItems.values().iterator();

            while (values.hasNext()) {
               scrcii = (CrcSyncItemInfo)values.next();
               sGUID = scrcii.getKey().getKeyAsString();
               lcrcii = (CrcSyncItemInfo)localItems.get(sGUID);
               if (lcrcii == null){
                   if (idfilter.equals(sGUID)){
                       newRows.add(scrcii.getKey());
                   }   
               }
            }
        } catch (Exception e){
            throw new EntityException ("Error setting New Item List " + e.getMessage());
        }
        
        
        return newRows;
                
    }

    /**
     * Creates deleted items id array
     *
     * @param localItems map with all local CrcSyncItemInfo
     * @param serverItems map with all Mail Server CrcSyncItemInfo
     * @return deleted items id array
     */
    public static ArrayList setDeletedItemKeys(HashMap localItems,
                                               HashMap serverItems)      
      throws EntityException {

        ArrayList       deleteRows = new ArrayList();
        CrcSyncItemInfo scrcii     = null;
        CrcSyncItemInfo lcrcii     = null;
        String          lGUID      = null;

        try {
            Iterator values = localItems.values().iterator();

            while (values.hasNext()) {
               lcrcii = (CrcSyncItemInfo)values.next();
               lGUID = lcrcii.getKey().getKeyAsString();
               scrcii = (CrcSyncItemInfo)serverItems.get(lGUID);
               if (scrcii == null){
                   deleteRows.add(lcrcii.getKey());
               }
            }   
        } catch (Exception e){
            throw new EntityException ("Error setting Deleted Item List " + e.getMessage());
        }
        
                
        return deleteRows;
        
    }

    /**
     * Creates deleted items id array
     *
     * @param time filter String
     * @param comparison GE or LT
     * @param localItems map with all local CrcSyncItemInfo
     * @param serverItems map with all Mail Server CrcSyncItemInfo
     * @return deleted items id array
     */
    public static ArrayList setDeletedItemKeysWithTimeFilter(java.util.Date timefilter,
                                                             int comparison,
                                                             HashMap localItems,
                                                             HashMap serverItems) 
     throws EntityException {

        ArrayList       deleteRows = new ArrayList();
        CrcSyncItemInfo scrcii     = null;
        CrcSyncItemInfo lcrcii     = null;
        String          lGUID      = null;
        java.util.Date  demail     = null;

        try {
            Iterator values = localItems.values().iterator();

            while (values.hasNext()) {
               lcrcii = (CrcSyncItemInfo)values.next();
               lGUID = lcrcii.getKey().getKeyAsString();
               scrcii = (CrcSyncItemInfo)serverItems.get(lGUID);
               if (scrcii == null){
                    if (comparison == ComparisonTerm.GE){
                        demail = lcrcii.getHeaderDate();
                        if (demail != null){
                            if (Utility.d1Afterd2(demail, timefilter)){
                                deleteRows.add(lcrcii.getKey());
                            }                                                                            
                        }
                    } else {
                        demail = lcrcii.getHeaderDate();
                        if (demail != null){
                            if (Utility.d1Befored2(demail, timefilter)){
                                deleteRows.add(lcrcii.getKey());
                            }                                                                            
                        }
                    }                
               }
            }
        } catch (Exception e){
            throw new EntityException ("Error setting Deleted Item List " + e.getMessage());
        }
        
                
        return deleteRows;
                
    }
    
    /**
     * Creates deleted items id array
     *
     * @param id filter String
     * @param localItems map with all local CrcSyncItemInfo
     * @param serverItems map with all Mail Server CrcSyncItemInfo
     * @return deleted items id array
     */
    public static ArrayList setDeletedItemKeysWithIdFilter(String idfilter, 
                                                           HashMap localItems,
                                                           HashMap serverItems) 
      throws EntityException {

        ArrayList       deleteRows = new ArrayList();
        CrcSyncItemInfo scrcii     = null;
        CrcSyncItemInfo lcrcii     = null;
        String          lGUID      = null;

        try {
            Iterator values = localItems.values().iterator();

            while (values.hasNext()) {
               lcrcii = (CrcSyncItemInfo)values.next();
               lGUID = lcrcii.getKey().getKeyAsString();
               scrcii = (CrcSyncItemInfo)serverItems.get(lGUID);
               if (scrcii == null){
                    if (idfilter.equals(lGUID)){
                       deleteRows.add(lcrcii.getKey());
                    }
               }
            }
        } catch (Exception e){
            throw new EntityException ("Error setting Deleted Item List " + e.getMessage());
        }
        
                
        return deleteRows;
        
    }    
    
    /**
     * Creates updated items id array
     *
     * @param localItems map with all local CrcSyncItemInfo
     * @param serverItems map with all Mail Server CrcSyncItemInfo
     * @return updated items id array
     */
    public static ArrayList setUpdatedItemKeys(HashMap localItems,
                                               HashMap serverItems)      
     throws EntityException {

        ArrayList updateRows         = new ArrayList();
        long serverItemLastCrc ;
        long localItemLastCrc  ;        
        CrcSyncItemInfo scrcii = null;
        CrcSyncItemInfo lcrcii = null;
        String sGUID = null;

        try {
            Iterator values = serverItems.values().iterator();
            while (values.hasNext()) {
               scrcii = (CrcSyncItemInfo)values.next();
               sGUID = scrcii.getKey().getKeyAsString();
               lcrcii = (CrcSyncItemInfo)localItems.get(sGUID);
               if (lcrcii != null){
                  serverItemLastCrc = scrcii.getLastCrc () ;
                  localItemLastCrc  = lcrcii.getLastCrc () ;
                  if (serverItemLastCrc != localItemLastCrc) {
                      updateRows.add(scrcii.getKey());
                  }
               }
            }   
        } catch (Exception e){
            throw new EntityException ("Error setting Updated Item List " + e.getMessage());
        }
        
        return updateRows ;
    }

    /**
     * Creates updated items id array
     *
     * @param time filter Date
     * @param comparison GE or LT
     * @param map with all mail server CrcSyncItemInfo
     * @param map with all local CrcSyncItemInfo
     * @return updated items id array
     */
    public static ArrayList setUpdatedItemKeysWithTimeFilter(java.util.Date timefilter,
                                                             int comparison,
                                                             HashMap localItems,
                                                             HashMap serverItems) 
      throws EntityException {
        
        ArrayList       updateRows = new ArrayList();
        CrcSyncItemInfo scrcii     = null;
        CrcSyncItemInfo lcrcii     = null;
        String          sGUID      = null;
        long serverItemLastCrc ;
        long localItemLastCrc  ;
        java.util.Date demail = null;

        try {
            Iterator values = serverItems.values().iterator();

            while (values.hasNext()) {
               scrcii = (CrcSyncItemInfo)values.next();
               sGUID  = scrcii.getKey().getKeyAsString();
               lcrcii = (CrcSyncItemInfo)localItems.get(sGUID);
               if (lcrcii != null){
                  serverItemLastCrc = scrcii.getLastCrc () ;
                  localItemLastCrc  = lcrcii.getLastCrc () ;
                  if (serverItemLastCrc != localItemLastCrc) {
                     if (comparison == ComparisonTerm.GE){
                        demail = scrcii.getHeaderDate();
                        if (demail != null){
                            if (Utility.d1Afterd2(demail, timefilter)){
                                updateRows.add(scrcii.getKey());
                            }                                                    
                        }
                     } else {
                        demail = scrcii.getHeaderDate();
                        if (demail != null){
                            if (Utility.d1Befored2(demail, timefilter)){
                                 updateRows.add(scrcii.getKey());
                            }                                                    
                        }
                     }                        
                  }
               }
            } 
        } catch (Exception e){
            throw new EntityException ("Error setting Updated Item List " + e.getMessage());
        }
        
        
        return updateRows;

    }
    
    
    
    /**
     * Creates updated items id array
     *
     * @param idFilter String
     * @param map with all mail server CrcSyncItemInfo
     * @param map with all local CrcSyncItemInfo
     * @return updated items id array
     */
    public static ArrayList setUpdatedItemKeysWithIdFilter(String idfilter,
                                                           HashMap localItems,
                                                           HashMap serverItems) 
      throws EntityException {
        
        ArrayList       updateRows = new ArrayList();
        CrcSyncItemInfo scrcii     = null;
        CrcSyncItemInfo lcrcii     = null;
        String          sGUID      = null;
        long serverItemLastCrc ;
        long localItemLastCrc  ;        

        try {
            Iterator values = serverItems.values().iterator();

            while (values.hasNext()) {
               scrcii = (CrcSyncItemInfo)values.next();
               sGUID  = scrcii.getKey().getKeyAsString();
               lcrcii = (CrcSyncItemInfo)localItems.get(sGUID);
               if (lcrcii != null){
                  serverItemLastCrc = scrcii.getLastCrc () ;
                  localItemLastCrc  = lcrcii.getLastCrc () ;
                  if (serverItemLastCrc != localItemLastCrc) {
                      if (idfilter.equals(sGUID)){
                         updateRows.add(scrcii.getKey());
                      }
                  }
               }
            }
        } catch (Exception e){
            throw new EntityException ("Error setting Updated Item List " + e.getMessage());
        }
        
        
        return updateRows;
        
    }
    
    /**
     *
     * in the updated list there are just the emails related to the time filter
     *
     * @param timefilter Date
     * @param comparison GE or LT
     * @param serverItemsmap with  all mail server CrcSyncItemInfo
     */
    public static ArrayList setUpdatedTimeFilter(java.util.Date timefilter, 
                                                 int comparison,
                                                 HashMap serverItems) 
     throws EntityException {
       
        ArrayList updateRows   = new ArrayList();
        CrcSyncItemInfo scrcii = null;
        java.util.Date demail  = null;
        
        try {
            
            Iterator values = serverItems.values().iterator();
            
            if (comparison == ComparisonTerm.GE){
                // get all email where time "great/equals"
                while (values.hasNext()) {
                   scrcii = (CrcSyncItemInfo)values.next();
                   demail = scrcii.getHeaderDate();
                   if (demail != null){
                       if (Utility.d1Afterd2(demail, timefilter)){
                           updateRows.add(scrcii.getKey());
                       }                                                                        
                   }
                }   
            } else {
                // get all emails where time "less than"
                while (values.hasNext()) {
                   scrcii = (CrcSyncItemInfo)values.next();
                   demail = scrcii.getHeaderDate();
                   if (demail != null){
                       if (Utility.d1Befored2(demail, timefilter)){
                           updateRows.add(scrcii.getKey());
                       }                                                                        
                   }
                }   
            }
        } catch (Exception e){
            throw new EntityException ("Error setting Updated Item List " + e.getMessage());
        }
        
        return updateRows;
        
    }

   

    
    /**
     *
     * in the updated list there is just the id in the filter
     *
     * @param updateRows ArrayList
     * @param id filter String
     * @param serverItems map with all the CrcSyncItemInfo
     */
    public static ArrayList setUpdatedIDFilter(String idfilter, 
                                               HashMap serverItems)      
      throws EntityException {
        
        ArrayList       updateRows = new ArrayList();
        CrcSyncItemInfo scrcii     = null ;
        String          key        = null;

        try {
            scrcii = (CrcSyncItemInfo)serverItems.get(idfilter);
            if (scrcii != null){
                updateRows.add(scrcii.getKey());
            }            
        } catch (Exception e){
            throw new EntityException ("Error setting Updated Item List " + e.getMessage());
        }
        
        return updateRows;
        
    }
    
    //---------------------------------------------------------- Private Methods

    /**
     * Checks if item is included in array of items
     *
     * @param items items array
     * @param item item to find
     * @return <p>true</p> if the item is included
     *         <p>false</p> if the item is not included
     **/
    private static boolean isItemInArray(CrcSyncItemInfo[] items,
                                  CrcSyncItemInfo   item) {

        return isIdInArray(items, item.getKey().getKeyAsString());

    }    
    
    /**
     * Checks if item id is included in ids from array of items
     *
     * @param items items array
     * @param id id to find
     * @return <p>true</p> if the item is included
     *         <p>false</p> if the item is not included
     **/
    private static boolean isIdInArray(CrcSyncItemInfo[] items,
                                String id) {
        boolean isFind = false;
        int l = items.length;
        for (int i = 0; i < l; i++) {
            if (items[i].getKey().getKeyAsString().equals(id)) {
                isFind = true;
                break;
            }
        }
        return isFind;
    }


    /**
     * Gets item by id
     *
     *
     * @param items CrcSyncItemInfo[]
     * @param id String
     * @return <p>item</p> if found
     *         <p>null</p> if not found
     */
    private static CrcSyncItemInfo getItemById(CrcSyncItemInfo[] items,
                                               String id) {
        CrcSyncItemInfo item = null;

        for (int i = 0, l = items.length; i < l; i++) {
            if (items[i].getKey().getKeyAsString().equals(id)) {
                item = items[i];
                break;
            }
        }

        return item;
    }

    
    
}

