/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.engine.cache;

import com.funambol.email.exception.EntityException;
import com.funambol.email.items.model.SendResult;
import java.util.logging.Logger;
import java.util.logging.Level;
import java.sql.PreparedStatement;
import java.sql.Connection;
import java.sql.ResultSet;
import java.util.ArrayList;
import javax.sql.DataSource;

import com.funambol.framework.tools.DBTools;

import com.funambol.email.util.Def;
import com.funambol.email.util.Query;
import com.funambol.email.util.Utility;
import com.funambol.email.util.UtilityDB;
import java.util.HashMap;
import java.util.Iterator;

/**
 * Contains methods to access to the table with the state of the items.
 *
 * @version $Id: CrcItemStateDAO.java,v 1.12 2006/11/15 17:41:59 gbmiglia Exp $
 *
 */
public class CrcItemStateDAO {

    // --------------------------------------------------------------- Constants


    private Logger log = Logger.getLogger(Def.LOGGER_NAME);

    // ------------------------------------------------------------ Private data

    /**         */
    private String jndiDataSourceName;

    /**         */
    private DataSource dataSource;


    // ------------------------------------------------------------- Constructor

    /**
     *
     */
    private CrcItemStateDAO() {
    }

    /**
     *
     *
     * @param jndiDataSourceName String
     */
    public CrcItemStateDAO(String jndiDataSourceName) {
        this.jndiDataSourceName = jndiDataSourceName;
    }


    // ---------------------------------------------------------- Public Methods

    /**
     *
     * @throws Exception
     */
    public void init() throws Exception {
        dataSource = UtilityDB.lookupDataSource(this.jndiDataSourceName);
    }

    /**
     *
     *
     * @return String
     */
    public String getJndiDataSourceName() {
        return jndiDataSourceName;
    }

    /**
     *
     * @param jndiDataSourceName String
     */
    public void setJndiDataSourceName(String jndiDataSourceName) {
        this.jndiDataSourceName = jndiDataSourceName;
    }



    /**
     * Delete local items
     *
     * @param sourceURI
     * @param principalId
     *
     * @throws Exception
     **/
    public void deleteSentPop(String sourceURI, long principalId)
     throws EntityException {

        Connection         connection  = null ;
        PreparedStatement  ps          = null ;

        if (log.isLoggable(Level.FINEST)) {
            log.finest("CrcItemStateDAO delete fnbl_email_sentpop "
                       + principalId+ " - " +sourceURI);
        }
        try {
            connection = dataSource.getConnection();
            ps = connection.prepareStatement(Query.SENT_DELETE_POP);
            ps.setString  (1, sourceURI ) ;
            ps.setLong    (2, principalId ) ;
            ps.executeUpdate();
        } catch (Exception e) {
            throw new EntityException("Error deleting fnbl_email_sentpop", e);
        } finally {
            DBTools.close(connection, ps, null);
        }
    }

    /**
     * Delete local items
     *
     *
     * @param sourceURI String
     * @param principal String
     * @throws Exception
     */
    public void deleteLocalItems(String sourceURI, long principal)
     throws EntityException {

        Connection         connection  = null ;
        PreparedStatement  ps          = null ;

        if (log.isLoggable(Level.FINEST)) {
            log.finest("CrcItemStateDAO delete start "+ principal+ " - " +sourceURI);
        }
        try {
            connection = dataSource.getConnection();
            ps = connection.prepareStatement(Query.CACHE_DELETE_LOCAL_ITEMS);
            ps.setString  (1, sourceURI ) ;
            ps.setLong    (2, principal ) ;
            ps.executeUpdate();
        } catch (Exception e) {
            throw new EntityException("Error deleting local items", e);
        } finally {
            DBTools.close(connection, ps, null);
        }
    }

    /**
     * Update local items
     *
     * @param items map with all the mail server CrcSyncItemInfo
     * @param itemsSent ArrayList
     * @param invalidItems CrcSyncItemInfo[]
     * @param sourceURI String
     * @param principal String
     * @throws Exception
     */
    public void updateLocalItems(HashMap items,
                                 ArrayList itemsSent,
                                 CrcSyncItemInfo[] invalidItems,
                                 String sourceURI,
                                 long principal)
    throws EntityException {

         if (log.isLoggable(Level.FINEST)) {
            log.finest("CrcItemStateDAO update start");
         }         
         
         this.deleteLocalItems (sourceURI, principal);
         
         this.addLocalItems (items, itemsSent, invalidItems, sourceURI, principal);
         
         if (log.isLoggable(Level.FINEST)) {
            log.finest("CrcItemStateDAO update end");
         }
    }

    /**
     * Gets local items
     *
     *
     * @param sourceURI String
     * @param principalId String
     * @return map with CrcSyncItemInfo
     * @throws Exception
     */
    public HashMap getLocalItems (String sourceURI, long principalId)
        throws EntityException {

         Connection connection   = null;
         PreparedStatement ps    = null;
         ResultSet rs            = null;
         HashMap items           = null;
         String GUID             = null;
         long   lastCRC          = 0;
         String isInvalid        = null;
         String isInternal       = null;
         String isEmail          = null;
         String messageID        = null;
         java.util.Date date     = null;

         try {

             connection = dataSource.getConnection();

             items = new HashMap();

             ps = connection.prepareStatement(Query.CACHE_SELECT_ITEMS_BY_PRINCIPAL_BY_SOURCE);

             ps.setString(1, sourceURI);
             ps.setLong(2, principalId);

             rs = ps.executeQuery();

             while(rs.next()) {
                 
                 // ... select id, last_crc, invalid, internal, messageid, headerdate 
                 GUID      = rs.getString(1);
                 lastCRC   = rs.getLong(2);
                 isInvalid   = rs.getString(3);
                 isInternal  = rs.getString(4);
                 messageID = rs.getString(5);
                 date      = Utility.UtcToDate(rs.getString(6));
                 isEmail   = rs.getString(7);
                 
                 items.put(GUID, new CrcSyncItemInfo(GUID, lastCRC, 
                                                     messageID, date, null,
                                                     isInvalid, isInternal,  
                                                     isEmail, null));
             }

         } catch (Exception e) {
             throw new EntityException("Error getting local items", e);
         } finally {
             DBTools.close(connection, ps, rs);
         }

         return items;
    }
    
    
    /**
     * Gets local items
     *
     *
     * @param sourceURI String
     * @param principalId String
     * @return CrcSyncItemInfo[]
     * @throws Exception
     */
    public CrcSyncItemInfo[] getInvalidItems (String sourceURI,
                                              long principalId)
        throws EntityException {

         Connection connection   = null;
         PreparedStatement ps    = null;
         ResultSet rs            = null;
         ArrayList rows          = null;
         CrcSyncItemInfo[] items = null;

         try {

             connection = dataSource.getConnection();

             rows = new ArrayList();

             ps = connection.prepareStatement(Query.CACHE_SELECT_INVALID_ITEMS);

             ps.setString(1, sourceURI);
             ps.setLong(2, principalId);

             rs = ps.executeQuery();

             // "select id, last_crc, invalid, internal, messageid, headerdate, isemail " +

             while(rs.next()) {
                 rows.add(new CrcSyncItemInfo(rs.getString(1), 
                                              rs.getLong(2), 
                                              rs.getString(5),
                                              Utility.UtcToDate(rs.getString(6)),
                                              null,
                                              rs.getString(3),
                                              rs.getString(4),
                                              rs.getString(7), 
                                              null));
             }

             items = (CrcSyncItemInfo[]) rows.toArray(new CrcSyncItemInfo[0]);

         } catch (Exception e) {
             throw new EntityException("Error getting local items", e);
         } finally {
             DBTools.close(connection, ps, rs);
         }

         return items;
    }
    

    // --------------------------------------------------------- Private Methods

    /**
     * Lookups the datasource
     *
     * JBOSS
     * private String jndiDataSourceName = "java:jdbc/sync4j";
     *
     * TOMCAT
     * private String jndiDataSourceName = "java:comp/env/jdbc/sync4j";
     *
     * @throws Exception if a error occurs during lookup
     */
    /*
    private void lookupDataSource() throws Exception {

        InitialContext ctx = new InitialContext();
        if (log.isLoggable(Level.FINEST)) {
              log.finest("CachedSyncSource: datasource lookup");
        }

        try {
          // try tomcat  i.e. java:comp/env/jdbc/sync4j
          dataSource = (DataSource) ctx.lookup("java:comp/env/"+jndiDataSourceName) ;
          if (log.isLoggable(Level.FINEST)) {
             log.finest("CachedSyncSource: datasource lookup done using java:comp/env/ prefix");
          }
        }
        catch (NamingException ne) {
          if (log.isLoggable(Level.FINEST)) {
              log.finest("CachedSyncSource: No tomcat datasource lookup!");
          }
          try {
              // try jboss i.e. java:jdbc/sync4j
              dataSource = (DataSource) ctx.lookup("java:"+jndiDataSourceName) ;
              if (log.isLoggable(Level.FINEST)) {
                 log.finest("CachedSyncSource: datasource lookup done using java: prefix");
              }
          }
          catch (Exception ex){
            if (log.isLoggable(Level.FINEST)) {
                log.finest("CachedSyncSource: No jboss datasource lookup!");
            }
            try {
                // try othser j2ee server jdbc/sync4j
                dataSource = (DataSource) ctx.lookup(""+jndiDataSourceName) ;
                if (log.isLoggable(Level.FINEST)) {
                   log.finest("CachedSyncSource: datasource lookup done using - prefix");
                }
            }
            catch (Exception e){
              if (log.isLoggable(Level.FINEST)) {
                log.finest("CachedSyncSource: No j2ee server datasource lookup!");
              }
              throw new Exception("Data source "+ jndiDataSourceName
                                  + " not found " + e.toString());
            }
          }
        }
    }
    */
    
    /**
     * Adds local items
     *
     *
     * @param items map with all the mail server CrcSyncItemInfo
     * @param itemsSent ArrayList
     * @param invalidItems CrcSyncItemInfo[]
     * @param sourceURI String
     * @param principal String
     * @throws Exception
     */
    private void addLocalItems(HashMap items,
                               ArrayList itemsSent,
                               CrcSyncItemInfo[] invalidItems,
                               String sourceURI,
                               long principal) 
     throws EntityException {

         SendResult         sr            ;
         Connection         connection    = null;
         PreparedStatement  ps            = null;
         String             id            = null;
         String             GUID          = null;
         String             LUID          = null;
         String             isInvalid     = null;
         String             isInternal    = null;
         String             isEmail       = null;
         String             messageId     = null;
         java.util.Date     headerDate    = null;
         long               lastCrc       ;
         boolean            isSent        ;
         boolean            isPOP         ;
         

         try {

            connection = dataSource.getConnection();

            if (log.isLoggable(Level.FINEST)) {
               log.finest("CrcItemStateDAO add start " + items.size());
            }

            int sentLen    = itemsSent.size();
            int invalidLen = invalidItems.length;

            
            Iterator values = items.values().iterator();
            
            CrcSyncItemInfo scrcii = null;
            while (values.hasNext()) {
                
                scrcii = (CrcSyncItemInfo)values.next();
                
                ps = connection.prepareStatement(Query.CACHE_ADD_LOCAL_ITEM);

                // get id
                id       = scrcii.getKey().getKeyAsString();
                
                // get CRC
                lastCrc  = scrcii.getLastCrc() ;

                // set the invalid flag
                isInvalid = null;
                for (int k=0; k<invalidLen; k++){
                    if (invalidItems[k].getKey().getKeyAsString().equals(id)) {
                        isInvalid = invalidItems[k].getInvalid();
                        break;
                    }
                }
                
                // change the id for Sent and Outbox folder
                if (id.startsWith(Def.FOLDER_SENT_ID)){
                    for (int y=0; y<sentLen; y++) {
                        sr = (SendResult) itemsSent.get(y);
                        GUID   = sr.getGUID();
                        isSent = sr.isSent();
                        if (GUID.equals(id) && isSent){
                            id = Utility.convertSentIDToOutboxID(id);
                            break;
                        }
                    }
                }

                // messageID
                messageId  = scrcii.getMessageID();
                
                // header date
                headerDate = scrcii.getHeaderDate();
                String headerDateS = null;
                if (headerDate != null){
                    headerDateS = Utility.UtcFromDate(headerDate);
                }
                
                // internal email
                isInternal = scrcii.getInternal();

                // flag for email or folder
                isEmail = scrcii.getIsEmail();
                
                
                ps.setString(1, id       );
                ps.setLong  (2, lastCrc  );
                ps.setString(3, isInvalid);
                ps.setString(4, isInternal);
                ps.setString(5, sourceURI);
                ps.setLong  (6, principal);
                ps.setString(7, messageId);
                ps.setString(8, headerDateS);                
                ps.setString(9, isEmail);                

                ps.executeUpdate();

                ps.close() ;
                ps = null  ;
                
            }   
                        
         } catch (Exception e) {
             throw new EntityException("Error adding local items", e);
         } finally {
             DBTools.close(connection, ps, null);
         }
         
    }
    
}
