/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.engine.source;

import java.io.Serializable;
import java.util.logging.Logger;
import java.util.ArrayList;
import java.util.logging.Level;
import java.sql.Timestamp;

import com.funambol.framework.engine.SyncItem;
import com.funambol.framework.engine.SyncItemKey;
import com.funambol.framework.engine.source.FilterableSyncSource;
import com.funambol.framework.engine.source.SyncSourceException;
import com.funambol.framework.engine.SyncItemState;
import com.funambol.framework.tools.beans.LazyInitBean;
import com.funambol.framework.tools.beans.BeanInitializationException;

import com.funambol.email.pdi.mail.*;
import com.funambol.email.exception.EntityException;
import com.funambol.email.exception.EmailAccessException;
import com.funambol.email.util.Def;
import com.funambol.email.util.Utility;
import com.funambol.email.items.manager.ImapEntityManager;
import com.funambol.email.items.model.SendResult;
import com.funambol.email.engine.cache.CrcSyncItemInfo;
import com.funambol.email.exception.SendingException;
import com.funambol.email.pdi.folder.Folder;
import com.funambol.framework.engine.source.SyncContext;
import java.util.HashMap;
import javax.mail.MessagingException;


/**
 * This class implements common methods of EmailIMAPSyncSource
 * <p/>
 * version $Id: ImapEmailSyncSource.java,v 1.10 2006/11/21 14:50:49 gbmiglia Exp $
 */
public final class ImapEmailSyncSource
        extends EmailSyncSource
        implements FilterableSyncSource, Serializable, LazyInitBean {
    // --------------------------------------------------------------- Constants

    /**  */
    protected Logger log = Logger.getLogger(Def.LOGGER_NAME);

    // -------------------------------------------------------------- Properties

    /**  */
    protected ImapEntityManager iem = null;

    {
        this.setInPort("" + Def.PROTOCOL_IMAP_PORT);
    }

    // ---------------------------------------------------------- Public Methods


    /**
     *
     */
    public void beginSync(SyncContext context) throws SyncSourceException {
        if (log.isLoggable(Level.FINEST)) {
            log.finest("IMAP EmailSyncSource - Start beginSync [" + sourceURI + "]");
        }
        super.beginSync(context);
    }


    /**
     *
     */
    public void endSync() throws SyncSourceException {                    
        super.endSync();
    }

    
    /**
     * Init the syncSource
     *
     * @throws BeanInitializationException
     */
    public void init() throws BeanInitializationException {
        super.init();
    }


    /**
     * @param syncItem SyncItem
     * @return boolean
     * @throws SyncSourceException
     */
    public boolean isSyncItemInFilterClause(SyncItem syncItem)
      throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("IMAP isSyncItemInFilterClause(syncItem)");
        }
        
        boolean isInFilter = false;
        
        char status = ' ';
        try {
            status = getSyncItemStateFromId(syncItem.getKey());
        } catch (SyncSourceException e) {
            try {
                this.mswf.releaseConnection();
            } catch (EmailAccessException e1){
                throw new SyncSourceException("Error closing store connection: " +
                        e1.getMessage(), e1);
            }            
            throw new SyncSourceException("Error checking 'is in filter' for the item " +
                    syncItem, e);
        }
        
        try {
            String content = Utility.getContentFromSyncItem(syncItem);
            boolean isemail = isEmailFromContent(content);
            if (isemail) {
                isInFilter = isEmailInFilter(content, syncItem, status);
            } else {
                isInFilter = isFolderInFilter(content, syncItem);
            }
        } catch (EntityException e) {
            try {
                this.mswf.releaseConnection();
            } catch (EmailAccessException e1){
                throw new SyncSourceException("Error closing store connection: " +
                        e1.getMessage(), e1);
            }            
            throw new SyncSourceException("Error checking 'is in filter' for the item " +
                    syncItem, e);
        }

        return isInFilter;
    }

    /**
     * @param syncItemKey SyncItemKey
     * @return boolean
     * @throws SyncSourceException
     */
    public boolean isSyncItemInFilterClause(SyncItemKey syncItemKey)
      throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("IMAP isSyncItemInFilterClause(syncItemKey)");
        }
        
        boolean isInFilter = false;

        String key  = (String) syncItemKey.getKeyValue();

        char status = ' ';            
        try {
            status = getSyncItemStateFromId(new SyncItemKey(key));            
        } catch (SyncSourceException e) {
            try {
                this.mswf.releaseConnection();
            } catch (EmailAccessException e1){
                throw new SyncSourceException("Error closing store connection: " +
                        e1.getMessage(), e1);
            }            
            throw new SyncSourceException("Error checking 'is in filter' for the item " + key, e);
        } 

        try {
            String FID  = Utility.getKeyPart(key,1);
            String FMID = Utility.getKeyPart(key,2);
            String UIDV = Utility.getKeyPart(key,3);
            boolean isemail = isEmailFromKey(FID, FMID);
            if (isemail) {
                isInFilter = isEmailInFilter(key, FID, FMID, status);
            } else {
                isInFilter = isFolderInFilter(key);
            } 
        } catch (EntityException e) {
            try {
                this.mswf.releaseConnection();
            } catch (EmailAccessException e1){
                throw new SyncSourceException("Error closing store connection: " +
                        e1.getMessage(), e1);
            }            
            throw new SyncSourceException("Error checking 'is in filter' for the item " + key, e);
        } 

        return isInFilter;
    }


    /**
     * @param operation  String
     * @param statusCode int
     * @param keys       SyncItemKey[]
     */
    public void setOperationStatus(String operation,
                                   int statusCode,
                                   SyncItemKey[] keys) {

        if (log.isLoggable(Level.FINEST)) {
            StringBuffer message = new StringBuffer("Received status code '");
            message.append(statusCode).append("' for a '").append(operation).append("'").
                    append(" for this items: ");

            for (int i = 0; i < keys.length; i++) {
                message.append("\n- ").append(keys[i].getKeyAsString());
            }
            log.finest(message.toString());
        }
    }


    /**
     * @param syncItemKey SyncItemKey
     * @param s Timestamp
     * @param engineSoftDelete boolean
     * @throws SyncSourceException
     */
    public void removeSyncItem(SyncItemKey syncItemKey,
                               Timestamp s,
                               boolean engineSoftDelete)
      throws SyncSourceException {
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("remove item engineSoftDelete: " + engineSoftDelete + 
                       " config. soft delete " + this.softDelete);
        }

        try {
            if        (engineSoftDelete) {
                // nothing
            } else if (this.softDelete) {
                // nothing
            } else {    
                removeItem(syncItemKey);
            }
        } catch (EntityException e) {
            try {
                this.mswf.releaseConnection();
            } catch (EmailAccessException e1){
                throw new SyncSourceException("Error closing store connection: " +
                        e1.getMessage(), e1);
            }            
            throw new SyncSourceException("Error deleting the item " + syncItemKey, e);
        }        

    }


    /**
     * @param syncItemKey SyncItemKey
     * @return SyncItem
     * @throws SyncSourceException
     */
    public SyncItem getSyncItemFromId(SyncItemKey syncItemKey)
      throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getSyncItemFromId start");
        }

        SyncItem syncItem = null;
        
        String key = (String) syncItemKey.getKeyValue();
        
        // check if the item is invalid
        boolean isItemValid = this.isValidItem(key);
        
        if (isItemValid){
            
            String FID  = Utility.getKeyPart(key,1);
            String FMID = Utility.getKeyPart(key,2);
            String UIDV = Utility.getKeyPart(key,3);
            
            if (log.isLoggable(Level.FINEST)) {
                log.finest("getSyncItemFromId with [FID: " + FID + 
                           " ; FMID: " + FMID + " ; UIDV: " + UIDV + "]");
            }
            
            char status = ' ';
            
            try {
                 status = getSyncItemStateFromId(syncItemKey);
            } catch (SyncSourceException e) {
                 try {
                     this.mswf.releaseConnection();
                 } catch (EmailAccessException e1){
                     throw new SyncSourceException("Error closing store connection: " +
                           e1.getMessage(), e1);
                 }            
                 throw new SyncSourceException("Error adding the item " + syncItem, e);
            }
            
            
            try {
                boolean isEmail = isEmailFromKey(FID, FMID);
                if (isEmail) {
                    syncItem = getEmailFromId(syncItemKey, status);
                } else {
                    syncItem = getFolderFromId(syncItemKey);
                }
            } catch (EntityException e) {
                 try {
                     this.mswf.releaseConnection();
                 } catch (EmailAccessException e1){
                     throw new SyncSourceException("Error closing store connection: " +
                           e1.getMessage(), e1);
                 }            
                 throw new SyncSourceException("Error adding the item " + syncItem, e);
            }
            
        } else {
        
            log.severe("Item " + key + " is invalid!");
            
        }
        
        return syncItem;

    }


    /**
     * @return SyncItemKey[]
     * @throws SyncSourceException
     */
    public SyncItemKey[] getAllSyncItemKeys() throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getAllSyncItems start ");
        }

        String[]   allIds;

        try {
            this.iem = getImapEntityManager();
            //NOTE: in this map there are the folders id also
            HashMap si = getServerItems();
            allIds = iem.getAllEmails(filter, si);                        
        } catch (EmailAccessException e) {
            try {
                this.mswf.releaseConnection();
            } catch (EmailAccessException e1){
                throw new SyncSourceException("Error closing store connection: " +
                        e1.getMessage(), e1);
            }            
            throw new SyncSourceException("Error connecting manager layer: " +
                    e.getMessage(), e);
        } catch (EntityException e) {
            try {
                this.mswf.releaseConnection();
            } catch (EmailAccessException e1){
                throw new SyncSourceException("Error closing store connection: " +
                        e1.getMessage(), e1);
            }            
            throw new SyncSourceException("Error reading getAllSyncItems : " +
                    e.getMessage(), e);
        }

        SyncItemKey[] syncItemKeys = null;
        if (allIds != null) {
            // remove the GUID null (this is the case of the new folders)
            ArrayList onlyGoodIds = new ArrayList();
            String key;
            for (int i = 0; i < allIds.length; i++) {
                key = allIds[i];
                if (key != null) {
                    onlyGoodIds.add(key);
                }
            }
            int numfin = onlyGoodIds.size();
            // create the syncItemKeys without null
            syncItemKeys = new SyncItemKey[numfin];
            for (int i = 0; i < numfin; i++) {
                syncItemKeys[i] = new SyncItemKey(allIds[i]);
            }
        }

        return syncItemKeys;
    }


    /**
     * @return map with all Mail Server CrcSyncItemInfo
     * @throws SyncSourceException
     */
    //public CrcSyncItemInfo[] getAllSyncItemInfo()
    public HashMap getAllSyncItemInfo()
      throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getAllSyncItemInfo start");
        }

        HashMap allInfo;
        HashMap emailInfo;
        HashMap folderInfo;

        try {

            this.iem = getImapEntityManager();

            emailInfo = this.iem.getAllEmailsInfo(filter, this.userName, 
                                                  this.allMailboxActivation);

            folderInfo = this.iem.getAllFoldersInfo(filter, this.allMailboxActivation,
                                                    this.sourceURI, this.principalId);

            // add main folders
            allInfo = Utility.mergeHashMap(emailInfo, folderInfo, null, null, null);
            
        } catch (EmailAccessException e) {
            try {
                this.mswf.releaseConnection();
            } catch (EmailAccessException e1){
                throw new SyncSourceException("Error closing store connection: " +
                        e1.getMessage(), e1);
            }            
            throw new SyncSourceException("Error connecting manager layer: " +
                    e.getMessage(), e);
        } catch (EntityException e) {
            try {
                this.mswf.releaseConnection();
            } catch (EmailAccessException e1){
                throw new SyncSourceException("Error closing store connection: " +
                        e1.getMessage(), e1);
            }            
            throw new SyncSourceException("Error reading emails/folders: " +
                    e.getMessage(), e);
        }

        return allInfo;
    }

    /**
     * @throws SyncSourceException
     */
    public void clearServerInfo() throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("cleanServerInfo start");
        }

        try {
            this.iem = getImapEntityManager();
            this.iem.clearAllItems(this.allMailboxActivation,
                                   this.sourceURI, this.principalId);
        } catch (EmailAccessException e) {
            try {
                this.mswf.releaseConnection();
            } catch (EmailAccessException e1){
                throw new SyncSourceException("Error closing store connection: " +
                        e1.getMessage(), e1);
            }            
            throw new SyncSourceException("Error connecting manager layer: " +
                    e.getMessage(), e);
        } catch (EntityException e) {
            try {
                this.mswf.releaseConnection();
            } catch (EmailAccessException e1){
                throw new SyncSourceException("Error closing store connection: " +
                        e1.getMessage(), e1);
            }            
            throw new SyncSourceException("Error deleting all emails/folders: " +
                    e.getMessage(), e);
        }

    }

    /**
     * @param syncItem SyncItem
     * @return SyncItemKey[]
     * @throws SyncSourceException
     */
    public SyncItemKey[] getSyncItemKeysFromTwin(SyncItem syncItem)
            throws SyncSourceException {

        SyncItemKey[] syncItemKeys;

        try {
            String content = Utility.getContentFromSyncItem(syncItem);
            boolean isemail = isEmailFromContent(content);
            if (isemail) {
                syncItemKeys = getEmailFromTwin(content, syncItem);
            } else {
                syncItemKeys = getFolderFromTwin(content, syncItem);
            }
        } catch (EntityException e) {
            try {
                this.mswf.releaseConnection();
            } catch (EmailAccessException e1){
                throw new SyncSourceException("Error closing store connection: " +
                        e1.getMessage(), e1);
            }            
            throw new SyncSourceException("Error getting the item for twin" +
                    syncItem, e);
        }
        
        return syncItemKeys;
    }

    /**
     * @param syncItem SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     */
    public SyncItem addSyncItem(SyncItem syncItem) throws SyncSourceException {
        
        try {
            String content = Utility.getContentFromSyncItem(syncItem);
            boolean isEmail = isEmailFromContent(content);
            if (isEmail) {
                syncItem = addEmail(content, syncItem);
            } else {
                syncItem = addFolder(content, syncItem);
            }
        } catch (EntityException e) {
            try {
                this.mswf.releaseConnection();
            } catch (EmailAccessException e1){
                throw new SyncSourceException("Error closing store connection: " +
                        e1.getMessage(), e1);
            }            
            throw new SyncSourceException("Error adding the item " + syncItem, e);
        }
        
        return syncItem;
    }


    /**
     * @param syncItem SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     */
    public SyncItem updateSyncItem(SyncItem syncItem) throws SyncSourceException {

        try {
            String content = Utility.getContentFromSyncItem(syncItem);
            String key = syncItem.getKey().getKeyAsString();
            String parentId = Utility.getKeyPart(key,1);
            String objectId = Utility.getKeyPart(key,2);
            boolean isEmail = isEmailFromContent(content);
            if (isEmail) {
                syncItem = updateEmail(parentId, objectId, content, syncItem);
            } else {
                syncItem = updateFolder(parentId, objectId, content, syncItem);
            }
        } catch (EntityException e) {
            try {
                this.mswf.releaseConnection();
            } catch (EmailAccessException e1){
                throw new SyncSourceException("Error closing store connection: " +
                        e1.getMessage(), e1);
            }            
            throw new SyncSourceException("Error updateing the item " + syncItem, e);
        }

        return syncItem;

    }

    // --------------------------------------------------------- Private Methods

    /**
     * Create EmailIMAPManager istance (if not already exist)
     *
     * @return EmailIMAPManager
     * @throws EmailAccessException
     */
    private ImapEntityManager getImapEntityManager() throws EmailAccessException {
        try {
            if (this.iem == null) {
                this.iem = new ImapEntityManager(this.mswf, this.jndiDataSourceName);
            }
        } catch (Exception e) {
            throw new EmailAccessException(e.getMessage());
        }
        return this.iem;
    }

    /**
     * 
     * @param syncItemKey SyncItemKey
     * @throws SyncSourceException
     */
    private void removeItem(SyncItemKey syncItemKey)
       throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("removeItem  GUID:" + syncItemKey );
        }

        String key      = syncItemKey.getKeyAsString();
        String parentId = Utility.getKeyPart(key,1);
        if (parentId == null){
            // I have to set the parent
            parentId = "O";
        }        
        String objectId = Utility.getKeyPart(key,2);
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("remove item FID: " + parentId +" FMID: " + objectId);
        }

        boolean isEmail = isEmailFromKey(parentId, objectId);

        if (isEmail) {
            removeEmail(parentId, objectId, key);
        } else {
            removeFolder(parentId, objectId);
        }

    }

    /**
     * @param content  String
     * @param syncItem SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     */
    private SyncItem addEmail(String content, SyncItem syncItem)
      throws EntityException {

        String[] keys = Utility.getKeyParts(syncItem);
        String FID    = keys[0];
        String FMID   = keys[1];

        if (log.isLoggable(Level.FINEST)) {
            log.finest("addEmail [FID: " + FID + " luid: " + FMID + "]");
        }

        Email  ctmpNew       = null;
        Email  insertedEmail = null;
        String GUID          = null;

        try {
            ctmpNew = getFoundationMailFromXML(content);
        } catch (EntityException e) {
            throw new EntityException("Error converting foundation: " +
                    e.getMessage(), e);
        } 

        try {

            this.iem = getImapEntityManager();

            // add and send the mail
            insertedEmail = this.iem.addEmail(FID, FMID, ctmpNew,
                                              this.sourceURI, this.principalId);

            // message add and sent without error, set to true the result
            if (insertedEmail != null) {
                GUID = (String) insertedEmail.getUID().getPropertyValue();
                String idtmp = Utility.convertOutboxIDToSentID(GUID);
                SendResult sr = new SendResult(idtmp, FMID, true, false);
                sentItems.add(sr);
            }


        } catch (EmailAccessException e) {
            throw new EntityException("Error connecting manager layer: " +
                    e.getMessage(), e);
        } catch (EntityException e) {
            String errorMessage = "Error setting the item: " + syncItem;
            if (log.isLoggable(Level.SEVERE)) {
                log.severe(errorMessage);
            }
            throw new EntityException(errorMessage, e);
        } catch (SendingException e) {
            SendResult sr = new SendResult(GUID, FMID, false, false);
            sentItems.add(sr);
            //throw new SyncSourceException(e.getMessage(), 500);
            throw new EntityException(e.getMessage(), 500);
        }


        // create the output syncItem
        try {
            if (insertedEmail != null) {
                syncItem = createSyncItem(insertedEmail, SyncItemState.NEW);
            }
        } catch (Exception e) {
            throw new EntityException("Error creating SyncItem: " +  syncItem, e);
        }

        return syncItem;

    }


    /**
     *
     * check if the message is included in the filter clause
     *
     * @param content  String
     * @param syncItem SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     */
    private boolean isEmailInFilter(String key, 
                                    String FID, 
                                    String FMID, 
                                    char status) 
      throws EntityException {

        boolean isInFilter = false;
                
        try {
            this.iem   = getImapEntityManager();
            isInFilter = this.iem.isEmailInFilter(null, filter, 
                                                  key, FID, FMID, status,
                                                  this.sourceURI, this.principalId);
        } catch (EmailAccessException e) {
            throw new EntityException("Error connecting manager layer: " +
                                          e.getMessage(), e);
        } catch (EntityException e) {
            throw new EntityException("Error checking the item: " +
                                          e.getMessage() , e);
        }


        return isInFilter;

    }

    /**
     *
     * check if the message is included in the filter clause
     *
     * @param content  String
     * @param syncItem SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     */
    private boolean isEmailInFilter(String content, 
                                    SyncItem syncItem,
                                    char status)
      throws EntityException {

        boolean isInFilter = false;

        String key    = syncItem.getKey().getKeyAsString();        
        String[] keys = Utility.getKeyParts(syncItem);
        String FID    = keys[0];
        String FMID   = keys[1];

        Email email = null;
        try {
            email = getFoundationMailFromXML(content);
        } catch (EntityException e) {
            throw new EntityException("Error converting Item Content", e);
        }
                
        try {
            this.iem = getImapEntityManager();
            isInFilter = this.iem.isEmailInFilter(email, filter,
                                                  key, FID, FMID, status,
                                                  this.sourceURI, this.principalId);
        } catch (EmailAccessException e) {
            throw new EntityException("Error accessing MailBox", e);
        } catch (EntityException e) {
            throw new EntityException("Error matching mail with filter ", e);
        }

        return isInFilter;

    }

    /**
     *
     * check if the message is included in the filter clause
     *
     * @param content  String
     * @param syncItem SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     */
    private boolean isFolderInFilter(String key) 
      throws EntityException {
        boolean isInFilter = false;
        return isInFilter;

    }

    /**
     *
     * check if the message is included in the filter clause
     *
     * @param content  String
     * @param syncItem SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     */
    private boolean isFolderInFilter(String content, SyncItem syncItem)
     throws EntityException {
        boolean isInFilter = false;
        return isInFilter;

    }

    /**
     * Before inserting the folder it checks if the folder
     * already exists. This step is necessary because the
     * "twin" method returns always "item not found"
     * <p/>
     * the server in the slow sync will always call the Add method
     * (there is no filter provided by the Twin Methods)
     * <p/>
     * The add method must choose between
     * 1) the folder already exists (returns the old GUID)
     * 2) the folder must be created (returns the new GUID)
     *
     * @param content  String
     * @param syncItem SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     */
    private SyncItem addFolder(String content, SyncItem syncItem)
      throws EntityException {

        Folder ctmpNew;
        try {
            ctmpNew = getFoundationFolderFromXML(content);
        } catch (EntityException e) {
            throw new EntityException("Error converting foundation: " +
                    e.getMessage(), e);
        }

        Folder insertedFolder = null;
        String GUID;

        String name     = ctmpNew.getName().getPropertyValueAsString();
        String parentId = syncItem.getParentKey().getKeyAsString();

        if (parentId.equalsIgnoreCase("/")) {
            parentId = Def.FOLDER_ROOT_ID;
        }

        try {

            this.iem = getImapEntityManager();

            // check if the folder already exists
            GUID = this.iem.getFolderFromName(name, parentId, 
                                              this.sourceURI, this.principalId);

            // if null I have to insert the folder otherwise
            // the folder already exists
            if (GUID == null) {
                insertedFolder = this.iem.addFolder(name, parentId,
                                                    this.defaultFolder,
                                                    idFolderSpace,
                                                    this.sourceURI, 
                                                    this.principalId);
            }

        } catch (Exception e) {
            String errorMessage = "Error setting the item: " + syncItem;
            if (log.isLoggable(Level.SEVERE)) {
                log.severe(errorMessage);
            }
            throw new EntityException(errorMessage, e);
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("End insert folder. start create SyncItem");
        }

        // create the output syncItem
        try {
            if (insertedFolder != null) {
                syncItem = createSyncItem(insertedFolder, SyncItemState.NEW);
            }
        } catch (Exception e) {
            throw new EntityException("Error creating SyncItem: " +
                    syncItem, e);
        }

        return syncItem;
    }

    /**
     * @param syncItemKey SyncItemKey
     * @return SyncItem
     * @throws SyncSourceException
     */
    private SyncItem getEmailFromId(SyncItemKey syncItemKey, char status)
      throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getEmailFromId start");
        }

        SyncItem syncItem = null;
        Email    mail     = null;
       
        String key = syncItemKey.getKeyAsString();
        
        try {
            String parentId = Utility.getKeyPart(key,1);
            String mailId   = Utility.getKeyPart(key,2);
            this.iem = getImapEntityManager();
            mail = this.iem.getEmailFromUID(filter, mailId, parentId, status,
                                            this.sourceURI, this.principalId);
        } catch (EmailAccessException e) {
            throw new EntityException("Error connecting IMAP store: " + e.getMessage(), e);
        } catch (EntityException e) {
            if (e.getMessage().indexOf(Def.ERROR_FILTERING_EMAIL) != -1){
                // set as invalid the item in the cache, in the next sync will be ignored
                super.insertInvalidItem(key, null, null, "Y");
                throw new EntityException(e.getMessage(), e);
            } else {
                throw new EntityException("Error in getting Mail " + e.getMessage(), e);
            }
        }

        if (mail != null) {
            try {
                syncItem = createSyncItem(mail, status);
            } catch (Exception e) {
                throw new EntityException("Error in creating SyncItem ", e);
            }
        }

        return syncItem;

    }

    /**
     * the folder syncItemKey is the GUID in the
     * local DB.
     *
     * @param syncItemKey SyncItemKey
     * @return SyncItem
     * @throws SyncSourceException
     */
    private SyncItem getFolderFromId(SyncItemKey syncItemKey)
            throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getFolderFromId start");
        }

        SyncItem syncItem = null;
        Folder   folder   = null;

        String GUID = syncItemKey.getKeyAsString();

        try {
            this.iem = getImapEntityManager();
            folder = this.iem.getFolderFromUID(filter, GUID, 
                                               this.sourceURI, this.principalId);
        } catch (Exception e) {
            throw new SyncSourceException("Error in getting Folder ", e);
        }

        // get item status
        char status = getSyncItemStateFromId(syncItemKey);

        if (folder != null) {
            try {
                syncItem = createSyncItem(folder, status);
            } catch (Exception e) {
                throw new SyncSourceException("Error in creating SyncItem ", e);
            }
        }

        return syncItem;

    }

    /**
     * @param content  String
     * @param syncItem SyncItem
     * @return SyncItemKey[]
     * @throws SyncSourceException
     */
    private SyncItemKey[] getEmailFromTwin(String content,
                                           SyncItem syncItem)
     throws EntityException {

        String[] keys = Utility.getKeyParts(syncItem);
        String FID  = keys[0];
        String FMID = keys[1];

        String GUID;

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getEmailFromTwin with FID " + FID + " FMID " + FMID);
        }

        Email email;
        try {
            email = getFoundationMailFromXML(content);
        } catch (EntityException e) {
            throw new EntityException("Error converting foundation: " +
                    e.getMessage(), e);
        }

        HashMap serverItems = getServerItems();

        try {
            
            this.iem = getImapEntityManager();
            
            GUID = this.iem.getEmailFromClause(this.mswf.getSession(), 
                                               serverItems, FID, email);
            
        } catch (EmailAccessException e) {
            throw new EntityException("Error accessing MailBox", e);
        } catch (EntityException e) {
            throw new EntityException("Error getting twin", e);
        }

        SyncItemKey[] syncItemKeys;
        if (GUID != null) {
            syncItemKeys = new SyncItemKey[1];
            syncItemKeys[0] = new SyncItemKey(GUID);
        } else {
            syncItemKeys = new SyncItemKey[0];
        }

        return syncItemKeys;
    }

    /**
     * this method return always "twin not found"
     * the server will always call the Add method
     * The add method must choose between
     * 1) the folder already exists (returns the old GUID)
     * 2) the folder must be created (returns the new GUID)
     *
     * @param content  String
     * @param syncItem SyncItem
     * @return SyncItemKey[]
     * @throws SyncSourceException
     */
    private SyncItemKey[] getFolderFromTwin(String content,
                                            SyncItem syncItem)
     throws EntityException {

        SyncItemKey[] syncItemKeys;
        String GUID;

        String key = (String) syncItem.getKey().getKeyValue();
        if (log.isLoggable(Level.FINEST)) {
            log.finest("getSyncItemKeysFromTwin with UID: " + key);
        }
        
        Folder ctmpNew;
        try {
            ctmpNew = getFoundationFolderFromXML(content);
        } catch (EntityException e) {
            throw new EntityException("Error converting foundation: " +
                    e.getMessage(), e);
        }

        String name = ctmpNew.getName().getPropertyValueAsString();
        String parentId = syncItem.getParentKey().getKeyAsString();

        if (parentId.equalsIgnoreCase("/")) {
            parentId = Def.FOLDER_ROOT_ID;
        }

        try {

            this.iem = getImapEntityManager();

            // check if the folder already exists
            GUID = this.iem.getFolderFromName(name, parentId, 
                                              this.sourceURI, this.principalId);

            if (GUID != null) {
                syncItemKeys = new SyncItemKey [1];
                syncItemKeys[0] = new SyncItemKey(GUID);
            } else {
                syncItemKeys = new SyncItemKey[0];
            }


        } catch (Exception e) {
            String errorMessage = "Error setting the item: " + syncItem;
            if (log.isLoggable(Level.SEVERE)) {
                log.severe(errorMessage);
            }
            throw new EntityException(errorMessage, e);
        }

        return syncItemKeys;
    }

    /**
     * @param parentId String
     * @param mailId   mail id in the folder not unique
     * @throws SyncSourceException
     */
    private void removeEmail(String parentId, String mailId, String GUID)
      throws EntityException {

        try {
            this.iem = getImapEntityManager();
            HashMap serverItems = getServerItems();
            this.iem.removeEmail(parentId, 
                                 mailId, 
                                 GUID, 
                                 serverItems, 
                                 this.userName, 
                                 this.sourceURI, 
                                 this.principalId);
        } catch (EmailAccessException e) {
            throw new EntityException("Error accessing MailBox", e);
        } catch (EntityException e) {
            throw new EntityException("Error removing email", e);
        }
    }

    /**
     * @param parentId String
     * @param folderId String
     * @throws SyncSourceException
     */
    private void removeFolder(String parentId, String folderId)
      throws EntityException {

        try {
            this.iem = getImapEntityManager();
            this.iem.removeFolder(folderId, this.sourceURI, this.principalId);
        } catch (EmailAccessException e) {
            throw new EntityException("Error accessing Mailbox", e);
        } catch (EntityException e) {
            throw new EntityException("Error removing folder", e);
        }

    }

    /**
     *
     * @param content  String
     * @param syncItem SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     */
    private SyncItem updateEmail(String parentId,
                                 String mailId,
                                 String content,
                                 SyncItem syncItem)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("updateSyncItem FID " + parentId + " FMID " + mailId);
        }

        Email emailNew;
        try {
            emailNew = getFoundationMailFromXML(content);
        } catch (EntityException e) {
            throw new EntityException("Error converting foundation: " +
                    e.getMessage(), e);
        }

        Email emailOut;

        try {
            this.iem = getImapEntityManager();
            emailOut = this.iem.updateEmail(parentId, mailId, emailNew,
                                            this.sourceURI, this.principalId);
        } catch (Exception e) {
            String errorMessage = "Error setting the item: " + syncItem;
            if (log.isLoggable(Level.SEVERE)) {
                log.severe(errorMessage);
            }
            throw new EntityException(errorMessage, e);
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("End update. start create SyncItem");
        }

        // create the output syncItem
        try {
            if (emailOut != null) {
                syncItem = createSyncItem(emailOut, SyncItemState.UPDATED);
            }
        } catch (Exception e) {
            throw new EntityException("Error creating SyncItem: " + syncItem, e);
        }

        return syncItem;

    }

    /**
     * @param content  String
     * @param syncItem SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     */
    private SyncItem updateFolder(String parentId,
                                  String folderId,
                                  String content,
                                  SyncItem syncItem)
      throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("updateFolder FID " + parentId + " and FMID " + folderId);
        }
        Folder folderNew;
        try {
            folderNew = getFoundationFolderFromXML(content);
        } catch (EntityException e) {
            throw new EntityException("Error converting foundation: " +
                    e.getMessage(), e);
        }

        Folder folderOut;
        try {
            this.iem = getImapEntityManager();
            folderOut = this.iem.updateFolder(parentId, folderId, folderNew);
        } catch (Exception e) {
            String errorMessage = "Error setting the item: " + syncItem;
            if (log.isLoggable(Level.SEVERE)) {
                log.severe(errorMessage);
            }
            throw new EntityException(errorMessage, e);
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("End update folder. start create SyncItem");
        }

        // create the output syncItem
        try {
            if (folderOut != null) {
                syncItem = createSyncItem(folderOut, SyncItemState.UPDATED);
            }
        } catch (Exception e) {
            throw new EntityException("Error creating SyncItem: " + syncItem, e);
        }

        return syncItem;

    }   

}
