/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.inboxlistener.console;


import java.util.List;

import com.funambol.email.exception.InboxListenerException;
import com.funambol.email.exception.InboxListenerConfigException;
import com.funambol.email.inboxlistener.model.DBAccount;
import com.funambol.email.inboxlistener.model.MailboxAccount;
import com.funambol.email.inboxlistener.admin.InboxListenerConfig;
import com.funambol.email.util.UtilityIL;

/**
 * simple admin console
 *
 * @version $Id: Menu.java,v 1.10 2006/11/14 09:21:45 gbmiglia Exp $
 */
public class Menu {

  private DBAccount dba     = null;
  private MenuDAO   menuDAO = null;

  /**
   *
   */
  public Menu() {
     go();
  }

  /**
   *
   */
  public static void main(String[] args) {
     Menu m = new Menu();
  }


  /**
   *
   */
  public void go(){

     // load DataBase Properties
     String dbname        = "";
     String jdbcDriver    = "";
     String jdbcUrl       = "";
     String jdbcUser      = "";
     String jdbcPassword  = "";
     int    reloadingTime = 0;
     try {
        jdbcDriver    = InboxListenerConfig.getConfigInstance().getJdbcDriver();
        jdbcUrl       = InboxListenerConfig.getConfigInstance().getJdbcUrl();
        jdbcUser      = InboxListenerConfig.getConfigInstance().getJdbcUser();
        jdbcPassword  = InboxListenerConfig.getConfigInstance().getJdbcPassword();
        reloadingTime = InboxListenerConfig.getConfigInstance().getReloadingTime();
        dba          = new DBAccount(jdbcDriver,
                                     jdbcUrl,
                                     jdbcUser,
                                     jdbcPassword,
                                     reloadingTime);
     } catch (InboxListenerConfigException e) {
       e.printStackTrace();
     }

     //
     menuDAO = new MenuDAO(dba);

     try {
        // clean display
        for (int p=0;p<50;p++)
          System.out.println("");

        int option = -1;

        while (true) {
          option = mainMenu();
          if (option == 0) {
            break;
          }
        }
     }
     catch(java.lang.Exception e) {
        e.printStackTrace();
     }

  }



  /**
   *
   */
  public int mainMenu()
  {
    String option = null;
    int value = -1;

    try {
       System.out.println("");
       System.out.println("");
       System.out.println(" INBOX LISTENER ACCOUNT CONSOLE");
       System.out.println("");
       System.out.println(" 0   quit");
       System.out.println(" 1   list accounts");
       System.out.println(" 2   get account");
       System.out.println(" 3   insert account");
       System.out.println(" 4   delete account");
       System.out.println(" 5   update max email number");
       System.out.println(" 6   update password");
       System.out.println(" 7   update activation");
       System.out.println(" ");

       value = UtilityIL.readInteger();

       switch (value) {
        case 1: list();                 break;
        case 2: getAccount();           break;
        case 3: insert();               break;
        case 4: delete();               break;
        case 5: updateMaxEmailNumber(); break;
        case 6: updatePassword();       break;
        case 7: updateActivation();     break;
        case 0: break;
        default: System.out.println("error");
         return 0;
       }
      } catch(java.lang.Exception e){
        e.printStackTrace();
        return 0;
      }
      return value;
   }


  //------------------------------------------------------------ PRIVATE METHODS


  /**
   *
   */
  private void list() throws InboxListenerException {
     try {
        List l = menuDAO.getAccounts();
        if (l != null) {
          if (l.size() == 0){
            System.out.println("No accounts found");
          }
          for (int i=0; i < l.size(); i++){
            System.out.println(((MailboxAccount)l.get(i)).toString());
          }
        } else {
          System.out.println("No accounts found");
        }
     } catch (Exception ex) {
        throw new InboxListenerException("Error reading accounts (" +
                                          ex.getMessage() + ")");
     }
  }

  
  /**
   *
   */
  private void getAccount() throws InboxListenerException {
     try {
         
       System.out.print("username: ");
       String username = UtilityIL.readValue();
         
        MailboxAccount ma = menuDAO.getAccount(username);
        
        if (ma != null) {
           System.out.println(ma.toString());
        } else {
           System.out.println("Account not found");
        }
        
     } catch (Exception ex) {
        throw new InboxListenerException("Error getting account (" +
                                          ex.getMessage() + ")");
     }
  }  
  
  /**
   *
   */
  private void insert() throws InboxListenerException {
     try {
       System.out.print("username: ");
       String username = UtilityIL.readValue();
       System.out.print("password: ");
       String password = UtilityIL.readValue();
       System.out.print("listener id (number): ");
       String listenerID = UtilityIL.readValue();
       System.out.print("Max Email Number in Cache (number): ");
       int maxNumEmail = Integer.parseInt(UtilityIL.readValue());

       MailboxAccount ma = new MailboxAccount(username,
                                              password,
                                              true,
                                              listenerID, 
                                              maxNumEmail);

       menuDAO.insertAccount(ma);

     } catch (Exception ex) {
       throw new InboxListenerException("Error updating accounts (" +
                                         ex.getMessage() + ")");
     }
  }

  /**
   *
   */
  private void updatePassword() throws InboxListenerException {
     try {
       System.out.print("username: ");
       String username = UtilityIL.readValue();
       System.out.print("password: ");
       String password = UtilityIL.readValue();

       int result = menuDAO.updatePassword(username,password);

       if (result > 0){
          System.out.print("Password updated");
       } else {
          System.out.print("Account not updated");
       }
       
     } catch (Exception ex) {
       throw new InboxListenerException("Error updating accounts (" +
                                         ex.getMessage() + ")");
     }
  }  
  
  
  /**
   *
   */
  private void updateActivation() throws InboxListenerException {
     try {
       System.out.print("username: ");
       String username = UtilityIL.readValue();
       System.out.print("activation value (y/n - case sensitive): ");
       String activation = UtilityIL.readValue();

       int result = menuDAO.updateActivation(username,activation);

       if (result > 0){
          System.out.print("Activation updated");
       } else {
          System.out.print("Account not updated");
       }
       
     } catch (Exception ex) {
       throw new InboxListenerException("Error updating accounts (" +
                                         ex.getMessage() + ")");
     }
  }  
  
  
  
  /**
   *
   */
  private void updateMaxEmailNumber() throws InboxListenerException {
     try {
       System.out.print("username: ");
       String username = UtilityIL.readValue();
       System.out.print("Max Email Number in Cache (number): ");
       int maxNumEmail = Integer.parseInt(UtilityIL.readValue());

       int result = menuDAO.updateMaxEmailNumber(username,maxNumEmail);

       if (result > 0){
          System.out.print("Max Email Number updated");
       } else {
          System.out.print("Account not updated");
       }
       
     } catch (Exception ex) {
       throw new InboxListenerException("Error inserting accounts (" +
                                         ex.getMessage() + ")");
     }
  }  
  
  /**
   *
   */
  private void delete() throws InboxListenerException {
     try {
       System.out.print("username: ");
       String username = UtilityIL.readValue();

       menuDAO.deleteAccount(username);

     } catch (Exception ex) {
       throw new InboxListenerException("Error deleting accounts (" +
                                         ex.getMessage() + ")");
     }
  }

}
