/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.inboxlistener.console;


import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;
import java.sql.PreparedStatement;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;

import com.funambol.email.util.Def;
import com.funambol.email.exception.InboxListenerException;
import com.funambol.email.inboxlistener.engine.DBTools;
import com.funambol.email.inboxlistener.model.DBAccount;
import com.funambol.email.inboxlistener.model.MailboxAccount;
import com.funambol.email.util.Query;

/**
 * Contains methods to access to the DB tables
 *
 * @version $Id: MenuDAO.java,v 1.8 2006/11/15 17:39:31 gbmiglia Exp $
 */
public class MenuDAO {

  // --------------------------------------------------------------- Constants

  private Logger log = Logger.getLogger(Def.LOGGER_NAME);


  // ------------------------------------------------------------ Private data

  /**   */
  private DBAccount dba   = null;

  /**   */
  private DBTools   dbtools   = null;

  // ------------------------------------------------------------- Constructor

  /**
   *
   */
  public MenuDAO(DBAccount _dba) {
     this.dba = _dba;
     this.dbtools = new DBTools(_dba);
  }




  // ---------------------------------------------------------- Public Methods



  /**
   * Gets Accounts
   * returns a list that contains MailAccount
   *
   */
  public List getAccounts () throws InboxListenerException {

         Connection connection = null;
         PreparedStatement ps  = null;
         ResultSet rs          = null;
         List accounts         = new ArrayList();;
         MailboxAccount ma     = null;
         
         String  username      = null;
         String  password      = null;
         
         boolean isEnable      = false;
         String  isEnableS     = null;
         
         String  listenerID    = null;
         
         int maxNumEmail       = Def.MAX_INBOX_EMAIL_NUMBER;
         String  maxNumEmailS  = ""+Def.MAX_INBOX_EMAIL_NUMBER;

         try {

             connection = this.dbtools.getConnection();

             ps = connection.prepareStatement(Query.ACCOUNT_SELECT_ALL_ACCOUNTS);

             rs = ps.executeQuery();

             while(rs.next()) {
                 username     = rs.getString(1);
                 password     = rs.getString(2);
                 listenerID   = rs.getString(3);
                 maxNumEmailS = rs.getString(4);
                 if (maxNumEmailS != null && !maxNumEmailS.equals("")){
                     maxNumEmail = Integer.parseInt(maxNumEmailS);
                 }
                 isEnableS = rs.getString(5);
                 if (isEnableS != null && !isEnableS.equals("")){
                     if (isEnableS.equalsIgnoreCase("y")){
                         isEnable = true;
                     } else {
                         isEnable = false;
                     }
                 } else {
                     isEnable = false;
                 }

                 ma = new MailboxAccount(username,
                                         password,
                                         isEnable,
                                         listenerID,
                                         maxNumEmail);

                 accounts.add(ma);
             }

         } catch (SQLException e) {
             throw new InboxListenerException("Error reading DataBase (" +
                                     e.getMessage() + ")");
         } catch (InboxListenerException e) {
             throw new InboxListenerException("Error connecting DataBase (" +
                                     e.getMessage() + ")");
         } catch (Exception e) {
             throw new InboxListenerException("Generic Error", e);
         } finally {
             try {
                 this.dbtools.releaseConnection(rs, ps);
             } catch (InboxListenerException e){
                 throw new InboxListenerException("Error setting ServerItems into DataBase (" +
                                                  e.getMessage() + ")" );
             }
         }


         return accounts;
  }

  
  
  /**
   * Get Account; returns the MailAccount Object with 
   * all the info about the user
   * @param username String
   * @return account info MailboxAccount
   */
  public MailboxAccount getAccount (String username) throws InboxListenerException {

         Connection connection = null;
         PreparedStatement ps  = null;
         ResultSet rs          = null;
         MailboxAccount ma     = null;
         
         String  password      = null;
         
         boolean isEnable      = true;
         
         String  listenerID    = null;
         
         int maxNumEmail       = Def.MAX_INBOX_EMAIL_NUMBER;
         String  maxNumEmailS  = ""+Def.MAX_INBOX_EMAIL_NUMBER;

         try {

             connection = this.dbtools.getConnection();

             ps = connection.prepareStatement(Query.ACCOUNT_GET_ACCOUNT);

             ps.setString(1, username);
             
             rs = ps.executeQuery();

             while(rs.next()) {
                 //username     = rs.getString(1);
                 password     = rs.getString(2);
                 listenerID   = rs.getString(3);
                 maxNumEmailS = rs.getString(4);
                 if (maxNumEmailS != null && !maxNumEmailS.equals("")){
                     maxNumEmail = Integer.parseInt(maxNumEmailS);
                 }

                 ma = new MailboxAccount(username,
                                         password,
                                         isEnable,
                                         listenerID,
                                         maxNumEmail);

             }

         } catch (SQLException e) {
             throw new InboxListenerException("Error reading DataBase (" +
                                     e.getMessage() + ")");
         } catch (InboxListenerException e) {
             throw new InboxListenerException("Error connecting DataBase (" +
                                     e.getMessage() + ")");
         } catch (Exception e) {
             throw new InboxListenerException("Generic Error", e);
         } finally {
             try {
                 this.dbtools.releaseConnection(rs, ps);
             } catch (InboxListenerException e){
                 throw new InboxListenerException("Error setting ServerItems into DataBase (" +
                                                  e.getMessage() + ")" );
             }
         }

         return ma;
  }
  

  /**
   * Gets Accounts
   * returns a list that contains MailAccount
   *
   */
  public void insertAccount (MailboxAccount account)
    throws InboxListenerException {

         Connection connection  = null;
         PreparedStatement ps_1 = null;
         PreparedStatement ps_2 = null;

         try {

             connection = this.dbtools.getConnection();

             String listenerID = account.getListenerID();
             String username   = account.getUser();
             String password   = account.getPassword();
             boolean isEnable  = account.isEnable();
             int maxNumEmail   = account.getMaxEmailNumber();
             String activation = null;
             if (isEnable){
                activation = "y";
             } else {
                activation = "n";
             }
             boolean alreadyIn = false;

             ps_2 = connection.prepareStatement(Query.ACCOUNT_INSERT_USER);

             ps_2.setString(1, username);
             ps_2.setString(2, password);
             ps_2.setString(3, listenerID);
             ps_2.setString(4, activation);
             ps_2.setString(5, ""+maxNumEmail);

             ps_2.executeUpdate();


         } catch (SQLException e) {
             throw new InboxListenerException("Error handling DataBase (" +
                                     e.getMessage() + ")");
         } catch (InboxListenerException e) {
             throw new InboxListenerException("Error connecting DataBase (" +
                                     e.getMessage() + ")");
         } catch (Exception e) {
             throw new InboxListenerException("Generic Error", e);
         }  finally {
             try {
                 this.dbtools.releaseConnection(ps_1, ps_2);
             } catch (InboxListenerException e){
                 throw new InboxListenerException("Error setting ServerItems into DataBase (" +
                                                  e.getMessage() + ")" );
             }
         }
         

  }

  
  /**
   * update the password of the Account
   * @param username String
   * @param new password String
   */
  public int updatePassword (String username, String password)
    throws InboxListenerException {

         Connection connection  = null;
         PreparedStatement ps = null;
         int result = 0;
         
         try {

             connection = this.dbtools.getConnection();

             ps = connection.prepareStatement(Query.ACCOUNT_UPDATE_PASSWORD);

             ps.setString(1, password);
             ps.setString(2, username);

             result = ps.executeUpdate();


         } catch (SQLException e) {
             throw new InboxListenerException("Error handling DataBase (" +
                                     e.getMessage() + ")");
         } catch (InboxListenerException e) {
             throw new InboxListenerException("Error connecting DataBase (" +
                                     e.getMessage() + ")");
         } catch (Exception e) {
             throw new InboxListenerException("Generic Error", e);
         } finally {
             try {
                 this.dbtools.releaseConnection(ps);
             } catch (InboxListenerException e){
                 throw new InboxListenerException("Error setting ServerItems into DataBase (" +
                                                  e.getMessage() + ")" );
             }
         }
         
         return result;

  }  
  
  /**
   * update the password of the Account
   * @param username String
   * @param new password String
   */
  public int updateActivation (String username, String activation)
    throws InboxListenerException {

         Connection connection  = null;
         PreparedStatement ps = null;
         int result = 0;
         
         try {

             connection = this.dbtools.getConnection();

             ps = connection.prepareStatement(Query.ACCOUNT_UPDATE_ACTIVATION);

             ps.setString(1, activation);
             ps.setString(2, username);

             result = ps.executeUpdate();


         } catch (SQLException e) {
             throw new InboxListenerException("Error handling DataBase (" +
                                     e.getMessage() + ")");
         } catch (InboxListenerException e) {
             throw new InboxListenerException("Error connecting DataBase (" +
                                     e.getMessage() + ")");
         } catch (Exception e) {
             throw new InboxListenerException("Generic Error", e);
         } finally {
             try {
                 this.dbtools.releaseConnection(ps);
             } catch (InboxListenerException e){
                 throw new InboxListenerException("Error setting ServerItems into DataBase (" +
                                                  e.getMessage() + ")" );
             }
         }
         
         return result;

  }  
  
  /**
   * update the password of the Account
   * @param username String
   * @param new password String
   */
  public int updateMaxEmailNumber (String username, int maxemailnumber)
    throws InboxListenerException {

         Connection connection  = null;
         PreparedStatement ps = null;
         int result = 0; 
         
         try {

             connection = this.dbtools.getConnection();

             ps = connection.prepareStatement(Query.ACCOUNT_UPDATE_MAXEMAIL);

             ps.setString(1, ""+maxemailnumber);
             ps.setString(2, username);

             result = ps.executeUpdate();

         } catch (SQLException e) {
             throw new InboxListenerException("Error handling DataBase (" +
                                     e.getMessage() + ")");
         } catch (InboxListenerException e) {
             throw new InboxListenerException("Error connecting DataBase (" +
                                     e.getMessage() + ")");
         } catch (Exception e) {
             throw new InboxListenerException("Generic Error", e);
         } finally {
             try {
                 this.dbtools.releaseConnection(ps);
             } catch (InboxListenerException e){
                 throw new InboxListenerException("Error setting ServerItems into DataBase (" +
                                                  e.getMessage() + ")" );
             }
         }
         
         return result;

  }    
  
  /**
   * Gets Accounts
   * returns a list that contains MailAccount
   *
   */
  public void deleteAccount (String username)
    throws InboxListenerException {

         Connection connection  = null;
         PreparedStatement ps = null;

         try {
             
             connection = this.dbtools.getConnection();
             
             ps = connection.prepareStatement(Query.ACCOUNT_DELETE_USER);
             ps.setString(1, username);
             
             ps.executeUpdate();

         } catch (SQLException e) {
             throw new InboxListenerException("Error deleteing account (" +
                                     e.getMessage() + ")");
         } catch (InboxListenerException e) {
             throw new InboxListenerException("Error connecting DataBase (" +
                                     e.getMessage() + ")");
         } catch (Exception e) {
             throw new InboxListenerException("Generic Error", e);
         } finally {
             try {
                 this.dbtools.releaseConnection(ps);
             } catch (InboxListenerException e){
                 throw new InboxListenerException("Error setting ServerItems into DataBase (" +
                                                  e.getMessage() + ")" );
             }
         }

  }

  // --------------------------------------------------------- Private Methods

  /**
   *
   *
   * @return Connection
   * @throws SeccException
   */
  /*
  private Connection getConnection() throws InboxListenerException {

       Connection conn = null;

       try {
         Class.forName(dba.getJdbcDriver()).newInstance();
         conn = DriverManager.getConnection(dba.getJdbcUrl(),
                                            dba.getJdbcUser(),
                                            dba.getJdbcPassword());
       } catch (ClassNotFoundException e){
         throw new InboxListenerException("Error connecting DB (Class Not Found): (" +
                                e.getMessage() + ")");
       } catch (InstantiationException e){
         throw new InboxListenerException("Error connecting DB (Instantiation Exception): (" +
                                e.getMessage() + ")");
       } catch (IllegalAccessException e){
         throw new InboxListenerException("Error connecting DB (Illegal Access): " +
                                e.getMessage() + ")");
       } catch (SQLException e){
         throw new InboxListenerException("Error connecting DB (SQLException): (" +
                                e.getMessage() + ")" );
       }

       return conn;
  }
  */

}
