/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.inboxlistener.engine;


import com.funambol.email.exception.InboxListenerException;
import com.funambol.email.inboxlistener.model.DBAccount;
import com.funambol.email.util.Def;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.logging.Logger;



/**
 * This class groups some utility methods for DB invoking
 *
 * @version $Id: DBTools.java,v 1.2 2006/11/15 17:39:17 gbmiglia Exp $
 */
public class DBTools {
    
    // --------------------------------------------------------------- Constants

    private Logger log = Logger.getLogger(Def.LOGGER_NAME);

    // ------------------------------------------------------------ Private data

    /**
     * The DB connection
     */
    private Connection conn = null;

    /**
     * 
     */
    private DBAccount dba = null;

    // ------------------------------------------------------------ Constructors

    /**
     * Creates a new DBTools with the given DBAccount
     *
     * @param _dbs the DBAccount
     */
    public DBTools(DBAccount _dba) {
        this.dba = _dba;
    }

    
  /**
   *
   *
   * @return Connection
   * @throws SeccException
   */
  public Connection getConnection() 
    throws InboxListenerException {

       try {
         Class.forName(dba.getJdbcDriver()).newInstance();
         conn = DriverManager.getConnection(dba.getJdbcUrl(),
                                            dba.getJdbcUser(),
                                            dba.getJdbcPassword());
       } catch (ClassNotFoundException e){
         throw new InboxListenerException("Error connecting DB (Class Not Found): (" +
                                e.getMessage() + ")");
       } catch (InstantiationException e){
         throw new InboxListenerException("Error connecting DB (Instantiation Exception): (" +
                                e.getMessage() + ")");
       } catch (IllegalAccessException e){
         throw new InboxListenerException("Error connecting DB (Illegal Access): " +
                                e.getMessage() + ")");
       } catch (SQLException e){
         throw new InboxListenerException("Error connecting DB (SQLException): (" +
                                e.getMessage() + ")" );
       }
       
       return conn;

  }

    
  /**
   *
   *
   * @return Connection
   * @throws SeccException
   */
  public void releaseConnection(ResultSet rs, 
                                PreparedStatement ps) 
    throws InboxListenerException {

         try {
             if (rs != null) {
                 rs.close();
             }
             if (ps != null) {
                 ps.close();
             }
             if (conn != null) {
                 conn.close();
             }
         } catch (SQLException e){
             log.severe("Error closing DB " + e.getMessage());
             throw new InboxListenerException("Error closing DB (" +
                                              e.getMessage() + ")" );
         }
         
  }

  /**
   *
   *
   * @return Connection
   * @throws SeccException
   */
  public void releaseConnection(PreparedStatement ps) 
    throws InboxListenerException {

         try {
             if (ps != null) {
                 ps.close();
             }
             if (conn != null) {
                 conn.close();
             }
         } catch (SQLException e){
             log.severe("Error closing DB " + e.getMessage());
             throw new InboxListenerException("Error closing DB (" +
                                              e.getMessage() + ")" );
         }
         
  }
  
  /**
   *
   *
   * @return Connection
   * @throws SeccException
   */
  public void releaseConnection(PreparedStatement ps1, PreparedStatement ps2) 
    throws InboxListenerException {

         try {
             if (ps1 != null) {
                 ps1.close();
             }
             if (ps2 != null) {
                 ps2.close();
             }
             if (conn != null) {
                 conn.close();
             }
         } catch (SQLException e){
             log.severe("Error closing DB " + e.getMessage());
             throw new InboxListenerException("Error closing DB (" +
                                              e.getMessage() + ")" );
         }
         
  }

}
