/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.inboxlistener.engine;


import com.funambol.email.exception.InboxListenerException;
import com.funambol.email.inboxlistener.model.DBAccount;
import com.funambol.email.inboxlistener.model.MSAccount;
import com.funambol.email.inboxlistener.model.MailboxAccount;
import com.funambol.email.inboxlistener.model.WSAccount;
import java.util.List;
import java.util.ArrayList;
import java.util.logging.Logger;
import java.util.logging.Level;

import com.funambol.email.util.Def;


/**
 *
 *
 * @version $Id: ListenerRegistry.java,v 1.7 2006/11/14 15:34:45 gbmiglia Exp $
 */
public class ListenerRegistry extends Thread {

  //------------------------------------------------------------------ CONSTANTS

  private Logger log = Logger.getLogger(Def.LOGGER_NAME);

  //----------------------------------------------------------------- PROPERTIES

  /** list wtih all account to monitor  */
  private List accountList = new ArrayList() ;


  /**   */
  private DBAccount dba = null;

  /**   */
  private MSAccount msa = null;

  /**   */
  private WSAccount wsa = null;

  /**   */
  private String listenerId = null;

  /**   */
  private boolean startupNotification = false;

  /**   */
  private ListenerRegistryDAO lrdao = null;

  /** to stop the thread  */
  private boolean go = true ;


  //--------------------------------------------------------------- CONSTRUCTORS

  /**
   *
   */
  public ListenerRegistry(String listenerId,
                          boolean startupNotification,
                          DBAccount dba,
                          MSAccount msa,
                          WSAccount wsa) {
     this.listenerId          = listenerId;
     this.startupNotification = startupNotification;
     this.dba                 = dba;
     this.msa                 = msa;
     this.wsa                 = wsa;
     this.lrdao               = new ListenerRegistryDAO(dba);
  }


  //------------------------------------------------------------- PUBLIC METHODS

  /**
   *
   *
   */
  public void run() {

     if (log.isLoggable(Level.FINEST)) {
        log.finest("Run ListenerRegistry " );
     }

     while (go) {

        if (log.isLoggable(Level.FINEST)) {
           log.finest("Read Accounts");
        }

        // get list
        List accountListNew = null;
        try {
           accountListNew = lrdao.getAccounts(listenerId);
        } catch (Exception e){
           e.printStackTrace();
        }

        List accountListOld = this.accountList;

        if (accountListNew != null){
           startMailboxListener(accountListNew,accountListOld);
        }

        // sleep for X secs
        try{
           sleep(dba.getReloadingTime());
        } catch (Exception e){
           e.printStackTrace();
        }

     }

  }

  /**
   * close all the MailboxManager before closing the 
   * Inbox-Listener Application
   */
   public void closeMailboxManager() throws InboxListenerException {
       
       if (log.isLoggable(Level.FINEST)) {
          log.finest("Start Closing all the MailboxManager");
       }
       
       String userNew = null;
       MailboxManager mm = null;
       for (int i=0; i<accountList.size(); i++){
            userNew = ((MailboxAccount) accountList.get(i)).getUser();
            mm = getUserListener(userNew);
            if (mm != null){
                try {
                    mm.closeMailServerConnection();
                } catch (InboxListenerException e) {
                    throw new InboxListenerException(e.getMessage());
                }     
            }            
       }  
       
   }  


  //------------------------------------------------------------ PRIVATE METHODS

  /**
   * 
   *
   */
  private void startMailboxListener(List accountListNew,
                                    List accountListOld){

      String userOld        = null;
      String userNew        = null;
      int    maxNew         = Def.MAX_INBOX_EMAIL_NUMBER;
      String pwdNew         = null;
      MailboxManager mm     = null;
      MailboxAccount mbaNew = null;
      
      // remove listener for a deleted user
      for (int i=0; i<accountListOld.size(); i++){
          userOld = ((MailboxAccount) accountListOld.get(i)).getUser();
          if (toRemove(userOld,accountListNew)){
             if (log.isLoggable(Level.FINEST)) {
                log.finest("Removing Listener for User " + userOld);
             }
             // stop mailbox listener
             mm = getUserListener(userOld);
             mm.stopThread();
          }
      }

      // add listener for a new user
      for (int i=0; i<accountListNew.size(); i++){
          mbaNew   = (MailboxAccount) accountListNew.get(i);
          userNew = mbaNew.getUser();
          if (toAdd(userNew,accountListOld)){
             if (log.isLoggable(Level.FINEST)) {
                log.finest("Adding Listener for User " + userNew );
             }
             // start mailbox listener
             mm = new MailboxManager(this.startupNotification,
                                     mbaNew, this.msa, this.dba, this.wsa);
             mm.setName(userNew);
             mm.start();
          }
      }

      // update a listener
      for (int i=0; i<accountListNew.size(); i++){
          userNew = ((MailboxAccount) accountListNew.get(i)).getUser();
          maxNew  = ((MailboxAccount) accountListNew.get(i)).getMaxEmailNumber();
          pwdNew  = ((MailboxAccount) accountListNew.get(i)).getPassword();
          if (toUpdate(userNew, accountListOld)){
             if (log.isLoggable(Level.FINEST)) {
                log.finest("Updating Listener for User " + userNew);
             }
             // update mailbox listener
             mm = getUserListener(userNew);
             mm.setMaxEmailNumber(maxNew);
             mm.setPassword(pwdNew);
          }
      }      
      
      
      // replace the new account list
      accountList.clear();
      for (int i=0; i<accountListNew.size(); i++){
         mbaNew = (MailboxAccount) accountListNew.get(i);
         accountList.add(mbaNew);
      }


  }


  /**
   *
   * checks if the username is already present in the
   * account list to monitor.
   *
   * to remove
   * ---------
   * user is the Old user
   * accounts is the New List
   * if the username is not in the list it retrun true, the
   * listener on the account must be deregistered
   *
   *
   */
  private boolean toRemove(String userOld, List accountsNew){
     String         userNew           = null;
     boolean        isAlreadyRegister = false;
     boolean        toRemove          = true;

     for (int i=0; i<accountsNew.size(); i++){
        userNew = ((MailboxAccount) accountsNew.get(i)).getUser();
        if (userNew.equals(userOld)){
           isAlreadyRegister = true;
           break;
        }
     }

     if (isAlreadyRegister){
        toRemove = false;
     }

     return toRemove;
  }

  /**
   *
   * checks if the username is already present in the
   * account list to monitor.
   * @param user user to check
   * @param accounts is the Old List
   * @return return true if if the username is not in the old account list
   *
   */
  private boolean toAdd(String user, List accountsOld){
     String         userOld           = null;
     boolean        isAlreadyRegister = false;
     boolean        toAdd             = true;

     for (int i=0; i<accountsOld.size(); i++){
        userOld = ((MailboxAccount) accountsOld.get(i)).getUser();
        if (userOld.equals(user)){
           isAlreadyRegister = true;
           break;
        }
     }

     if (isAlreadyRegister){
        toAdd = false;
     }
     return toAdd;
  }

  /**
   *
   * checks if the username is already present in the
   * account list to monitor.
   * @param user user to check
   * @param accounts is the Old List
   * @return return true if if the username is not in the old account list
   *
   */
  private boolean toUpdate(String user, List accountsOld){
     String         userOld           = null;
     boolean        isAlreadyRegister = false;
     boolean        toUpdate          = false;

     for (int i=0; i<accountsOld.size(); i++){
        userOld = ((MailboxAccount) accountsOld.get(i)).getUser();
        if (userOld.equals(user)){
           isAlreadyRegister = true;
           break;
        }
     }

     if (isAlreadyRegister){
        toUpdate = true;
     }
     return toUpdate;
  }  
  
  
  /**
   * returns the listener for a specific user
   */
  private MailboxManager getUserListener(String user){

     MailboxManager mm = null;
     Thread tobj       = null;
     String tname      = null;
     
     int activeCount = Thread.activeCount();
     ThreadGroup tg = Thread.currentThread().getThreadGroup();
     
     Thread[] list = new Thread[activeCount];     
     tg.enumerate(list);
     
     for (int i=0; i<list.length; i++){
         tobj = (Thread)list[i];
         if (tobj != null){
            tname = tobj.getName();
            if ( tname.startsWith(user) ){
               mm = (MailboxManager)tobj; 
               break;
            }
         }
     }

     return mm;
  }

}
