/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.inboxlistener.engine;


import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;
import java.sql.PreparedStatement;
import java.sql.Connection;
import java.sql.ResultSet;

import com.funambol.email.util.Def;
import com.funambol.email.exception.InboxListenerException;
import com.funambol.email.inboxlistener.model.DBAccount;
import com.funambol.email.inboxlistener.model.MailboxAccount;
import com.funambol.email.util.Query;


/**
 *
 * Contains methods to access to the tables with the
 * information about Mail Server Account.
 *
 * @version $Id: ListenerRegistryDAO.java,v 1.7 2006/11/14 15:34:45 gbmiglia Exp $
 */
public class ListenerRegistryDAO {

  // --------------------------------------------------------------- Constants

  private Logger log = Logger.getLogger(Def.LOGGER_NAME);


  // ------------------------------------------------------------ Private data

  /**   */
  private DBAccount dba   = null;
  /**   */
  private DBTools   dbtools   = null;


  // ------------------------------------------------------------- Constructor

  /**
   *
   */
  public ListenerRegistryDAO(DBAccount _dba) {
     this.dba = _dba;
     this.dbtools = new DBTools(_dba);
  }


  // ---------------------------------------------------------- Public Methods



  /**
   * Gets Accounts
   * returns a list that contains MailAccount
   *
   */
  public List getAccounts (String listenerId)
    throws InboxListenerException {

         Connection connection = null;
         PreparedStatement ps  = null;
         ResultSet rs          = null;
         List accounts         = null;

         String  username     = null;
         String  password     = null;
                  
         String  listenerID   = null;
         
         int maxNumEmail      = Def.MAX_INBOX_EMAIL_NUMBER;
         String  maxNumEmailS = ""+Def.MAX_INBOX_EMAIL_NUMBER;

         try {

             connection = dbtools.getConnection();

             ps = connection.prepareStatement(Query.ACCOUNT_SELECT_LISTENER_ACCOUNT);

             ps.setString(1, listenerId);
             ps.setString(2, "y");

             rs = ps.executeQuery();


             MailboxAccount ma = null;
             accounts          = new ArrayList();

             while(rs.next()) {
                 username     = rs.getString(1);
                 password     = rs.getString(2);
                 listenerID   = rs.getString(3);
                 maxNumEmailS = rs.getString(4);
                 if (maxNumEmailS != null && !maxNumEmailS.equals("")){
                     maxNumEmail = Integer.parseInt(maxNumEmailS);
                 }
                 
                 ma = new MailboxAccount(username,
                                         password,
                                         true,
                                         listenerID,
                                         maxNumEmail);

                 accounts.add(ma);
             }

         } catch (InboxListenerException e) {
             throw new InboxListenerException("Error getting Accounts from DB (" +
                                     e.getMessage() + ")");
         } catch (Exception e) {
             throw new InboxListenerException("Generic Error", e);
         } finally {
             try {
                 dbtools.releaseConnection(rs, ps);
             } catch (InboxListenerException e){
                 throw new InboxListenerException("Error getting Accounts from DB (" +
                                                  e.getMessage() + ")" );
             }
         }

         return accounts;
         
  }

}
