/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.inboxlistener.engine;


import com.funambol.email.exception.EmailAccessException;
import com.funambol.email.exception.InboxListenerException;
import com.funambol.email.inboxlistener.model.MSAccount;
import com.funambol.email.inboxlistener.model.MailboxAccount;
import com.funambol.email.items.model.DefaultFolder;
import com.funambol.email.transport.IMailServerWrapper;
import com.funambol.email.transport.ImapMailServerWrapper;
import com.funambol.email.transport.PopMailServerWrapper;
import com.funambol.email.util.Def;
import java.util.logging.Logger;
import javax.mail.Folder;
import javax.mail.MessagingException;



/**
 * This class groups some utility methods for Mail Server invoking
 *
 * @version $Id: MSTools.java,v 1.2.2.1 2006/12/06 13:36:01 gbmiglia Exp $
 */
public class MSTools {
    
   // --------------------------------------------------------------- Constants

   private Logger log = Logger.getLogger(Def.LOGGER_NAME);

   // ------------------------------------------------------------ Private data

   /**    */
   private MSAccount msa = null;

   /**    */
   private MailboxAccount mba = null;

   /**    */
   private Folder  inbox     = null;
    
    // ------------------------------------------------------------ Constructors

   /**
     * Creates a new MSTools with the given MSAccount
     * 
     * 
     * 
     * @param _msa the DBAccount
     */
   public MSTools(MSAccount _msa, MailboxAccount _mba) {
        this.msa = _msa;
        this.mba = _mba;
   }

   // ---------------------------------------------------------- Public Methods
    
   /**
    * open the Mail Server connection; Note the IL Tool
    * works only the INBOX folder
    * @param mswf reference to the mail server wrapper IMailServerWrapper
    * @param inbox reference to inbox Folder
    */
   public void openConnection(IMailServerWrapper mswf) throws InboxListenerException {
        
         try {
              DefaultFolder df = new DefaultFolder();
              
              df.setInboxName(Def.FOLDER_INBOX_ENG);
              df.setInboxId(Def.FOLDER_INBOX_ID);
                
              mswf.openConnection(mswf.getServerType(), msa.getHost(),
                                  "25", false, msa.getHost(), msa.getPort()+"",
                                  df, mba.getUser(), mba.getPassword(),
                                  msa.getIsSSL(), msa.getKeystorePath(),
                                  msa.getKeystorePassphrase());
                
              if (mswf instanceof ImapMailServerWrapper){
                  // imap
                  ImapMailServerWrapper imsw = (ImapMailServerWrapper) mswf ;
                  this.inbox = (Folder) imsw.folderInboxOpened;
              } else {
                  // pop3
                  PopMailServerWrapper  pmsw = (PopMailServerWrapper) mswf ;
                  this.inbox = (Folder) pmsw.folderInboxOpened;
              }
                           
              if (!this.inbox.isOpen()) {
                  this.inbox.open(Folder.READ_ONLY);
              }
                                
         } catch (MessagingException e) {
             throw new InboxListenerException(
                     "Error opening the Store " + e.getMessage());
         } catch (EmailAccessException e) {
             throw new InboxListenerException(
                     "Error opening the Mail Server Connection " + e.getMessage());
         }
        
   }
    
   /**
    * close the Mail Server connection; Note the IL Tool
    * works only the INBOX folder
    * @param mswf reference to the mail server wrapper IMailServerWrapper
    * @param inbox reference to inbox Folder
    */
   public void closeConnection(IMailServerWrapper mswf) throws InboxListenerException {
         try {
              if (this.inbox != null){
                  if (this.inbox.isOpen()){
                     this.inbox.close(true);
                  }
              }
              if (mswf != null){
                 mswf.releaseConnection();
              }
         } catch (MessagingException e) {
             throw new InboxListenerException(
                     "Error closing the Folder " + e.getMessage());
         } catch (EmailAccessException e) {
             throw new InboxListenerException(
                     "Error closing the Mail Server Connection " + e.getMessage());
         }
    }
    
    
  /**
   * return the inbox folder
   */
   public Folder getInbox() {
       return this.inbox;
   }
  
   
}
