/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.inboxlistener.engine;


import com.funambol.email.engine.cache.CrcSyncItemInfo;
import java.util.logging.Logger;
import java.sql.PreparedStatement;
import java.sql.Connection;
import java.sql.ResultSet;

import com.funambol.email.util.Def;
import com.funambol.email.exception.InboxListenerException;
import com.funambol.email.inboxlistener.model.DBAccount;
import com.funambol.email.inboxlistener.model.MSAccount;
import com.funambol.email.inboxlistener.model.MailboxAccount;
import com.funambol.email.util.Query;
import com.funambol.email.util.Utility;
import java.sql.SQLException;
import java.util.Date;
import java.util.HashMap;


/**
 * Contains methods to access to the tables with the
 * information about Mail Server Mailbox.
 *
 * @version $Id: MailboxDBDAO.java,v 1.7 2006/11/14 08:06:44 gbmiglia Exp $
 */
public class MailboxDBDAO {

  // --------------------------------------------------------------- Constants

  private Logger log = Logger.getLogger(Def.LOGGER_NAME);


  // ------------------------------------------------------------ Private data

  /**   */
  private DBAccount dba       = null;

  /**   */
  private MailboxAccount mba  = null;

  /**   */
  private MSAccount msa  = null;

  /**   */
  private DBTools   dbtools   = null;


  // ------------------------------------------------------------- Constructor

  /**
   *
   */
  public MailboxDBDAO(DBAccount _dba, MailboxAccount _mba, MSAccount _msa) {
     this.dba = _dba;
     this.mba = _mba;
     this.msa = _msa;
     this.dbtools = new DBTools(_dba);
  }

  // ---------------------------------------------------------- Public Methods

  /**
   *
   * Set the ServerItems into DB
   *
   */
  public void setItems (CrcSyncItemInfo[] infos)
    throws InboxListenerException {

         Connection connection = null;
         PreparedStatement ps  = null;
         ResultSet rs          = null;

         CrcSyncItemInfo syncItemInfo = null;
         String          GUID         = null;
         long            lastCRC      = 0;
         String          messageID    = null;
         Date            headerDate   = null;
         String          headerDateS  = null;
         String          isInvalid    = null;
         String          isInternal   = null;

         try {

             connection = this.dbtools.getConnection();

             connection.setAutoCommit(false);

             // remove data
             //System.out.println("**************remove data to db");
             ps = connection.prepareStatement(Query.INBOX_DELETE_MAILBOX_INFO);
             ps.setString  (1, this.mba.getUser()) ;
             ps.setString  (2, this.msa.getProtocol());
             ps.executeUpdate();

             // for testing purpose
             //System.out.println("**************wait 10 secs");
             //try {
             //  Thread.currentThread().sleep(10000);
             //} catch (Exception e) {
             //}

             // insert data
             //System.out.println("**************insert data to db");
             ps = connection.prepareStatement(Query.INBOX_INSERT_MAILBOX_INFO);
             for (int i=0; i<infos.length; i++ ){
                 syncItemInfo = infos[i];
                 GUID        = syncItemInfo.getKey().getKeyAsString();
                 lastCRC     = syncItemInfo.getLastCrc();
                 messageID   = syncItemInfo.getMessageID();
                 headerDate  = syncItemInfo.getHeaderDate();
                 if (headerDate != null){
                     headerDateS = Utility.UtcFromDate(headerDate);
                 }
                 isInvalid   = syncItemInfo.getInvalid();
                 isInternal  = syncItemInfo.getInternal();

                 ps.setString(1, this.mba.getUser());
                 ps.setString(2, this.msa.getProtocol());
                 ps.setString(3, GUID);
                 ps.setLong  (4, lastCRC);
                 ps.setString(5, isInvalid);
                 ps.setString(6, isInternal);
                 ps.setString(7, messageID);
                 ps.setString(8, headerDateS);

                 ps.executeUpdate();
             }

             connection.commit();

             //System.out.println("**************end transaction");

         } catch (InboxListenerException e) {
             throw new InboxListenerException("Error setting ServerItems into DataBase (" +
                                     e.getMessage() + ")");
         } catch (SQLException sqle) {
             try {
                connection.rollback();
             } catch (SQLException sqlee){
                throw new InboxListenerException("Error rollbacking", sqlee);
             }
             throw new InboxListenerException("Error refreshing Local Items", sqle);
         } finally {
             try {
                 this.dbtools.releaseConnection(rs, ps);
             } catch (InboxListenerException e){
                 throw new InboxListenerException("Error setting ServerItems into DataBase (" +
                                                  e.getMessage() + ")" );
             }
         }

   }

  /**
   *
   *
   */
  public boolean hasNewMessages(CrcSyncItemInfo[] serverInfos,
                                HashMap localInfos)
    throws InboxListenerException {

        boolean hasNewEmail    = false;
        String  idFromMS       = null;
        String  GUIDFromMS     = null;
        String  idFromDB       = null;
        String  syncLabel      = null;
        CrcSyncItemInfo lcrcii = null;

        try {

            if (localInfos.size() == 0 && serverInfos.length > 0){

                return true;

            }
            else if (localInfos.size() < serverInfos.length){

                return true;

            } else {

                for (int i=0; i<serverInfos.length; i++){

                    hasNewEmail = false;

                    GUIDFromMS  = serverInfos[i].getKey().getKeyAsString();
                    //idFromMS    = serverInfos[i].getMessageID();
                    syncLabel   = serverInfos[i].getInternal();

                    if (syncLabel == null) {  // external email

                       if (GUIDFromMS != null){

                            lcrcii = (CrcSyncItemInfo)localInfos.get(GUIDFromMS);
                            if (lcrcii == null){
                                hasNewEmail = true;
                                break;
                            }

                       }

                   }

                }

            }

        } catch (Exception me){
            throw new InboxListenerException("Error checking new messages", me);
        }

        return hasNewEmail;
    }


  /**
   *
   * Get the Items from DB
   *
   * @return map with all local CrcSyncItemInfo
   *
   */
  public HashMap getItems () throws InboxListenerException {

         HashMap items         = new HashMap();
         Connection connection = null;
         PreparedStatement ps  = null;
         ResultSet rs          = null;
         CrcSyncItemInfo syncItemInfo = null;
         String id         = null;
         long last_crc     = 0;
         String isInvalid  = null;
         String isInternal = null;
         String messageid  = null;
         Date headerdate   = null;

         try {

             connection = this.dbtools.getConnection();

             ps = connection.prepareStatement(Query.INBOX_SELECT_MAILBOX_INFO);

             ps.setString(1, this.mba.getUser());
             ps.setString(2, this.msa.getProtocol());

             rs = ps.executeQuery();


             while(rs.next()) {

                 id         = rs.getString(1);
                 last_crc   = rs.getLong(2);
                 isInvalid  = rs.getString(3);
                 isInternal = rs.getString(4);
                 messageid  = rs.getString(5);
                 headerdate = Utility.UtcToDate(rs.getString(6));

                 items.put(id, new CrcSyncItemInfo(id, last_crc,
                                                   messageid, headerdate, null,
                                                   isInvalid, isInternal, "Y", null));
             }

         } catch (InboxListenerException e) {
             throw new InboxListenerException("Error setting ServerItems into DataBase (" +
                                     e.getMessage() + ")");
         } catch (Exception e) {
             throw new InboxListenerException("Generic Error", e);
         } finally {
             try {
                 this.dbtools.releaseConnection(rs, ps);
             } catch (InboxListenerException e){
                 throw new InboxListenerException("Error setting ServerItems into DataBase (" +
                                                  e.getMessage() + ")" );
             }
         }

         return items;

   }

}
