/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.inboxlistener.engine;


import com.funambol.email.engine.cache.CrcSyncItemInfo;
import com.funambol.email.exception.InboxListenerException;
import com.funambol.email.inboxlistener.model.MSAccount;
import com.funambol.email.inboxlistener.model.MailboxAccount;
import com.funambol.email.transport.IMailServerWrapper;
import com.funambol.email.util.Def;
import com.funambol.email.util.Utility;
import com.funambol.email.util.UtilityDate;
import com.sun.mail.imap.IMAPFolder;
import com.sun.mail.pop3.POP3Folder;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.logging.Logger;
import java.util.logging.Level;
import java.util.List;
import javax.mail.Message;
import javax.mail.MessagingException;



/**
 * This class groups some utility methods for Mail Server invoking
 *
 * @version $Id: MailboxMSDAO.java,v 1.7.2.1 2006/11/24 10:18:30 gbmiglia Exp $
 */
public class MailboxMSDAO {

   // --------------------------------------------------------------- Constants

   private Logger log = Logger.getLogger(Def.LOGGER_NAME);

   // ------------------------------------------------------------ Private data

   /**    */
   private MSAccount msa = null;

   /**    */
   private MailboxAccount mba = null;

   /**    */
   private MSTools mstools = null;

   // ------------------------------------------------------------ Constructors

   /**
     * Creates a new MSTools with the given MSAccount
     *
     *
     *
     * @param _msa the DBAccount
     */
   public MailboxMSDAO(MSAccount _msa, MailboxAccount _mba) {
        this.msa = _msa;
        this.mba = _mba;
        this.mstools = new MSTools(_msa, _mba);

   }

   // ---------------------------------------------------------- Public Methods

   /**
    *
    */
   public void setMaxEmailNumber(int _maxEmailNumber){
       this.mba.setMaxEmailNumber(_maxEmailNumber);
   }

   /**
    *
    */
   public void setPassword(String _password){
       this.mba.setPassword(_password);
   }


    /**
     * get messages from inbox folder
     * This method returns all the items in the server
     * inbox folder
     *
     * @param mswf IMailServerWrapper
     * @param localInfos map with all the loacal CrcSyncItemInfo
     * @return CrcSyncItemInfo[]
     * @throws EntityException
     */
    public CrcSyncItemInfo[] getEmailsFromInbox(IMailServerWrapper mswf,
                                                HashMap localInfos)
      throws InboxListenerException {

        CrcSyncItemInfo syncItemInfo    = null;
        ArrayList       allSyncItemInfo = new ArrayList();


        try {

            mstools.openConnection (mswf);

            int max = mba.getMaxEmailNumber();

            // get max Messages
            Message[] messages = getMessages(max);

            // create list with all SyncItemInfo
            if (messages != null) {
                allSyncItemInfo = this.createInfoList(messages, localInfos);
            }

            mstools.closeConnection(mswf);

        } catch (MessagingException e) {
            throw new InboxListenerException(e.getMessage());
        }

        return (CrcSyncItemInfo[]) allSyncItemInfo.toArray(new CrcSyncItemInfo[0]);

    }

    /**
     *  close the Mail Server Connection
     */
    public void closeConnection(IMailServerWrapper mswf)
      throws InboxListenerException {
         mstools.closeConnection(mswf);
    }
    
    //-----------------------------------------------------------Private Methods

    /**
     *
     */
    private Message[] getMessages(int max) 
      throws MessagingException {
        
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("get Messages from Inbox Mailbox number : " + max);
        }
        
        Message[] messages = null;
        
        int mNumber  = this.mstools.getInbox().getMessageCount();
        
        if (mNumber > max){
            if (log.isLoggable(Level.FINEST)) {
                log.finest("get "+max+" Messages from Inbox Mailbox");
            }
            messages = this.mstools.getInbox().getMessages((mNumber-(max-1)),mNumber);            
        } else {
            if (log.isLoggable(Level.FINEST)) {
                log.finest("get all Messages from Inbox Mailbox");
            }
            messages = this.mstools.getInbox().getMessages();
        }
        
        return messages;
    }

    /**
     *
     */
    private ArrayList createInfoList(Message[] messages, HashMap localInfos) 
    throws MessagingException {
        
        ArrayList       allSyncItemInfo = new ArrayList();
        
        CrcSyncItemInfo lcrcii          = null;       
        CrcSyncItemInfo syncItemInfo    = null;       
        
        String          GUID            = null;
        long            lastCRC         = 0;
        String          messageID       = null;
        Date            headerDate      = null;
        long            lastCRCold      = 0;
        String          messageIDold    = null;
        Date            headerDateold   = null;
        
        for (int i = 0; i < messages.length; i++) {

            GUID = this.createGUID(messages[i]);

            if (GUID != null){

                // save the old value
                lcrcii = (CrcSyncItemInfo)localInfos.get(GUID);
                if (lcrcii != null){
                    messageIDold  = lcrcii.getMessageID();
                    headerDateold = lcrcii.getHeaderDate();
                    lastCRCold    = lcrcii.getLastCrc();
                }

                // create the new values
                try {
                    messageID  = Utility.getHeaderMessageID(messages[i]);
                    headerDate = UtilityDate.getHeaderDate(messages[i],null);
                    lastCRC    = Utility.getLastCrc(messages[i], messageID, ""+headerDate);
                } catch (Exception e){
                    log.severe("Error parsing creating new SyncItemInfo for Caching System "
                               + e.getMessage());
                    messageID  = messageIDold;
                    headerDate = headerDateold;
                    lastCRC    = lastCRCold;
                }

                // insert in the list 
                if (!isMessageIDPresent(messageID, allSyncItemInfo)){
                    syncItemInfo = new CrcSyncItemInfo(GUID, lastCRC,
                                                       messageID, headerDate, null, 
                                                       null, null, "Y", null);
                    allSyncItemInfo.add(syncItemInfo);                                
                }


            }

        }
         
        return allSyncItemInfo; 
        
    }

    
    /**
     *
     */
    private String createGUID(Message msg) throws MessagingException {

        String GUID = null;
        String FMID = "";
        String UIDV = "";

        if (this.msa.getProtocol().equals(Def.PROTOCOL_IMAP)){
           FMID = ""+((IMAPFolder)this.mstools.getInbox()).getUID(msg);
           UIDV = ""+((IMAPFolder)this.mstools.getInbox()).getUIDValidity();
           GUID = Utility.createIMAPGUID(Def.FOLDER_INBOX_ID, FMID, UIDV);
        } else {
           FMID = ((POP3Folder)this.mstools.getInbox()).getUID(msg);
           GUID = Utility.createPOPGUID(Def.FOLDER_INBOX_ID, FMID);
        }
        return GUID;
    }
    
    /**
     *
     */
    private boolean isMessageIDPresent(String messageID, 
                                       List allSyncItemInfo) {
        
        String messageIDTmp = null;
        boolean check = false;
        
        if (messageID != null) {
            for (int i=0; i<allSyncItemInfo.size(); i++){
                messageIDTmp = ((CrcSyncItemInfo)allSyncItemInfo.get(i)).getMessageID();
                if (messageID.equals(messageIDTmp)){
                    check = true;
                    break;
                }
            }
        }        
        
        return check;        
    }
       
}
