/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.inboxlistener.engine;


import com.funambol.email.engine.cache.CrcSyncItemInfo;
import java.util.List;
import java.util.ArrayList;
import java.util.logging.Logger;
import java.util.logging.Level;

import com.funambol.email.transport.IMailServerWrapper;
import com.funambol.email.transport.MailServerWrapperFactory;

import com.funambol.email.util.Def;
import com.funambol.email.exception.InboxListenerException;
import com.funambol.email.inboxlistener.model.DBAccount;
import com.funambol.email.inboxlistener.model.MSAccount;
import com.funambol.email.inboxlistener.model.MailboxAccount;
import com.funambol.email.inboxlistener.model.WSAccount;
import java.util.Date;
import java.util.HashMap;

/**
 *
 *
 * @version $Id: MailboxManager.java,v 1.5 2006/11/10 10:13:58 gbmiglia Exp $
 */
public class MailboxManager extends Thread {
    
    //------------------------------------------------------------------ CONSTANTS
    
    private Logger log = Logger.getLogger(Def.LOGGER_NAME);
    
    //----------------------------------------------------------------- PROPERTIES
    
    /** owner of the thread */
    private List messageIDs = new ArrayList();
    
    /**  */
    private Date lastReceivedDate = null;
    
    /**   */
    private boolean startupNotification = false ;
    
    /** owner of the thread */
    private MailboxAccount mba;
    
    /** mail server coords */
    private MSAccount msa;
    
    /** db coords */
    private DBAccount dba;
    
    /** webservices coords */
    private WSAccount wsa;
    
    /** to stop the thread  */
    private boolean go = true ;
    
    /**   */
    private IMailServerWrapper mswf = null;

    /**   */
    private MailboxMSDAO mmsdao = null;
    
    /**   */
    private MailboxDBDAO mdbdao = null;
    
    
    //--------------------------------------------------------------- CONSTRUCTORS
    
    /**
     *
     */
    public MailboxManager(boolean startupNotification,
                          MailboxAccount _mba,
                          MSAccount _msa,
                          DBAccount _dba,
                          WSAccount _wsa) {
        this.startupNotification = startupNotification;
        this.mba                 = _mba;
        this.msa                 = _msa;
        this.dba                 = _dba;
        this.wsa                 = _wsa;
    }
    
    
    //------------------------------------------------------------- PUBLIC METHODS
    
    /**
     *
     *
     * this thread is not always connected to the mailbox.
     * every X seconds (interval) it starts a connection and read the ids
     * in the inbox folder; then compares the previous time read ids.
     * If there are new mails call the Sync4j AdminManager web service
     * in order to trigger the push message
     *
     *
     */
    public void run() {
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("START NEW THREAD for : " + mba.toString() +
                    " interval : " + msa.getInterval() +
                    " protocol : " + msa.getProtocol() +
                    " startupNotification : " + startupNotification );
        }
        
        
        boolean hasNew    = false;
        int     firstLoop = 1;        
        long    timeStart = 0;
        long    timeStop  = 0;
        long    totTimeStart = 0;
        long    totTimeStop  = 0;
        
        mswf = MailServerWrapperFactory.getMailServerWrapper(msa.getProtocol());
                
        mmsdao = new MailboxMSDAO(this.msa, this.mba);
        mdbdao = new MailboxDBDAO(this.dba, this.mba, this.msa);
                
        while (go) {

            // get info
            totTimeStart = System.currentTimeMillis();
            try {
                    
                // myabe the password or the max email number are changed
                mmsdao.setMaxEmailNumber(this.mba.getMaxEmailNumber());
                mmsdao.setPassword(this.mba.getPassword());
                
                // get local info; map with (GUID, CrcSyncItemInfo)
                HashMap localInfos = mdbdao.getItems();
                
                // get info in Mail Server
                CrcSyncItemInfo[] serverInfos = mmsdao.getEmailsFromInbox(mswf, 
                                                                          localInfos);
                
                // check new email
                hasNew = mdbdao.hasNewMessages(serverInfos,localInfos);
                
                // set info in DB
                mdbdao.setItems(serverInfos);                
                
            } catch (InboxListenerException e) {
                e.printStackTrace();
            }
            totTimeStop = System.currentTimeMillis();
            if (log.isLoggable(Level.FINEST)) {
                log.finest("create Local Cache "+mba.getUser()+" Execution Time: " +
                                       (totTimeStop - totTimeStart) + " ms");
            }
            
            
            if (log.isLoggable(Level.FINEST)) {
                log.finest("MailboxListener for : " + mba.toString() +
                           " send Notification: " + hasNew );
            }
            
            // call notification system
            try {
                if (hasNew){
                    if (firstLoop == 1 && !startupNotification){
                        // do nothing
                    } else {
                        NotificationWrapper nw = new NotificationWrapper(mba, wsa);
                        nw.callNotification();
                    }
                }
            } catch (Exception e){
                e.printStackTrace();
            }
            
            hasNew    = false;
            firstLoop = 2;
            
            // wait the next polling time
            try{
                sleep(msa.getInterval());
            } catch (Exception e){
                e.printStackTrace();
            }
                        
        }
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("STOP EXECUTION THREAD for : " + mba.toString() );
        }
                
    }    
    
    /**
     *
     */
    public void setMaxEmailNumber(int _maxEmailNumber){
        this.mba.setMaxEmailNumber(_maxEmailNumber);
    }
    
    /**
     *
     */
    public void setPassword(String _password){
        this.mba.setPassword(_password);
    }
    
    /**
     * close the Mail Server Connection
     */
    public void closeMailServerConnection() throws InboxListenerException {
        if (log.isLoggable(Level.FINEST)) {
            log.finest("Shutting down MailboxListener for : " + mba.toString() );
        }
        mmsdao.closeConnection(mswf);
    }
           
    
    /**
     *
     */
    public void stopThread(){
        if (log.isLoggable(Level.FINEST)) {
            log.finest("Shutting down MailboxListener for : " + mba.toString() );
        }
        this.go = false;
    }          
    
}
