/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.inboxlistener.engine;



import java.util.logging.Logger;
import java.util.logging.Level;

import com.funambol.email.util.Def;
import com.funambol.email.exception.InboxListenerConfigException;
import com.funambol.email.exception.InboxListenerException;
import com.funambol.email.inboxlistener.admin.InboxListenerConfig;
import com.funambol.email.inboxlistener.model.DBAccount;
import com.funambol.email.inboxlistener.model.MSAccount;
import com.funambol.email.inboxlistener.model.WSAccount;
import com.funambol.email.util.UtilityIL;

/**
 *
 *
 * @version $Id: MessageProcessor.java,v 1.3 2006/11/14 09:21:54 gbmiglia Exp $
 */

public class MessageProcessor
{

  //------------------------------------------------------------------ CONSTANTS

  private static Logger log = Logger.getLogger(Def.LOGGER_NAME);

  /**    */
  private static ListenerRegistry lr = null;
  
  //--------------------------------------------------------------- CONSTRUCTORS

  public MessageProcessor(){
  }

  //------------------------------------------------------------- PUBLIC METHODS

  /**
   * starting method
   *
   * @param args String[]
   */
  public static void main(String args[]) {

     // load the listener configuration
     String  listenerId          = null;
     boolean startupNotification = false;
     int    interval             = 0;
     try {
        listenerId          = InboxListenerConfig.getConfigInstance().getListenerId();
        startupNotification = InboxListenerConfig.getConfigInstance().getStartupNotification();
        interval            = InboxListenerConfig.getConfigInstance().getInterval();
     } catch (InboxListenerConfigException e) {
       e.printStackTrace();
     }
     
     
     // load WebServices Properties
     WSAccount wsa       = null;
     String wsEndPoint   = null;
     String wsUsername   = null;
     String wsPassword   = null;
     String wsSyncSource = null;
     String wsSyncSourceType = null;
     try {
        wsEndPoint   = InboxListenerConfig.getConfigInstance().getWsEndPoint();
        wsUsername   = InboxListenerConfig.getConfigInstance().getWsUsername();
        wsPassword   = InboxListenerConfig.getConfigInstance().getWsPassword();
        wsSyncSource = InboxListenerConfig.getConfigInstance().getWsSyncSource();
        wsSyncSourceType = InboxListenerConfig.getConfigInstance().getWsSyncSourceType();
        wsa          = new WSAccount(wsEndPoint,
                                     wsUsername,
                                     wsPassword,
                                     wsSyncSourceType,
                                     wsSyncSource);
     } catch (InboxListenerConfigException e) {
       e.printStackTrace();
     }
     
     // load DataBase Properties
     DBAccount dba        = null;
     String dbName        = null;
     String jdbcDriver    = null;
     String jdbcUrl       = null;
     String jdbcUser      = null;
     String jdbcPassword  = null;
     int    reloadingTime = 0;
     try {
        jdbcDriver    = InboxListenerConfig.getConfigInstance().getJdbcDriver();
        jdbcUrl       = InboxListenerConfig.getConfigInstance().getJdbcUrl();
        jdbcUser      = InboxListenerConfig.getConfigInstance().getJdbcUser();
        jdbcPassword  = InboxListenerConfig.getConfigInstance().getJdbcPassword();
        reloadingTime = InboxListenerConfig.getConfigInstance().getReloadingTime();
        dba          = new DBAccount(jdbcDriver,
                                     jdbcUrl,
                                     jdbcUser,
                                     jdbcPassword,
                                     reloadingTime);
     } catch (InboxListenerConfigException e) {
       e.printStackTrace();
     }


     // load Mail Server Properties from DS-Server
     MSAccount msa      = null;
     try {
         msa = UtilityIL.getServerBean(wsa, interval);
     } catch (InboxListenerConfigException e) {
         e.printStackTrace();
     }

     if (log.isLoggable(Level.FINEST)) {
         log.finest("Start Inbox Listener "                      +
                    "\nInboxListener Parameters"                 +
                    "\nlistener id         : " + listenerId          +
                    "\nstartupNotification : " + startupNotification +
                    "\nMail Server Parameters"                   +
                    "\nHost         : " + msa.getHost()          +
                    "\nProtocol     : " + msa.getProtocol()      +
                    "\nSSL          : " + msa.getIsSSL()         +
                    "\nKeystorePath : " + msa.getKeystorePath()  +
                    "\nport         : " + msa.getPort()          +
                    "\ninterval     : " + msa.getInterval()      +
                    "\nDataBase Parameters"                      +
                    "\njdbcDriver   : " + dba.getJdbcDriver()    +
                    "\njdbcUrl      : " + dba.getJdbcUrl()       +
                    "\njdbcUser     : " + dba.getJdbcUser()      +
                    "\nreloadingTime: " + dba.getReloadingTime() +
                    "\nWebServices Parameters"                   +
                    "\nwsEndPoint   : " + wsa.getWsEndPoint()    +
                    "\nwsUsername   : " + wsa.getWsUsername()    +
                    "\nwsPassword   : " + wsa.getWsPassword()    +
                    "\nwsSyncSourceType : " + wsa.getWsSyncSourceType()  +
                    "\nwsSyncSource : " + wsa.getWsSyncSource()  );
     }

     // register the shutdown hook 
     Runtime.getRuntime().addShutdownHook(new Thread() { 
        public void run() { 
        endApp(); 
     } 
     });; 

     // start listener registry
     lr = new ListenerRegistry(listenerId, startupNotification, dba, msa, wsa);
     lr.setName("ListenerRegistry-"+listenerId);
     lr.start();

  }
  
  /**
   * close all the connection    
   */
  public static void endApp() { 
     System.out.println("Start Closing Mail Server Connection");
     try {
        lr.closeMailboxManager();
     } catch (InboxListenerException ile) {
         ile.printStackTrace();
     }
     System.out.println("End Closing Mail Server Connection");
  }     
  

}
