/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.inboxlistener.tools;


import com.funambol.email.exception.InboxListenerConfigException;
import com.funambol.email.exception.InboxListenerException;
import com.funambol.email.inboxlistener.admin.InboxListenerConfig;
import com.funambol.email.inboxlistener.engine.MSTools;
import com.funambol.email.inboxlistener.model.MSAccount;
import com.funambol.email.inboxlistener.model.MailboxAccount;
import com.funambol.email.inboxlistener.model.WSAccount;
import com.funambol.email.util.UtilityIL;
import com.funambol.email.transport.IMailServerWrapper;
import com.funambol.email.transport.MailServerWrapperFactory;
import javax.mail.Folder;
import javax.mail.MessagingException;
import com.sun.mail.imap.IMAPFolder;
import java.util.ArrayList;
import javax.mail.NoSuchProviderException;


/**
 * <p>this class implements the methods to get the Folders in a specific
 * mailbox</p>
 *
 * @version $Id: FolderManager.java,v 1.5 2006/11/14 09:22:12 gbmiglia Exp $
 */

public class FolderManager
{

    //--------------------------------------------------------------- Properties
    
    private String username      = "";
    private String password      = "";
    
    private MSAccount msa        = null;
    
        
  //--------------------------------------------------------------- Constructors
    
    public FolderManager() {
        go();
    }


    public static void main(String[] args) {
        FolderManager fm = new FolderManager();
    }

  //---------------------------------------------------------- Protected Methods
    
  /**
   *
   */
  protected void go(){

     // load WebServices Properties
     WSAccount wsa       = null;
     String wsEndPoint   = null;
     String wsUsername   = null;
     String wsPassword   = null;
     String wsSyncSource = null;
     String wsSyncSourceType = null;
     try {
        wsEndPoint   = InboxListenerConfig.getConfigInstance().getWsEndPoint();
        wsUsername   = InboxListenerConfig.getConfigInstance().getWsUsername();
        wsPassword   = InboxListenerConfig.getConfigInstance().getWsPassword();
        wsSyncSource = InboxListenerConfig.getConfigInstance().getWsSyncSource();
        wsSyncSourceType = InboxListenerConfig.getConfigInstance().getWsSyncSourceType();
        wsa          = new WSAccount(wsEndPoint,
                                     wsUsername,
                                     wsPassword,
                                     wsSyncSourceType,
                                     wsSyncSource);
     } catch (InboxListenerConfigException e) {
       e.printStackTrace();
     }
     
     // load Mail Server Properties from DS-Server
     try {
         // the interval doesn't matter
         msa = UtilityIL.getServerBean(wsa, 1);
     } catch (InboxListenerConfigException e) {
         e.printStackTrace();
     }
     
     // start menu ; set the username/password
     try {
        // clean display
        for (int p=0;p<50;p++)
          System.out.println("");

        int option = -1;

        while (true) {
          option = mainMenu();
          if (option == 0) {
            break;
          }
        }
     } catch(java.lang.Exception e) {
         e.printStackTrace();
     }     

  }              
  
  //------------------------------------------------------------ Private Methods
     
  /**
   *
   */
  private int mainMenu() {
      
    String option = null;
    int value = -1;

    try {
       System.out.println("");
       System.out.println("");
       System.out.println(" IMAP FOLDER MANAGER TOOL ");
       System.out.println("");
       System.out.println(" 0   quit");
       System.out.println(" 1   select account");
       System.out.println(" ");

       value = UtilityIL.readInteger();

       switch (value) {
        case 1: setAccount(); break;
        case 0: break;
        default: System.out.println("error");
         return 0;
       }
      } catch(java.lang.Exception e){
        e.printStackTrace();
        return 0;
      }
      return value;
   }    

  
  /**
   *
   */
  private void setAccount() throws InboxListenerException {
     try {
        System.out.print("username: ");
        username = UtilityIL.readValue();
        System.out.print("password: ");
        password = UtilityIL.readValue();
     } catch (Exception ex) {
        throw new InboxListenerException("Error setting account (" +
                                         ex.getMessage() + ")");
     }
     
     try {
        imapFolderList();
     } catch (Exception ex) {
        throw new InboxListenerException("Error getting folder (" +
                                         ex.getMessage() + ")");
     }
   
  }
  
  
  /**
   *
   */
  private void imapFolderList() throws InboxListenerException {
          
     try {
         
          
          IMailServerWrapper mswf = 
                  MailServerWrapperFactory.getMailServerWrapper(msa.getProtocol());
                   
          MailboxAccount mba = new MailboxAccount(username,
                                                  password,
                                                  true,
                                                  "1",
                                                  50);
          
          MSTools mstools = new MSTools(msa, mba);
         
          mstools.openConnection (mswf);
          
          ArrayList fs = new ArrayList();         
          IMAPFolder folder = (IMAPFolder) mswf.getStore().getDefaultFolder();
          searchSubFolder(folder, fs);
          System.out.println("IMAP Folders in the Account " + username);
          for (int i = 0; i < fs.size(); i++) {
             System.out.println(fs.get(i));
          }          
          
          mstools.closeConnection(mswf);            
                            
     } catch (NoSuchProviderException ex) {
         throw new InboxListenerException("Error opening Account - Check the protocol");
     } catch (MessagingException ex) {
         throw new InboxListenerException("Error connecting Account - Check the user");
     }
     

  }
  
  
  /**
   *
   * @param folder IMAPFolder
   * @throws MessagingException
   */
  private static void searchSubFolder(IMAPFolder folder, ArrayList fs) 
    throws MessagingException {
      
      Folder[] folderList = folder.list();
      
      IMAPFolder f;
      String s = "";
      for (int i = 0; i < folderList.length; i++) {
          f = ((IMAPFolder) folderList[i]);
          s = "folder name: " + f.getName() + " - full name: " + f.getFullName();
          fs.add(s);
          searchSubFolder(f, fs);
      }
      
  }
  
  
}
