/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.items.dao;


import com.funambol.email.engine.cache.CrcSyncItemInfo;
import java.util.logging.Logger;
import java.util.logging.Level;
import java.sql.PreparedStatement;
import java.sql.Connection;
import java.sql.ResultSet;
import javax.mail.Message;
import javax.mail.Flags;
import javax.mail.Folder;
import javax.mail.MessagingException;
import javax.sql.DataSource;
import com.sun.mail.imap.IMAPFolder;

import com.funambol.framework.tools.DBTools;
import com.funambol.server.tools.IdSpaceGenerator;
import com.funambol.email.items.model.ItemFolder;
import com.funambol.email.transport.IMailServerWrapper;
import com.funambol.email.exception.EntityException;
import com.funambol.email.items.model.EmailFilter;
import com.funambol.email.transport.CommonMailServerWrapper;
import com.funambol.email.util.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Locale;
import javax.mail.internet.MimeMessage;


/**
 * This class implements methods to access data
 * in Mail Server datastore.
 *
 * @version $Id: EntityDAO.java,v 1.28.2.1 2006/11/24 10:18:06 gbmiglia Exp $
 */
public class EntityDAO {


    // ------------------------------------------------------------ Private data

    protected Logger  log = Logger.getLogger(Def.LOGGER_NAME);

    /**     */
    protected String jndiDataSourceName = null;

    /**     */
    //protected DataSource dataSource;


    // -------------------------------------------------------------- Properties


    /**
     *
     */
    public EntityDAO(){
    }

    /**
     *
     * @param jndiDataSourceName String
     */
    public EntityDAO(String jndiDataSourceName){
        this.jndiDataSourceName = jndiDataSourceName;
    }


    // -----------------------------------------------------------Public Methods

    /**
     *
     * insert a custom folder
     *
     * @param folder IMAPFolder
     * @param defFullpath String
     * @param idFolderSpace IdSpaceGenerator
     * @param source_uri String
     * @param principalId long
     * @throws EntityException
     */
    public void insertCustomFolder(IMAPFolder folder,
                                   String defFullpath,
                                   IdSpaceGenerator idFolderSpace,
                                   String source_uri,
                                   long principalId)
       throws EntityException {

        String[] allFolder = null;

        // check all subfolders and insert if nedded
        try {
            insertSubFolder(folder, idFolderSpace, source_uri, principalId);
        } catch (MessagingException me){
            throw new EntityException(me);
        }


    }


    /**
     *
     * gets the folder fullpath from the local DB
     *
     *
     * @param GUID folder id
     * @param source_uri String
     * @param principalId long
     * @return String
     * @throws EntityException
     */
    public String getFullPathFromGUID(String GUID,
                                      String source_uri,
                                      long principalId)
      throws EntityException {

        String fullpath       = null;
        Connection connection = null;
        PreparedStatement  ps = null;
        ResultSet rs          = null;

        try {

           DataSource dataSource = UtilityDB.lookupDataSource(this.jndiDataSourceName);
           
           connection = dataSource.getConnection();
           ps = connection.prepareStatement(Query.FOLDER_SELECT_FULLPATH);

           ps.setString (1, GUID        ) ;
           ps.setString (2, source_uri  ) ;
           ps.setLong   (3, principalId ) ;

           rs = ps.executeQuery();
           while(rs.next()) {
               fullpath = rs.getString(1);
           }

           ps.close() ;
           ps = null  ;
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }

        return fullpath;

    }


   /**
    *
    * gets the folder fullpath from the local DB
    * In the local db the GUID is composed by:
    * parentId + separator + objectId
    * This method look up the item with the
    * GUID = '%/FID'
    *
    *
    *
    * @param FID folder id
    * @param source_uri String
    * @param principalId long
    * @return String
    * @throws EntityException
    */
    public String getFullPathFromFID(String FID,
                                      String source_uri,
                                      long principalId)
      throws EntityException {

        String fullpath       = null;
        Connection connection = null;
        PreparedStatement  ps = null;
        ResultSet rs          = null;


        try {

           DataSource dataSource = UtilityDB.lookupDataSource(this.jndiDataSourceName);

           connection = dataSource.getConnection();
           ps = connection.prepareStatement(Query.FOLDER_SELECT_FULLPATH_FROM_FID);

           ps.setString (1, "%" + Def.SEPARATOR_FIRST + FID ) ;
           ps.setString (2, source_uri                ) ;
           ps.setLong   (3, principalId               ) ;

           rs = ps.executeQuery();
           while(rs.next()) {
               fullpath = rs.getString(1);
           }

           ps.close() ;
           ps = null  ;
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }

        return fullpath;

    }


    /**
     *
     * gets the Folder id from the local DB
     *
     *
     * @param fullpath folder full path
     * @param source_uri String
     * @param principalId long
     * @return String
     * @throws EntityException
     */
    public String getGUIDFromFullPath(String fullpath,
                                      String source_uri,
                                      long principalId)
      throws EntityException {

        String GUID           = null;
        Connection connection = null;
        PreparedStatement  ps = null;
        ResultSet rs          = null;

        try {

           DataSource dataSource = UtilityDB.lookupDataSource(this.jndiDataSourceName);

           connection = dataSource.getConnection();
           ps = connection.prepareStatement(Query.FOLDER_SELECT_FOLDERID);

           ps.setString (1, fullpath    ) ;
           ps.setString (2, source_uri  ) ;
           ps.setLong   (3, principalId ) ;

           rs = ps.executeQuery();
           while(rs.next()) {
               GUID     = rs.getString(1);
           }

           ps.close() ;
           ps = null  ;
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }

        return GUID;

    }


    /**
     * This method inserts the folder in the MailServer
     *
     * @param cmsw CommonMailServerWrapper
     * @param folderName String
     * @throws EntityException
     */
    public void insertDefaultFolderInMailServer(CommonMailServerWrapper cmsw,
                                                String folderName)
      throws EntityException {

        try {
           Folder folderToInsert = cmsw.getMailDefaultFolder().getFolder(folderName);
           int type = 0; // set the right folder type
           if (!folderToInsert.exists()) {
              folderToInsert.create(Folder.HOLDS_MESSAGES);
           }
        } catch (Exception e) {
           throw new EntityException(e);
        }

    }

    /**
     *
     * serch a folder using the GUID in the local DB
     *
     *
     * @param GUID folder id
     * @param source_uri String
     * @param principalId long
     * @return ItemFolder
     * @throws EntityException
     */
    public ItemFolder getFolderFromUID(String GUID,
                                       String source_uri,
                                       long principalId)
       throws EntityException {

        ItemFolder eisFolder   = null;
        String     fullpath    = null;
        Connection connection  = null;
        PreparedStatement ps   = null;
        ResultSet rs           = null;

        try {

           DataSource dataSource = UtilityDB.lookupDataSource(this.jndiDataSourceName);

           connection = dataSource.getConnection();
           ps = connection.prepareStatement(Query.FOLDER_SELECT_FOLDER);
           ps.setString (1, GUID ) ;
           ps.setString (2, source_uri ) ;
           ps.setLong   (3, principalId ) ;
           rs = ps.executeQuery();

           String FID  = null;
           while(rs.next()) {
               GUID      = rs.getString(1);
               FID       = rs.getString(2);
               fullpath  = rs.getString(3);
           }

           if (fullpath != null) {

               String name = Utility.getFolderNameFromFullpath(fullpath);

               eisFolder = new ItemFolder(GUID, name, FID, null,
                                          null, null, null, null);
           }

           ps.close() ;
           ps = null  ;

        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }

        return eisFolder;
    }
    
    /**
     *
     * get the max number of emails in the cache
     *
     * @param source_uri String
     * @param source_uri String
     * @param principalId long
     * @return ItemFolder
     * @throws EntityException
     */
    public int getMaxEmailNumber(String username)
       throws EntityException {

        Connection connection  = null;
        PreparedStatement ps   = null;
        ResultSet rs           = null;
        int max                = Def.MAX_INBOX_EMAIL_NUMBER;

        try {
            
           DataSource dataSource = UtilityDB.lookupDataSource(this.jndiDataSourceName);

           connection = dataSource.getConnection();
           ps = connection.prepareStatement(Query.ACCOUNT_GET_ACCOUNT);
           ps.setString (1, username ) ;
           rs = ps.executeQuery();

           while(rs.next()) {               
               max = Integer.parseInt(rs.getString(4));
           }

           ps.close() ;
           ps = null  ;

        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }

        return max;
    }    

    /**
     *
     *
     * @param f IMAPFolder
     * @param cmsw CommonMailServerWrapper
     * @throws EntityException
     */
    public void checkMessageIDforDrafts(IMAPFolder f,
                                        CommonMailServerWrapper cmsw)
      throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("DAO start checkMessageIDforDrafts");
        }

        try {

            Message[] messages = f.getMessages();
            int n = messages.length;

            Message msg = null;
            Message msgNew  = null;

            for (int i=0; i<n; i++) {
                msg = messages[i];

                String message_id = Utility.getHeaderMessageID(msg);

                if (message_id == null){

                    // create a new message id and set the msg
                    java.util.Random generator = new java.util.Random();
                    long random        = generator.nextLong();
                    long validity      = f.getUIDValidity();
                    long id            = f.getUID(msg);
                    String userAddress = cmsw.getUserAddress();
                    String serverOut   = cmsw.getServerOut();
                    message_id = "<" + userAddress + "." +
                                 validity    + "." +
                                 id          + "." +
                                 random      + "@" + serverOut + ">";

                    // clone the message and set the Message-ID
                    Message newMsg = new MimeMessage((MimeMessage)msg);
                    //newMsg.setHeader(Def.HEADER_LABEL_MESSAGE_ID, message_id);
                    newMsg.saveChanges();

                    // delete old
                    Message[] msgsToDelete = new Message[1];
                    msgsToDelete[0] = msg;
                    f.setFlags(msgsToDelete, new Flags(Flags.Flag.DELETED), true);

                    // append new
                    Message[] msgsToAppend = new Message[1];
                    msgsToAppend[0] = newMsg;
                    f.appendMessages(msgsToAppend);

                }

            }

        } catch (MessagingException e) {
            throw new EntityException(e);
        }

    }


  /**
   *
   *
   *
   * @param fullPath folder full path
   * @param parentId parent folder id
   * @param idFolderSpace IdSpaceGenerator
   * @param source_uri String
   * @param principalId long
   * @return String
   * @throws EntityException
   */
  public String insertFolder(String fullPath,
                             String parentId,
                             IdSpaceGenerator idFolderSpace,
                             String source_uri,
                             long principalId)
    throws EntityException {

      Connection connection = null;
      PreparedStatement  ps = null;
      String GUID           = null;
      String FID            = null;

      try {

         DataSource dataSource = UtilityDB.lookupDataSource(this.jndiDataSourceName);

         connection = dataSource.getConnection();
         ps = connection.prepareStatement(Query.FOLDER_INSERT_FOLDER);

         GUID = Utility.createFolderGUID(parentId, idFolderSpace.next());

         FID = parentId;

         ps.setString(1, GUID);
         ps.setString(2, FID);
         ps.setString(3, source_uri);
         ps.setLong(4, principalId);
         ps.setString(5, fullPath);

         ps.executeUpdate();

         ps.close() ;
         ps = null  ;

      } catch (Exception e) {
          throw new EntityException(e);
      } finally {
          DBTools.close(connection, ps, null);
      }

      return GUID;

  }


  /**
   * check if the Folder is in the mail server
   *
   * @param msw - IMailServerWrapper .
   * @param fullpath - String .
   * @return true if the folder is in the Mail Server - boolean.
   * @throws EntityException
   */
  public Folder getDefaultFolder(IMailServerWrapper msw,
                                  String fullpath)
     throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("DAO start isDefaultFolderPresent");
        }

        Folder f = null;

        try {
            f = msw.getMailDefaultFolder().getFolder(fullpath);
            if (f.exists()) {
               return f;
            }
        } catch (MessagingException e) {
            throw new EntityException(e);
        }

        return null;
    }

  /**
    *
    *
    *
    * @param GUID String
    * @param source_uri String
    * @param principalId long
    * @throws EntityException
    */
   public void removeFolder(String GUID,
                            String source_uri,
                            long principalId)
     throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("DAO start removeFolder UID " + GUID );
        }
        Connection connection = null;
        PreparedStatement  ps = null ;
        try {
            
           DataSource dataSource = UtilityDB.lookupDataSource(this.jndiDataSourceName);

           connection = dataSource.getConnection();
           
           ps = connection.prepareStatement(Query.FOLDER_DELETE_FOLDER);
           ps.setString (1, GUID ) ;
           ps.setString (2, source_uri ) ;
           ps.setLong   (3, principalId ) ;
           ps.executeUpdate();
           ps.close() ;
           ps = null  ;
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }

    }

   
    /**
     * @param username String
     * @return The Items Collection with all info
     * @throws EntityException
     */
    public HashMap getAllEmailsInbox(String username, String protocol)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("DAO start getAllEmails Inbox");
        }

        HashMap    items      = new HashMap();
        Connection connection = null;
        PreparedStatement ps  = null;
        ResultSet rs;
        String GUID       = null;
        long lastCRC      = 0;
        String isInvalid  = null;
        String isInternal = null;
        String messageid  = null;
        java.util.Date headerdate   = null;                    
        
        try {

            DataSource dataSource = UtilityDB.lookupDataSource(this.jndiDataSourceName);
            
            connection = dataSource.getConnection();
                        
            ps = connection.prepareStatement(Query.INBOX_SELECT_MAILBOX_INFO);

            ps.setString(1, username);
            ps.setString(2, protocol);

            rs = ps.executeQuery();
                                                   
            while(rs.next()) {
                 GUID       = rs.getString(1);
                 lastCRC    = rs.getLong(2);
                 isInvalid  = rs.getString(3);
                 isInternal = rs.getString(4);
                 messageid  = rs.getString(5);
                 headerdate = Utility.UtcToDate(rs.getString(6));     
                 
                 items.put(GUID, new CrcSyncItemInfo(GUID, lastCRC, 
                                                     messageid, headerdate, null,
                                                     isInvalid, isInternal, 
                                                     "Y", null));
                 
             }
            
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }
        if (items != null) {
            if (log.isLoggable(Level.FINEST)) {
                log.finest("number of items " + items.size());
            }
        }       

        return items;

    }
       
   
  /**
    *
    *
    *
    * @param GUID String
    * @param source_uri String
    * @param principalId long
    * @throws EntityException
    */
   public void removeEmailFromInbox(String username, 
                                    String protocol, 
                                    String GUID)
     throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("DAO start removeEmailFromInbox UID " + GUID + " username " + username);
        }
        Connection connection = null;
        PreparedStatement  ps = null ;
        try {
            
           DataSource dataSource = UtilityDB.lookupDataSource(this.jndiDataSourceName);
           
           connection = dataSource.getConnection();
           
           ps = connection.prepareStatement(Query.INBOX_DELETE_EMAIL);
           
           ps.setString (1, username ) ;
           ps.setString (2, protocol ) ;
           ps.setString (3, GUID ) ;

           ps.executeUpdate();
           ps.close() ;
           ps = null  ;
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }

    }   

  /**
   * remove the item from the map 
   *
   *
   * @param GUID String
   * @param serverItems all info in CrcSyncItemInfo
   * @throws EntityException
   */
   public void removeEmailFromServerItems(String GUID,
                                          HashMap serverItems)
     throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("DAO start removeEmailFromServerItems UID " + GUID );
        }
        
        serverItems.remove(GUID);

    }   
   
   
  /**
   *
   * This method inserts one of the default folders if needed
   *
   *
   * @param GUID String
   * @param parentId String
   * @param fullPath String
   * @param source_uri String
   * @param principalId long
   * @throws EntityException
   */
  public void insertDefaultFolderInDB(String GUID,
                                      String parentId,
                                      String fullPath,
                                      String source_uri,
                                      long principalId)
    throws EntityException {

      Connection connection = null;
      PreparedStatement  ps = null;

      try {

         DataSource dataSource = UtilityDB.lookupDataSource(this.jndiDataSourceName);
         
         connection = dataSource.getConnection();
         
         ps = connection.prepareStatement(Query.FOLDER_INSERT_FOLDER);

         ps.setString (1, GUID        ) ;
         ps.setString (2, parentId    ) ;
         ps.setString (3, source_uri  ) ;
         ps.setLong   (4, principalId ) ;
         ps.setString (5, fullPath    ) ;

         ps.executeUpdate();

         ps.close() ;
         ps = null  ;

      } catch (Exception e) {
          throw new EntityException(e);
      } finally {
          DBTools.close(connection, ps, null);
      }

  }

 /**
   *
   * This method inserts one of the default folders if needed
   *
   *
   * @param GUID String
   * @param source_uri String
   * @param principalId long
   * @throws EntityException
   */
  public void insertInvalidItemInCache(String GUID,
                                       String messageId,
                                       java.util.Date headerDate,
                                       String isEmail,
                                       String source_uri,
                                       long principalId)
    throws EntityException {

      Connection connection = null;
      PreparedStatement  ps = null;

      try {

         DataSource dataSource = UtilityDB.lookupDataSource(this.jndiDataSourceName);
         
         connection = dataSource.getConnection();
         
         // try to update an item in the local db
         ps = connection.prepareStatement(Query.CACHE_INVALID_ITEM);

         ps.setString (1, GUID        ) ;
         ps.setString (2, source_uri  ) ;
         ps.setLong   (3, principalId ) ;

         int num = ps.executeUpdate();

         // if the item is not in the local DB
         if (num==0){
             
            ps = connection.prepareStatement(Query.CACHE_ADD_LOCAL_ITEM);
            
            String headerDateS = null;
            if (headerDate != null){
               headerDateS = Utility.UtcFromDate(headerDate); 
            } 
                        
            ps.setString(1, GUID);
            ps.setLong  (2, 1  );
            ps.setString(3, "Y");
            ps.setString(4, null);
            ps.setString(5, source_uri);
            ps.setLong  (6, principalId);
            ps.setString(7, messageId);
            ps.setString(8, headerDateS);
            ps.setString(9, isEmail);
            
            num = ps.executeUpdate();
         }
         
         ps.close() ;
         ps = null  ;

      } catch (Exception e) {
          throw new EntityException(e);
      } finally {
          DBTools.close(connection, ps, null);
      }

  }

  
  //-------------------------------------------------------- PROTECTED METHODS


  /**
   *
   */
   protected Message[] getTimeFilteredItems(Message[] tmpmessages, 
                                            EmailFilter filter, 
                                            Locale loc) 
      throws MessagingException {
       
         Message[] messages;
         java.util.Date d1   = null;
         boolean parserError = false;
         ArrayList filteredmessages = new ArrayList();
         
         for (int i=0; i<tmpmessages.length; i++){
             parserError = false;
             try {
                 d1 = UtilityDate.getHeaderDate(tmpmessages[i], loc);
             } catch (EntityException ee){
                 parserError = true;
                 log.severe("Error parsing Message " + ee.getMessage());
             }
             if (!parserError){
                 if (Utility.d1Afterd2(d1, filter.getTime())){
                     filteredmessages.add(tmpmessages[i]);
                 } 
             }                        
         }

         messages = (Message[])filteredmessages.toArray(new Message[0]);
         
         return messages;
   }

  
  
  
    /**
     * Lookups the datasource
     *
     * JBOSS
     * private String jndiDataSourceName = "java:jdbc/sync4j";
     *
     * TOMCAT
     * private String jndiDataSourceName = "java:comp/env/jdbc/sync4j";
     *
     * @throws Exception if a error occurs during lookup
     */
   /*
    protected void lookupDataSource() throws Exception {

        InitialContext ctx = new InitialContext();

        try {
          // try tomcat  i.e. java:comp/env/jdbc/sync4j
          dataSource = (DataSource) ctx.lookup("java:comp/env/"+jndiDataSourceName) ;
        }
        catch (NamingException ne) {
          try {
              // try jboss i.e. java:jdbc/sync4j
              dataSource = (DataSource) ctx.lookup("java:"+jndiDataSourceName) ;
          }
          catch (Exception ex){
            try {
                // try othser j2ee server jdbc/sync4j
                dataSource = (DataSource) ctx.lookup(""+jndiDataSourceName) ;
            }
            catch (Exception e){
              throw new Exception("Data source "+ jndiDataSourceName
                                  + " not found " + e.toString());
            }
          }
        }
      }
   */

  //------------------------------------------------------------ PRIVATE METHODS

    /**
     *
     * @param folder starting folder
     * @param fs all folders container
     * @throws MessagingException
     */
    private void insertSubFolder(IMAPFolder folder,
                                 IdSpaceGenerator idFolderSpace,
                                 String source_uri,
                                 long principalId)
       throws MessagingException, EntityException {

        javax.mail.Folder[] folderList = folder.list();

        IMAPFolder f;

        String fullpath = null;
        String GUID = null;
        String parentGUID = null;
        String parentId = null;
        String parentfullpath = null;

        for (int i = 0; i < folderList.length; i++) {

            f = ((IMAPFolder) folderList[i]);

            fullpath = f.getFullName();

            GUID = getGUIDFromFullPath(fullpath, source_uri, principalId);


            if (GUID == null){

                 parentfullpath = Utility.getParentFullpathFromFullpath(fullpath);

                 parentGUID = getGUIDFromFullPath(parentfullpath, source_uri, principalId);

                 parentId = Utility.getKeyPart(parentGUID,1);

                 // insert the missed folder
                 insertFolder(fullpath, parentId,  idFolderSpace,
                              source_uri, principalId);

            }

            insertSubFolder(f, idFolderSpace, source_uri, principalId);
        }

    }

}
