/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.items.manager ;


import com.funambol.email.engine.cache.CrcSyncItemInfo;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.Locale;
import java.io.InputStream;
import java.io.ByteArrayInputStream;

import javax.mail.Message;
import javax.mail.Session;
import javax.mail.Folder;
import javax.mail.Flags;
import javax.mail.Transport;
import com.sun.mail.imap.IMAPFolder;
import javax.mail.internet.MimeMessage;
import javax.mail.MessagingException;

import com.funambol.email.pdi.mail.Email;
import com.funambol.email.pdi.mail.EmailItem;
import com.funambol.email.items.dao.EntityDAO;
import com.funambol.email.items.model.ItemMessage;
import com.funambol.email.transport.IMailServerWrapper;
import com.funambol.email.transport.CommonMailServerWrapper;
import com.funambol.email.transport.ImapMailServerWrapper;
import com.funambol.email.util.Def;
import com.funambol.email.util.Utility;
import com.funambol.email.exception.*;
import com.funambol.email.items.model.ItemFolder;
import com.funambol.email.items.model.DefaultFolder;
import com.funambol.email.items.model.EmailFilter;
import com.funambol.email.pdi.mail.Ext;
import com.funambol.email.util.UtilityDate;
import com.funambol.server.tools.IdSpaceGenerator;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import javax.mail.Address;
import javax.mail.SendFailedException;




/**
 *
 * @version $Id: EntityManager.java,v 1.45.2.1 2006/11/24 10:18:15 gbmiglia Exp $
 */
public class EntityManager {



    // -------------------------------------------------------------- Properties

    /**     */
    protected CommonMailServerWrapper cmsw = null;

    /**     */
    protected Logger  log = Logger.getLogger(Def.LOGGER_NAME);

    /**     */
    public EntityDAO ed       = null;

    /**     */
    protected long timeStart = 0;
    protected long timeStop  = 0;

    // ------------------------------------------------------------ Costructors


    /**
     *
     */
    public EntityManager(){
    }

    /**
     *
     *
     * @param msw_e IMailServerWrapper
     * @param jndiDataSourceName String
     * @throws EmailAccessException
     */
    public EntityManager(IMailServerWrapper msw_e, String jndiDataSourceName)
     throws EmailAccessException
    {
       cmsw = (CommonMailServerWrapper)msw_e;
       ed   = new EntityDAO(jndiDataSourceName);
    }

   // ----------------------------------------------------------- Public Methods

    /**
     *
     * Return all email ids. it uses the CrcSyncItemInfo[], this array
     * is set by the caching system method in the beginSync method.
     * the si param includes inbox/sent folders info for the POP3 SyncSource
     * and inbox/outbox/sent/drafts/trash folders info for the IMAP
     * SyncSource.
     *
     * @param filter EmailFilter
     * @param si mail server info
     * @return The all items fulfilling the filter requirements as an array of Strings.
     * @throws EntityException
     */
    public String[] getAllEmails(EmailFilter filter, HashMap si)
      throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("Start - getAllItems() ");
        }

        String[] allitems;
        String   GUID       = null;
        Date     headerDate = null;
        boolean  isEmail    = false;

        try {

            if (filter.getId() != null) {
                ArrayList tmpIds = getFilteredItemByID(si, filter.getId());
                allitems = createArray(tmpIds);
            } else {
                if (filter.getTime() != null) {
                    ArrayList tmpIds = getFilteredItemByTime(si, filter.getTime());
                    allitems = createArray(tmpIds);
                } else {
                    // get all items
                    allitems = new String[si.size()];
                    Iterator keys = si.keySet().iterator();
                    int i = 0;
                    while (keys.hasNext()) {
                        allitems[i] = (String)keys.next();
                        i++;
                    }
                }
            }

        } catch (Exception e) {
            throw new EntityException("Error Getting all emails (" +
                    e.getMessage() + ")");
        }


        if (log.isLoggable(Level.FINEST)) {
            log.finest("End - getAllItems() ");
        }

        return allitems;
    }


    /**
     *
     * create a javaMail Message from a funambol foundation Email
     *
     * @param s Session
     * @param ctmp Email
     * @param FID String
     * @param toBeSend boolean
     * @return Message
     * @throws EntityException
     */
    public Message createMessage(Session s,
                                 Email ctmp,
                                 String FID,
                                 boolean toBeSend,
                                 boolean read,
                                 boolean forwarded,
                                 boolean replied)
      throws EntityException {

        Message doc;

        try {

            EmailItem emailItem = ctmp.getEmailItem();

            String rfc2822 = emailItem.getStringRFC2822();

            String messageID = Utility.getHeaderMessageID(rfc2822);

            byte[] b = rfc2822.getBytes();

            InputStream is = new ByteArrayInputStream(b);

            doc = new MimeMessage(s, is);

            // if draft set the FLAG
            if (FID.equalsIgnoreCase(Def.FOLDER_DRAFTS_ID)){
               doc.setFlag(Flags.Flag.DRAFT, true);
            }

            // for all folders set eventually the Flags
            doc.setFlags(new Flags(Flags.Flag.SEEN)     , read);
            //doc.setFlags(new Flags(Flags.Flag.FORWARDED), false);
            doc.setFlags(new Flags(Flags.Flag.ANSWERED) , replied);

            if (toBeSend){
               // don't set the HEADER_LABEL_SYNCMESSAGE
            } else {
               // we must set the message as "sync message"
               doc.setHeader(Def.HEADER_LABEL_SYNCMESSAGE, Def.HEADER_VALUE_SYNCMESSAGE);
            }

            if (messageID != null){
               doc.setHeader(Def.HEADER_LABEL_MESSAGE_ID,messageID);
            }

        } catch (Exception e){
            throw new EntityException(e);
        }

        return doc;
    }


    /**
     *
     *
     * @param im ItemMessage
     * @return Email
     * @throws EntityException
     */
    public Email createFoundationMail(ItemMessage im)
     throws EntityException {

        Email mtmp       = new Email();

        String guid      = null;  // parentId + separator + mailId
        String parentId  = null;

        String emailItem = null;
        String encodingType = null;

        String created   = null; // Date created
        String received  = null; // Date received
        String modified  = null; // Date modified
        String read      = null; // boolean read
        String forwarded = null; // boolean forwarded
        String replied   = null; // boolean replied
        String deleted   = null; // boolean deleted
        String flagged   = null; // boolean flagged

        Ext    ext       = null;

        try {

           guid      = im.getGUID();
           parentId  = im.getParentId();

           emailItem = im.getStreamMessage();
           encodingType = im.getEncodingType();

           created   = im.getCreated();
           received  = im.getReceived();
           modified  = im.getModified();
           read      = im.getRead();
           forwarded = im.getForwarded();
           replied   = im.getReplied();
           deleted   = im.getDeleted();
           flagged   = im.getFlagged();

           ext       = im.getExt();

        } catch (Exception e) {
           throw new EntityException("Error creating the foundation email " + e.getMessage());
        }

        mtmp.getUID().setPropertyValue(guid);

        mtmp.getParentId().setPropertyValue(parentId);

        mtmp.getEmailItem().setPropertyValue(emailItem);
        mtmp.getEmailItem().setEncoding(encodingType);

        if (created != null){
          mtmp.getCreated().setPropertyValue(created);
        }
        if (received != null){
          mtmp.getReceived().setPropertyValue(received);
        }
        if (modified != null){
          mtmp.getModified().setPropertyValue(modified);
        }

        if (read != null){
          mtmp.getRead().setPropertyValue(read);
        }
        if (forwarded != null){
          mtmp.getForwarded().setPropertyValue(forwarded);
        }
        if (replied != null){
          mtmp.getReplied().setPropertyValue(replied);
        }
        if (deleted != null){
          mtmp.getDeleted().setPropertyValue(deleted);
        }
        if (flagged != null){
          mtmp.getFlagged().setPropertyValue(flagged);
        }

        if (ext != null){
            mtmp.setExt(ext);
        }

        return mtmp;
    }


    /**
     *
     * create the foundation folder object
     *
     * @param f ItemFolder
     * @return Folder
     * @throws EntityException
     */
    public com.funambol.email.pdi.folder.Folder createFoundationFolder(
                                                               ItemFolder f)
      throws EntityException {

        com.funambol.email.pdi.folder.Folder fout = new
                                         com.funambol.email.pdi.folder.Folder();

        fout.getUID().setPropertyValue(f.getGUID());

        fout.getParentId().setPropertyValue(f.getParentId());

        fout.getName().setPropertyValue(f.getName());

        fout.getCreated().setPropertyValue(f.getCreated());

        fout.getAccessed().setPropertyValue(f.getAccessed());

        fout.getModified().setPropertyValue(f.getModified());

        fout.getRole().setPropertyValue(f.getRole());

        //fout.getAttributes().setPropertyValue(f.getAttributes);

        return fout;
    }

    /**
     *
     * get the max number of email in the cache
     *
     * @param username String
     * @param source_uri String
     * @param pID principal ID long
     * @throws EntityException
     */
    public int getMaxEmailNumber(String userName)
       throws EntityException {
        int max = 0;
        try {
           max = this.ed.getMaxEmailNumber(userName);
        } catch (EntityException e) {
           throw new EntityException("Error getting max number of email " + e.getMessage());
        }
        return max;
    }

    /**
     *
     * insert the default folder if needed
     *
     *
     * @param df DefaultFolder
     * @param source_uri String
     * @param pID principal ID long
     * @throws EntityException
     */
    public void  insertDefaultFolder(DefaultFolder df,
                                     String protocol,
                                     String source_uri,
                                     long pID)
      throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
          log.finest("EntityManager insertDefaultFolder start");
        }

        String GUID       = null;
        String folderName = null;
        String parentId   = Def.FOLDER_ROOT_ID ;

        try {

          GUID       = Utility.createFolderGUID(parentId, df.getInboxId());
          folderName = df.getInboxName();
          addDefaultFolder(df, "I", GUID, folderName, parentId, protocol,
                           source_uri, pID);

          GUID       = Utility.createFolderGUID(Def.FOLDER_ROOT_ID, df.getOutboxId());
          folderName = df.getOutboxName();
          addDefaultFolder(df, "O", GUID, folderName, parentId, protocol,
                           source_uri, pID);

          GUID       = Utility.createFolderGUID(Def.FOLDER_ROOT_ID, df.getSentId());
          folderName = df.getSentName();
          addDefaultFolder(df, "S", GUID, folderName, parentId, protocol,
                           source_uri, pID);

          GUID       = Utility.createFolderGUID(Def.FOLDER_ROOT_ID, df.getDraftId());
          folderName = df.getDraftName();
          addDefaultFolder(df, "D", GUID, folderName, parentId, protocol,
                           source_uri, pID);

          GUID       = Utility.createFolderGUID(Def.FOLDER_ROOT_ID, df.getTrashId());
          folderName = df.getTrashName();
          addDefaultFolder(df, "T", GUID, folderName, parentId, protocol,
                           source_uri, pID);


       } catch (EntityException ee){
           throw new EntityException("Error inserting the default folder", ee);
       }

       if (log.isLoggable(Level.FINEST)) {
          log.finest("EntityManager insertDefaultFolder end" );
       }

    }


    /**
     *
     * checks if a mail match the filter
     *
     * @param filter EmailFilter
     * @param parentId String
     * @param GUID String
     * @param email  Email
     * @return true if the message match the filter
     * @throws EntityException
     */
    public boolean isEmailInFilter(Email email,
                                   EmailFilter filter,
                                   String GUID,
                                   String FID,
                                   String FMID,
                                   Session session,
                                   Locale loc,
                                   String source_uri,
                                   long principalId)
      throws EntityException {


        boolean isInFilter = false;

        try {

            // get Properties
            boolean[] flags = this.setFlags(email);
            boolean read      = flags[0];
            boolean forwarded = flags[1];
            boolean replied   = flags[2];

            Message msg = createMessage(session, email, FID,
                                        false,  read,  forwarded, replied);

            // check if this message is included in the filter

            if (filter.getId() != null) {
                // check GUID
                if (filter.getId().equals(GUID)){
                    isInFilter = true;
                } else {
                    isInFilter = false;
                }
            } else {
                // check date
                Date msgdate = UtilityDate.getHeaderDate(msg, loc);
                if ((filter.getTime()!=null) && (msgdate!=null)){
                    if (hasMatchedDate(msgdate,filter)){
                        isInFilter = true;
                    } else {
                        isInFilter = false;
                    }
                } else {
                    // check filter size
                    if (hasMatchedSize(msg,filter)){
                        isInFilter = true;
                    } else {
                        isInFilter = false;
                    }
                }
            }

        } catch (Exception e) {
                throw new EntityException(e);
        }

        return isInFilter;
    }

    /**
     *
     * insert invalid item
     *
     * @param GUID String
     * @param source_uri String
     * @param pID principal ID long
     * @throws EntityException
     */
    public void  insertInvalidItem(String GUID,
                                   String messageId,
                                   java.util.Date headerDate,
                                   String isEmail,
                                   String source_uri,
                                   long pID)
      throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
          log.finest("EntityManager insertInvalidItem for GUID " + GUID);
        }
        try {
           this.ed.insertInvalidItemInCache(GUID, messageId,
                                            headerDate, isEmail,
                                            source_uri, pID);
       } catch (EntityException ee){
           throw new EntityException("Error inserting the invalid item", ee);
       }

    }

   /**
    * Checks if an email is already inserted in the db
    * and returns the GUID.
    *
    * @param session mail server session
    * @param serverItems all server items info CrcSyncItemInfo[] (I need the Message-ID)
    * @param parentId parent if of the email
    * @param ctmp the Email to check
    * @return the GUID
    * @throws EntityException
    */
    public String getEmailFromClause(Session session,
                                     HashMap serverItems,
                                     String parentId,
                                     Email ctmp)
      throws EntityException {

        String messageIdExt = null;
        String messageIdInt = null;
        String GUID         = null;
        Message msgExt      = null;

        // create message
        try {
            boolean[] flags = this.setFlags(ctmp);
            boolean read      = flags[0];
            boolean forwarded = flags[1];
            boolean replied   = flags[2];
            msgExt = createMessage(session, ctmp, parentId,
                                   false, read, forwarded, replied);
        } catch (Exception e) {
            throw new EntityException(e);
        }

        try {
            messageIdExt = Utility.getHeaderMessageID(msgExt);
            if (messageIdExt != null) {
                CrcSyncItemInfo crcii = null;

                Iterator values = serverItems.values().iterator();
                while (values.hasNext()) {
                    crcii = (CrcSyncItemInfo)values.next();
                    messageIdInt = crcii.getMessageID();
                    if (messageIdInt != null){
                        if (messageIdExt.equals(messageIdInt)){
                            GUID = crcii.getKey().getKeyAsString();
                            break;
                        }
                    }
                }
                /*
                for (int i=0; i<allServerItems.length; i++){
                     crcii = allServerItems[i];
                     messageIdInt = crcii.getMessageID();
                     if (messageIdInt != null){
                         if (messageIdExt.equals(messageIdInt)){
                             GUID = crcii.getKey().getKeyAsString();
                             break;
                         }
                     }
                }
                */
            }
        } catch (Exception e) {
            throw new EntityException(e);
        }

        return GUID;

    }


    /**
     * check if in the Drafts folder there is an item without
     * the message-id
     *
     *
     * @param source_uri  String
     * @param principalId long
     * @throws EntityException
     */
    public void checkMessageIDforDrafts(String source_uri,
                                        long principalId)
      throws EntityException {

        IMAPFolder f        = null;
        String     fullpath = null;
        EntityException finalExc = null;

        try {

            fullpath = this.ed.getFullPathFromFID(Def.FOLDER_DRAFTS_ID,
                                                  source_uri,
                                                  principalId);

            if (fullpath != null) {
               f = (IMAPFolder)  this.cmsw.getMailDefaultFolder().getFolder(fullpath);
               if (f.exists()) {
                    timeStart = System.currentTimeMillis();
                    f.open(Folder.READ_WRITE);
                    this.ed.checkMessageIDforDrafts(f, this.cmsw);
               }
            }

        } catch (MessagingException me) {
            throw new EntityException(me);
        } finally {
            if (f != null) {
                try {
                    if (f.isOpen()) {
                        f.close(true);
                        timeStop = System.currentTimeMillis();
                        if (log.isLoggable(Level.FINEST)) {
                            log.finest("checkMessageIDforDrafts Execution Time: " +
                                    (timeStop - timeStart) + " ms");
                        }
                    }
                }
                catch (MessagingException me) {
                    finalExc = new EntityException(me);
                }
            }
        }
        if (finalExc != null) {
            throw finalExc;
        }

    }


   /**
     *
     * Check if the local DB folders are consistent with the MailBox folders
     *
     *
     * @param source_uri String
     * @param principalId long
     * @throws EntityException
     */
    public void  insertCustomFolder(IMailServerWrapper msw,
                                    boolean allMailboxActivation,
                                    IdSpaceGenerator idFolderSpace,
                                    String source_uri,
                                    long principalId)
      throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
          log.finest("EntityManager insertCustomFolder start");
        }

        try {

            if (msw instanceof ImapMailServerWrapper){

               ImapMailServerWrapper imsw = (ImapMailServerWrapper)msw;

                if (allMailboxActivation){
                    String     defFullpath;
                    IMAPFolder imapf;

                    // insert all folders in Inboxbox + inboxbox and so on
                    defFullpath = imsw.getDefaultFolder().getInboxName();
                    imapf       = imsw.getInboxFolder();
                    this.ed.insertCustomFolder(imapf, defFullpath, idFolderSpace,
                                               source_uri, principalId);

                    defFullpath = imsw.getDefaultFolder().getOutboxName();
                    imapf       = imsw.getOutboxFolder();
                    this.ed.insertCustomFolder(imapf, defFullpath, idFolderSpace,
                                               source_uri, principalId);

                    defFullpath = imsw.getDefaultFolder().getSentName();
                    imapf       = imsw.getSentFolder();
                    this.ed.insertCustomFolder(imapf, defFullpath, idFolderSpace,
                                               source_uri, principalId);

                    defFullpath = imsw.getDefaultFolder().getDraftName();
                    imapf       = imsw.getDraftsFolder();
                    this.ed.insertCustomFolder(imapf, defFullpath, idFolderSpace,
                                               source_uri, principalId);

                    defFullpath = imsw.getDefaultFolder().getTrashName();
                    imapf       = imsw.getTrashFolder();
                    this.ed.insertCustomFolder(imapf, defFullpath, idFolderSpace,
                                               source_uri, principalId);

                }

            }

        } catch (EmailAccessException ee){
           throw new EntityException("Error getting all folder", ee);
        } catch (EntityException ee){
           throw new EntityException("Error inserting the custom folder", ee);
        }

        if (log.isLoggable(Level.FINEST)) {
          log.finest("EntityManager insertCustomFolder end" );
        }

    }


    /**
     *
     * Check if the item is a folder or an email
     * using the SyncItemKey
     *
     *
     * @param parentId String
     * @param objectId String
     * @param source_uri String
     * @param principalId long
     * @return boolean
     * @throws EntityException
     */
    public boolean isEmail(String parentId,
                           String objectId,
                           String source_uri,
                           long principalId)
      throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
          log.finest("EntityManager isEmail start");
        }

        boolean ismail = false;
        ItemFolder it;


        try {

            String guid = Utility.createFolderGUID(parentId, objectId);

            it = this.ed.getFolderFromUID(guid, source_uri, principalId);

            if (it == null){
               ismail = true;
           }

        } catch (EntityException ee){
           throw new EntityException("Error determinig object type (mail/folder)",
                                     ee);
        }

        if (log.isLoggable(Level.FINEST)) {
          log.finest("EntityManager isEmail end (isEmail :" + ismail + ")" );
        }

        return ismail;

    }

     /**
      * convert Message into ServerMessages
      *
      *
      * @param GUID unique folder id
      * @param name folder name
      * @param FID parent id
      * @return ItemFolder
      * @throws EntityException
      */
     public ItemFolder setItemFolder(String GUID,
                                        String name,
                                        String FID,
                                        String folderRole,
                                        String creationDate)
       throws EntityException {

         if (log.isLoggable(Level.FINEST)) {
             log.finest("setItemFolder() start " + GUID);
         }

         ItemFolder    clientFoler;

         try {
             clientFoler = new ItemFolder(GUID,
                                         name,
                                         FID,
                                         creationDate,
                                         null, // accessed
                                         null, // modified
                                         folderRole,
                                         null // attributes
                                         );

         } catch (Exception e){
             throw new EntityException(e);
         }

         if (log.isLoggable(Level.FINEST)) {
             log.finest("setItemFolder() - end ");
         }

         return clientFoler;

     }

     /**
      *
      * convert Message into ServerMessages
      *
      * @param GUID unique mail id
      * @param msg Message
      * @param parentId String
      * @param loc Locale
      * @param encodingType String
      * @return ItemMessage
      * @throws EntityException
      */
     public ItemMessage setItemMessage(String GUID,
                                       String parentId,
                                       Message msg,
                                       boolean _read,
                                       boolean _forwarded,
                                       boolean _replied,
                                       Locale loc,
                                       String encodingType,
                                       Ext ext)
       throws EntityException {

         ItemMessage   clientMsg;
         String        message_id;
         String        stream;
         String        created;
         String        received;
         String        modified;
         String        read;
         String        forwarded;
         String        replied;
         String        deleted;
         String        flagged;

         try {

             // header message-id
             message_id = Utility.getHeaderMessageID(msg);

             // message
             stream = Utility.StringFromMessage(msg, encodingType);

             //origination date
             created = Utility.UtcFromDate(UtilityDate.getHeaderDate(msg, loc));
             //received date
             received = Utility.getReceivedDate(msg, loc);
             //modified date
             modified = Utility.UtcFromDate(UtilityDate.getHeaderDate(msg, loc)); // @todo: tmp

             // flags
             read      = String.valueOf(_read);
             forwarded = String.valueOf(_forwarded);
             replied   = String.valueOf(_replied);
             deleted   = String.valueOf(msg.isSet(Flags.Flag.DELETED));
             flagged   = null; //@todo: tmp String.valueOf(msg.isSet(Flags.Flag.FLAGGED));

             clientMsg = new ItemMessage(GUID,
                                         parentId,
                                         message_id,
                                         msg,
                                         stream,
                                         encodingType,
                                         created,
                                         received,
                                         modified,
                                         read,
                                         forwarded,
                                         replied,
                                         deleted,
                                         flagged,
                                         ext);

         } catch (MessagingException e){
             throw new EntityException(e);
         }

         return clientMsg;

     }


   /**
    * sends the email
    *
    * @param msg javaMail message to send; Message
    * @throws EntityException
    */
    public void sendEmail(Message msg) throws SendingException {

        String subject = null;

        try {

            subject = msg.getSubject();

            timeStart = System.currentTimeMillis();

            Transport.send(msg);

            timeStop = System.currentTimeMillis();

            if (log.isLoggable(Level.FINEST)) {
               log.finest("Sending Execution Time: " + (timeStop - timeStart) + " ms");

            }

        } catch (MessagingException me) {

            String errorMessage = "Error sending email '" + subject + "'\n" ;

            String invalidList     = "";
            String validUnsentList = "";
            String validSentList   = "";

            Exception ex = me;

            String completemessage = me.getMessage();

            do {
                if (ex instanceof SendFailedException) {

                    SendFailedException sfex = (SendFailedException) ex;

                    Address[] invalid = sfex.getInvalidAddresses();
                    if (invalid != null) {
                        for (int i = 0; i < invalid.length; i++) {
                            invalidList = invalidList + " " + invalid[i];
                        }
                        if (!invalidList.trim().equals("")){
                           invalidList = "Invalid address: " + invalidList;
                        }
                    }

                    Address[] validUnsent = sfex.getValidUnsentAddresses();
                    if (validUnsent != null) {
                        for (int i = 0; i < validUnsent.length; i++) {
                            validUnsentList = validUnsentList + " " + validUnsent[i];
                        }
                        if (!validUnsentList.trim().equals("")){
                           validUnsentList = "Valid unsent address: " + validUnsentList;
                        }
                    }

                    Address[] validSent = sfex.getValidSentAddresses();
                    if (validSent != null) {
                        for (int i = 0; i < validSent.length; i++) {
                            validSentList = validSentList + " " + validSent[i];
                        }
                        if (!validSentList.trim().equals("")){
                           validSentList = "Valid sent address: " + validSentList;
                        }
                    }

                }

                if (ex instanceof MessagingException) {
                    ex = ((MessagingException) ex).getNextException();
                } else {
                    ex = null;
                }

            } while (ex != null);

            errorMessage = errorMessage + completemessage +
                           invalidList + validUnsentList + validSentList ;
            log.severe(errorMessage);
            throw new SendingException(errorMessage);

        }

    }


  //---------------------------------------------------------- PROTECTED METHODS

    /**
     *
     * @param msgdate Message
     * @param filter EmailFilter
     * @return true if message date match the filter date
     */
    protected boolean hasMatchedDate(Date msgdate, EmailFilter filter){

        boolean check = false;

        Date filterDate = filter.getTime();

        String timeFilterClause = filter.getTimeClause();

        if (timeFilterClause.equalsIgnoreCase("EQ") ||
            timeFilterClause.equalsIgnoreCase("GT") ||
            timeFilterClause.equalsIgnoreCase("GE")) {

            if (Utility.d1Afterd2(msgdate, filterDate)){
                check = true;
            }


        } else if (timeFilterClause.equalsIgnoreCase("LT") ||
                   timeFilterClause.equalsIgnoreCase("LE")) {

            if (Utility.d1Befored2(msgdate, filterDate)){
                check = true;
            }

        }

        return check;
    }


   /**
     *
     * @param msg Message
     * @param filter EmailFilter
     * @return true if message date match the filter date
     */
    protected boolean hasMatchedSize(Message msg, EmailFilter filter){

        boolean check = false;

        // if get all message return true
        if (filter.getSize() == Def.FILTER_SIZE_H_B_A){
            return true;
        } else {
            /**
             * @todo
             */

        }

        // tmp
        check = true;

        return check;
    }


   /**
    * returns an array with three flags: read, forwarded, replied
    *
    * @param email EmailFID String
    * @return array with the read, forwarded, replied  flags
    *
    */
   protected boolean[] setFlags(Email email){

        String  flagS     = "false";

        boolean read      = false;
        boolean forwarded = false;
        boolean replied   = false;

        if (email != null) {
            flagS = (String)email.getRead().getPropertyValue();
            if (flagS != null){
               read = (Boolean.valueOf(flagS)).booleanValue();
            }
            flagS = (String)email.getForwarded().getPropertyValue();
            if (flagS != null){
               forwarded = (Boolean.valueOf(flagS)).booleanValue();
            }
            flagS = (String)email.getReplied().getPropertyValue();
            if (flagS != null){
               replied = (Boolean.valueOf(flagS)).booleanValue();
            }
        }

        boolean[] flags = { read, forwarded, replied };

        return flags;

   }

  //------------------------------------------------------------ PRIVATE METHODS

    /**
     *
     * insert the default folder if needed
     *
     *
     * @param df DefaultFolder
     * @param source_uri String
     * @param principalId long
     * @throws EntityException
     */
    private void  addDefaultFolder(DefaultFolder df,
                                   String flag,
                                   String GUID,
                                   String folderName,
                                   String parentId,
                                   String protocol,
                                   String source_uri,
                                   long pID)
      throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
          log.finest("EntityManager insertDefaultFolder start");
        }

        String folderInDB = null;
        Folder folderInMS = null;
        Folder folderToInsert = null;

        try {

          if (protocol.equals(Def.PROTOCOL_IMAP)){
             // IMAP PROTOCOL
             folderInMS = this.ed.getDefaultFolder(this.cmsw, folderName);
             folderInDB = this.ed.getFullPathFromGUID(GUID, source_uri, pID);
             if (folderInMS == null){
                 // is not in the mail server
                 if (folderInDB == null) {
                    // it is not in the DB
                    this.ed.insertDefaultFolderInMailServer(this.cmsw, folderName);
                    this.ed.insertDefaultFolderInDB(GUID, parentId, folderName,
                                                    source_uri, pID);
                    setDefaultFolderPresence(df, flag, protocol);
                 } else{
                    // ... but is in the DB --> error
                 }
             } else {
                 // is in the mail server
                 if (folderInDB != null){
                    // is in the DB --> do nothing
                    setDefaultFolderPresence(df, flag, protocol);
                 } else {
                    // is not in the DB --> insert in the DB
                    this.ed.insertDefaultFolderInDB(GUID, parentId,  folderName,
                                                    source_uri, pID);
                    setDefaultFolderPresence(df, flag, protocol);
                 }
             }

          } else {
             // POP3 PROTOCOL
             // folderInMS: for pop the only folder is the INBOX
             folderInDB = this.ed.getFullPathFromGUID(GUID, source_uri, pID);
             if (folderInDB == null) {
                // it is not in the DB --> insert in the DB
                this.ed.insertDefaultFolderInDB(GUID, parentId,  folderName,
                                                source_uri, pID);
                setDefaultFolderPresence(df, flag, protocol);
             }
          }


       } catch (EntityException ee){
           throw new EntityException("Error inserting the default folder", ee);
       }

    }


    /**
     *
     * insert the default folder if needed
     *
     *
     * @param df DefaultFolder
     * @param source_uri String
     * @param principalId long
     * @throws EntityException
     */
    private void  setDefaultFolderPresence(DefaultFolder df,
                                           String flag,
                                           String protocol)
      throws EntityException {


        if        (flag.equals("I")){
            if (protocol.equals(Def.PROTOCOL_IMAP)){
                df.setIsInboxInMailServer(true);
            } else {
                df.setIsInboxInMailServer(true);
            }
        } else if (flag.equals("O")){
            if (protocol.equals(Def.PROTOCOL_IMAP)){
                df.setIsOutboxInMailServer(true);
            } else {
                df.setIsOutboxInMailServer(true);
            }
        } else if (flag.equals("S")){
            if (protocol.equals(Def.PROTOCOL_IMAP)){
                df.setIsSentInMailServer(true);
            } else {
                df.setIsSentInMailServer(true);
            }
        } else if (flag.equals("D")){
            if (protocol.equals(Def.PROTOCOL_IMAP)){
                df.setIsDraftInMailServer(true);
            } else {
                df.setIsDraftInMailServer(false);
            }
        } else if (flag.equals("T")){
            if (protocol.equals(Def.PROTOCOL_IMAP)){
                df.setIsTrashInMailServer(true);
            } else {
                df.setIsTrashInMailServer(false);
            }
        }

    }

    /**
     *
     */
    private String[] createArray(ArrayList tmpIds){

        String[] allids = null;

        int l = tmpIds.size();
        allids = new String[l];

        for (int tt=0; tt<l; tt++){
            allids[tt] = (String)tmpIds.get(tt);
        }

        return allids;
    }


    /**
     *
     */
    private ArrayList getFilteredItemByID(HashMap si, String filterID){
        ArrayList tmpIds = new ArrayList();

        String GUID = null;
        Date   headerDate = null;
        boolean isEmail = false;
        CrcSyncItemInfo crcii = null;

        Iterator values = si.values().iterator();
        while (values.hasNext()) {
            crcii    = (CrcSyncItemInfo)values.next();
            GUID     = crcii.getKey().getKeyAsString();
            isEmail  = Utility.getBoolean(crcii.getIsEmail());
            if (GUID != null){
                if (!isEmail){
                    // folder
                    tmpIds.add(GUID);
                } else {
                    // email
                    if (GUID.equals(filterID)){
                        tmpIds.add(GUID);
                    }
                }
            }
        }
        return tmpIds;
    }

    /**
     *
     */
    private ArrayList getFilteredItemByTime(HashMap si, Date filterDate){

        ArrayList tmpIds = new ArrayList();

        String GUID = null;
        Date   headerDate = null;
        boolean isEmail = false;
        CrcSyncItemInfo crcii = null;

        Iterator values = si.values().iterator();
        while (values.hasNext()) {
            crcii      = (CrcSyncItemInfo)values.next();
            GUID       = crcii.getKey().getKeyAsString();
            headerDate = crcii.getHeaderDate();
            isEmail    = Utility.getBoolean(crcii.getIsEmail());
            if (GUID != null){
                if (!isEmail){
                    // folder
                    tmpIds.add(GUID);
                } else {
                    // email
                    if (headerDate != null){
                        if (Utility.d1Afterd2(headerDate, filterDate)){
                            tmpIds.add(GUID);
                        }
                    }
                }
            }
        }
        return tmpIds;
    }

}
