/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.items.manager;


import com.funambol.email.exception.EntityException;
import com.funambol.email.items.model.EmailFilter;
import com.funambol.email.items.model.ItemMessage;
import com.funambol.email.items.model.InternalPart;
import com.funambol.email.pdi.mail.Ext;
import com.funambol.email.util.Def;
import com.funambol.email.util.Utility;
import com.sun.mail.imap.IMAPNestedMessage;
import java.io.ByteArrayOutputStream;

import javax.mail.*;
import javax.mail.internet.*;
import javax.activation.DataHandler;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.*;

/**
 * <p>Filter class. </p>
 */

public class EntityManagerFilter extends EntityManager {
    
    
    
    //----------------------------------------------------------- PUBLIC METHODS
    
    
    /**
     * returns only header
     *
     * @param session opened mail server session
     * @param FID     parent folder id
     * @param FMID    mail id in the folder
     * @param msg     Message
     * @param status  status of the item; char
     * @param loc     Locale
     * @return ItemMessage
     * @throws EntityException
     */
    public ItemMessage get_H(Session session,
            String FID,
            String FMID,
            Message msg,
            char status,
            Locale loc,
            EmailFilter filter)
            throws EntityException {
        
        ItemMessage im = null;
        
        try {
            
            // save the flag
            boolean read      = msg.isSet(Flags.Flag.SEEN);
            boolean forwarded = false;
            boolean replied   = msg.isSet(Flags.Flag.ANSWERED);
            
            MimeMessage newMsg = new MimeMessage(session);
            
            // header
            String[] h = Utility.setNewHeader(newMsg, msg);
            String mid              = h[0];
            int    headersDimension = Integer.parseInt(h[1]);
            
            // default empty content
            newMsg.setContent(Def.CONTENT_BODY, Def.CONTENT_CONTENTTYPE);
            
            newMsg.saveChanges();
            
            // the save message command change the message-ID
            newMsg.setHeader(Def.HEADER_LABEL_MESSAGE_ID, mid);
            
            String encodingType = getEncodingType(newMsg);
            
            // get the info from original msg
            Ext ext = null;
            try {
               ArrayList parts = this.getAllPartsOfMessage(msg);
               ext = this.getExtComplete(parts);
            } catch (Exception e){
               log.severe("Error - Unsupported Encoding: " + e.getMessage());
               newMsg.setContent(Def.CONTENT_BODY_ALERT, Def.CONTENT_CONTENTTYPE);
            }                  
            
            im = setItemMessage(FMID, FID, newMsg,
                    read, forwarded, replied,
                    loc, encodingType, ext);
            
            // re-set the flag on the server
            msg.setFlag(Flags.Flag.SEEN, read);
            //msg.setFlag(Flags.Flag.SEEN, forwarded);
            msg.setFlag(Flags.Flag.ANSWERED,replied);
            
        } catch (MessagingException me) {
            throw new EntityException(me);
        }
        
        // remove the message if needed (syncml dimension optimization)
        if (Utility.needsFlagOptimization(status, FID, filter)) {
            im.setStreamMessage("");
        }
        
        return im;
        
    }
    
    
    /**
     *
     * returns header and body with byte quatity limitation
     *
     *
     * @param session  opened mail server session
     * @param FID      parent folder id
     * @param FMID     mail id in the folder
     * @param msg      Message
     * @param status   status of the item; char
     * @param loc      Locale
     * @param numBytes int
     * @return ItemMessage
     * @throws EntityException
     */
    public ItemMessage get_HB(Session session,
            String FID,
            String FMID,
            Message msg,
            char status,
            Locale loc,
            int numBytes,
            EmailFilter filter)
            throws EntityException {
        
        ItemMessage im = null;
        
        boolean needOpt = Utility.needsFlagOptimization(status, FID, filter);
        
        
        // keep the extension fields
        Ext ext = new Ext();
        
        
        try {
            
            // save the flag
            boolean read      = msg.isSet(Flags.Flag.SEEN);
            boolean forwarded = false;
            boolean replied   = msg.isSet(Flags.Flag.ANSWERED);
            
            MimeMessage newMsg = new MimeMessage(session);
            
            // header
            String[] h = Utility.setNewHeader(newMsg, msg);
            String mid              = h[0];
            int    headersDimension = Integer.parseInt(h[1]);
            
            // content
            if (needOpt){
                // set empty content (will be removed)
                newMsg.setContent(Def.CONTENT_BODY, Def.CONTENT_CONTENTTYPE);                
            } else if ((numBytes != -1) && (headersDimension > numBytes)) {                
                // set empty content because the header is too large
                newMsg.setContent(Def.CONTENT_BODY, Def.CONTENT_CONTENTTYPE);
                // get all parts for the ext fields
                try {
                   ArrayList parts = this.getAllPartsOfMessage(msg);
                   ext = this.getExtComplete(parts);
                } catch (Exception e){
                   log.severe("Error - Unsupported Encoding: " + e.getMessage());
                   newMsg.setContent(Def.CONTENT_BODY_ALERT, Def.CONTENT_CONTENTTYPE);
                }                  
            } else {                
                ArrayList parts = null;
                InternalPart v = null;                
                try {                    
                    parts = this.getAllPartsOfMessage(msg);
                    if (numBytes == -1){
                        // no filter for H+B (remove attachment eventually)
                        v = getPartForGetHB(parts, -1, ext);
                    } else {
                        // body will be truncated if needed
                        numBytes = numBytes - headersDimension;
                        v = getPartForGetHB(parts, numBytes, ext);
                    }
                    // set the content (will not be removed)
                    if (v != null){
                        newMsg.setContent(v.getPartValue(), v.getContentType());                    
                    } else {
                        newMsg.setContent(Def.CONTENT_BODY_ALERT, Def.CONTENT_CONTENTTYPE);
                    }
                } catch (Exception e){
                   log.severe("Error - Unsupported Encoding: " + e.getMessage());
                   newMsg.setContent(Def.CONTENT_BODY_ALERT, Def.CONTENT_CONTENTTYPE);
                }                 
            }
            
            newMsg.saveChanges();
            
            // the save message command change the message-ID
            newMsg.setHeader(Def.HEADER_LABEL_MESSAGE_ID, mid);
            
            String encodingType = getEncodingType(newMsg);
                        
            im = setItemMessage(FMID, FID, newMsg,
                    read, forwarded, replied,
                    loc, encodingType, ext);
            
            // re-set the flag on the server
            msg.setFlag(Flags.Flag.SEEN, read);
            //msg.setFlag(Flags.Flag.SEEN, forwarded);
            msg.setFlag(Flags.Flag.ANSWERED,replied);
            
        } catch (MessagingException me) {
            throw new EntityException(me);
        }
        
        // remove the message if needed (syncml dimension optimization)
        if (needOpt){
            im.setStreamMessage("");
        }
        
        return im;
        
    }
    
    /**
     *
     * if numBytes = -1 it returns all parts of the message
     * (header, body, attachements) without byte quatity limitation.
     *
     * if messageSize <= numBytes it returns all parts of the message
     * (header, body, attachements)
     *
     * otherwise it returns all parts of the message
     * (header, body, attachements) but there is the byte number limitation
     *
     *
     * @param session  opened mail server session
     * @param FID      parent folder id
     * @param FMID     mail id in the folder
     * @param msg      Message
     * @param status   status of the item; char
     * @param loc      Locale
     * @param numBytes int
     * @return ItemMessage
     * @throws EntityException
     */
    public ItemMessage get_HBA(Session session,
            String FID,
            String FMID,
            Message msg,
            char status,
            Locale loc,
            int numBytes,
            EmailFilter filter)
            throws EntityException {
        
        ItemMessage im = null;
        
        boolean needOpt = Utility.needsFlagOptimization(status, FID, filter);
        
        // keep the extension fields
        Ext ext = new Ext();        
        
        try {
            
            // save the flag
            boolean read      = msg.isSet(Flags.Flag.SEEN);
            boolean forwarded = false;
            boolean replied   = msg.isSet(Flags.Flag.ANSWERED);
            
            int messageSize   = msg.getSize();
                  
            if ( (numBytes == -1) || (messageSize <= numBytes) ){
                
                // entre message ; no ext fields
                String encodingType = getEncodingType((MimeMessage)msg);
                                
                im = setItemMessage(FMID, FID, msg,
                                    read, forwarded, replied,
                                    loc, encodingType, ext);
                
            } else {
                
                MimeMessage newMsg = new MimeMessage(session);
                
                // header
                String[] h = Utility.setNewHeader(newMsg, msg);
                String mid              = h[0];
                int    headersDimension = Integer.parseInt(h[1]);
                
                if (needOpt){
                    // optimization; set empty content (will be removed)
                    newMsg.setContent(Def.CONTENT_BODY, Def.CONTENT_CONTENTTYPE);
                } else{                    
                    if (headersDimension > numBytes){
                        // set empty content because the header is too large
                        newMsg.setContent(Def.CONTENT_BODY, Def.CONTENT_CONTENTTYPE);
                    } else {
                        // remove from numbytes the header dimension
                        numBytes = numBytes - headersDimension;

                        ArrayList parts = null;
                        try {                    
                            parts = this.getAllPartsOfMessage(msg);
                            // set filtered content (will not be removed)
                            ArrayList newContents = this.getPartsForGetHBA(parts, numBytes, ext);
                            if ( newContents.size() == 1){
                                BodyPart part = new MimeBodyPart();
                                InternalPart v = (InternalPart)newContents.get(0);
                                newMsg.setContent(v.getPartValue(), v.getContentType());
                            } else if (newContents.size() > 1){
                                BodyPart part;
                                MimeMultipart multipart = new MimeMultipart();
                                for (int i = 0; i < newContents.size(); i++) {
                                    InternalPart v = (InternalPart)newContents.get(i);
                                    part = new MimeBodyPart();
                                    part.setContent(v.getPartValue(), v.getContentType());
                                    if (newContents.size()>1){
                                        part.setDataHandler(v.getDHandler());
                                    }
                                    if (v.getDisposition() != null){
                                        part.setDisposition(v.getDisposition());
                                    }
                                    if (v.getFileName() != null){
                                        part.setFileName(v.getFileName());
                                    }
                                    multipart.addBodyPart(part);
                                }
                                newMsg.setContent(multipart);
                            } else {
                                // just in case of error
                                newMsg.setContent(Def.CONTENT_BODY, Def.CONTENT_CONTENTTYPE);
                            }
                        } catch (Exception e){
                           log.severe("Error - Unsupported Encoding: " + e.getMessage());
                           newMsg.setContent(Def.CONTENT_BODY_ALERT, Def.CONTENT_CONTENTTYPE);
                        }                                         
                    }
                }
                
                newMsg.saveChanges();
                
                // the save message command change the message-ID
                newMsg.setHeader(Def.HEADER_LABEL_MESSAGE_ID, mid);
                
                String encodingType = getEncodingType(newMsg);
                                
                im = setItemMessage(FMID, FID, newMsg,
                                    read, forwarded, replied,
                                    loc, encodingType, ext);
                
                // re-set the flag on the server
                msg.setFlag(Flags.Flag.SEEN, read);
                //msg.setFlag(Flags.Flag.SEEN, forwarded);
                msg.setFlag(Flags.Flag.ANSWERED,replied);
                
            }
            
                        
        } catch (MessagingException me) {
            throw new EntityException(me);
        }
        
        // remove the message if needed (syncml dimension optimization)
        if (needOpt){
            im.setStreamMessage("");
        }
        
        return im;
    }
    
    
    //---------------------------------------------------------- Private Methods

    /**
     *
     *
     */
    private ArrayList getAllPartsOfMessage(Message msg) {
                
        ArrayList ips = new ArrayList();
        
        try {
                        
            Part      part      = null;
            Multipart multipart = null;
            
            Object content      = msg.getDataHandler().getContent();
            if (content instanceof Multipart) {
                getMutliPartInfo(ips, msg, content);
            } else {
                InternalPart ip = createIP(msg);
                ips.add(ip);
            }           
                        
        } catch (MessagingException me){
            // @todo get the content using getTextBody(msg); then apply filter
            // if there is an Unsupported Encoding in the current version the
            // body will be empty
            log.severe("Error - Unsupported Encoding: " + me.getMessage());
        } catch (IOException ioe){
            // @todo get the content using getTextBody(msg); then apply filter
            // if there is an Unsupported Encoding in the current version the
            // body will be empty
            log.severe("Error - Unsupported Encoding: " + ioe.getMessage());
        }
              
        return ips;
        
    }
    
    
    /**
     *
     */
    private void getMutliPartInfo(ArrayList allParts, 
                                  Part part,
                                  Object currentContent) 
    throws MessagingException, IOException {
        
        Multipart multipart  = null;
        Object    contentInt = null;
        Part      partInt    = null;
        
        if (currentContent instanceof Multipart) {            
            multipart = (Multipart)currentContent;
            int num =  multipart.getCount();
            for (int i = 0; i<num; i++) {
                partInt    = multipart.getBodyPart(i);
                contentInt = partInt.getContent();
                getMutliPartInfo(allParts, partInt, contentInt);
            }    
        } else {
            InternalPart ip = createIP(part);
            allParts.add(ip);
        }        
    }    
    
    /**
     *
     */
    private InternalPart createIP(Part part) throws MessagingException, IOException {        
        InternalPart ip = new InternalPart();
        ip.setPartValue(part.getContent());
        ip.setContentType(part.getContentType());
        ip.setDHandler(part.getDataHandler());
        ip.setDisposition(part.getDisposition());
        ip.setFileName(part.getFileName());
        ip.setSize(part.getSize());        
        ip.setIsTextBody(isBodyTxt(ip));
        ip.setIsHtmlBody(isBodyHtml(ip));
        
        return ip;
        
    }
    
    /**
     * @todo
     * considera che ci pu essere solo l'html. quindi 
     * in quel caso devo settare come body un html
     *
     */
    private boolean isBodyTxt(InternalPart ip) {
        boolean isTextBody = false;
        if (ip.getContentType().indexOf("text/plain") != -1 ||
            ip.getContentType().indexOf("TEXT/PLAIN") != -1 ){
            if (ip.getFileName() == null){
                if (ip.getDisposition() == null){
                    isTextBody = true;
                } else {
                    // if Content-Disposition: inline
                    isTextBody = true;
                }
            }    
        }
        return isTextBody;
    }
    
    /**
     *
     */
    private boolean isBodyHtml(InternalPart ip) {
        boolean isHtmlBody = false;
        if (ip.getContentType().indexOf("text/html") != -1 ||
            ip.getContentType().indexOf("TEXT/HTML") != -1 ){
            if (ip.getFileName() == null){
                if (ip.getDisposition() == null){
                    isHtmlBody = true;
                } else {
                    // if Content-Disposition: inline
                    isHtmlBody = true;
                }
            }    
        }
        return isHtmlBody;
    }
    
    /**
     *
     * this method checks if message contains non-ASCII characters
     * and set the correct encoding type
     * It's a public method because there is an external unit test
     *
     * @param msg Message
     * @return encodingType String
     */
    private String getEncodingType(MimeMessage msg) throws EntityException {        
        String encodingType      = null;
        encodingType = Def.ENCODE_QUOTED_PRINTABLE;        
        return encodingType;
    }               
    
    /**
     * Utility method for obtaining a string representation of a
     * Message's body
     */
    private static String getTextBody(MimeMessage message) throws Exception {
        
        InputStream  bis  = null;
        OutputStream bos  = null;
        ByteArrayOutputStream bodyOs = new ByteArrayOutputStream();
        byte[] block = new byte[1024];
        int read = 0;
        
        try {
            bis = message.getRawInputStream();
            bos = bodyOs;
            while ((read = bis.read(block)) > -1) {
                bos.write(block, 0, read);
            }
            bos.flush();
            return bodyOs.toString();
        } catch(MessagingException me) {
            throw me;
        } finally {
            bis.close();
        }
        
    }
    
    
    /**
     * create an Ext object with the following info
     * - truncated
     * - body size
     * - all info about attachments
     *
     * @param part list ArrayList
     * @return ext Ext
     */
    private Ext getExtComplete(ArrayList ips){
        
        Ext ext = new Ext();
        
        try {
            
            // truncated
            ext.setTruncated(true); 
            
            // body & attachments
            ArrayList xvalList = new ArrayList();
            InternalPart item = null;
            String[] value = new String[2];
            for (int i=0; i<ips.size(); i++){
                
                 item = (InternalPart)ips.get(i);
                 
                 if (item.isTextBody()){
                     ext.setXvalBody(""+item.getSize());
                 }
                 
                 if (item.getDisposition() != null && 
                         item.getDisposition().equals("attachment")){
                     value[0] = item.getFileName();
                     value[1] = ""+item.getSize();
                     xvalList.add(value); 
                 }
            }
            ext.setXvalAttach(xvalList);                        
            
            
        } catch (Exception e){
            e.printStackTrace();
        }
              
        return ext;
        
    }
            
    
    
    /**
     *
     *
     *
     */
    private ArrayList getPartsForGetHBA(ArrayList ips, int numBytes, Ext ext)
      throws EntityException {
        
        InternalPart item                 = null;
        ArrayList    newMsgParts          = new ArrayList();
        boolean      isTextBodyAlreadyGot = false;        
        boolean      isHtmlBodyAlreadyGot = false;        
        int          numBytesTot          = 0;
        int          htmlBodySize         = 0;
        String[]     value                = new String[2];        
        ArrayList    xvalList             = new ArrayList();
        int          origSize             = 0;
        
        for (int i=0; i<ips.size(); i++){
                
             item = (InternalPart)ips.get(i);
             origSize = item.getSize();
             
             if (numBytesTot > numBytes){
                 
                 break;
                 
             } else {
                 
                 if (item.isTextBody()){
                     if ( numBytes < (numBytesTot + item.getSize()) ){
                         // truncated body
                         newMsgParts.add(truncBody(item, numBytes));
                         ext.setTruncated(true);                     
                         ext.setXvalBody(""+origSize);                         
                     } else {
                         // entire body
                         newMsgParts.add(item);                         
                     }
                     if (isHtmlBodyAlreadyGot){                     
                         numBytesTot = numBytesTot - htmlBodySize;
                         numBytesTot = numBytesTot + item.getSize();
                     } else {
                         numBytesTot = numBytesTot + item.getSize();
                     }
                     isTextBodyAlreadyGot = true;                 
                 }
                 if (item.isHtmlBody()){
                     if (!isTextBodyAlreadyGot){
                         if ( numBytes < (numBytesTot + item.getSize()) ){
                             // truncated body
                             newMsgParts.add(truncBody(item, numBytes));
                             ext.setTruncated(true);                     
                             ext.setXvalBody(""+origSize);
                         } else {
                             // entire body
                             newMsgParts.add(item);
                         }                 
                         isHtmlBodyAlreadyGot = true;                 
                         htmlBodySize = item.getSize();
                         numBytesTot = numBytesTot + item.getSize();                     
                     }
                 }
                 if (item.getDisposition() != null && 
                     item.getDisposition().equals("attachment")){

                     if ( numBytes < (numBytesTot + item.getSize()) ){
                         // don't get the attachment, but set the ext
                         ext.setTruncated(true);                     
                         value[0] = item.getFileName();
                         value[1] = ""+item.getSize();
                         xvalList.add(value); 
                     } else {
                         // don't set the ext, but get the attachment
                         newMsgParts.add(item);
                     }
                 }                 
             }                         
        }
        
        ext.setXvalAttach(xvalList);
        
        return newMsgParts;
        
    }
    
    
    /**
     *
     *
     *
     */
    private InternalPart getPartForGetHB(ArrayList ips, int numBytes, Ext ext)
      throws EntityException {
        
        boolean      isTextBodyAlreadyGot = false;
        InternalPart item                 = null;
        InternalPart itemOut              = null;
        String[]     value                = new String[2];        
        ArrayList    xvalList             = new ArrayList();
        int          origSize             = 0;
        
        for (int i=0; i<ips.size(); i++){
                
             item = (InternalPart)ips.get(i);
             origSize = item.getSize();

             if (item.isTextBody()){
                if (numBytes != -1){
                    // truncated
                    itemOut = truncBody(item, numBytes);
                    ext.setTruncated(true);
                    ext.setXvalBody(""+origSize);
                } else {
                    // entire
                    itemOut = item;
                }
                isTextBodyAlreadyGot = true;                 
             }
             if (item.isHtmlBody()){
                 if (!isTextBodyAlreadyGot){
                    if (numBytes != -1){
                        // truncated
                        itemOut = truncBody(item, numBytes);
                        ext.setTruncated(true);
                        ext.setXvalBody(""+origSize);
                    } else {
                        // entire
                        itemOut = item;
                    }
                 }
             }
             if (item.getDisposition() != null && 
                 item.getDisposition().equals("attachment")){
                 value[0] = item.getFileName();
                 value[1] = ""+item.getSize();
                 xvalList.add(value); 
             }
                         
        }
        
        ext.setXvalAttach(xvalList);

        return itemOut;
        
    }
    
    
    /**
     *
     */
    private InternalPart truncBody(InternalPart item, int numBytes)
      throws EntityException {

        try {
            String partContent = "";
            byte[] partBytes   = null;
            if (item.getPartValue() != null) {
                partContent = String.valueOf(item.getPartValue().toString());
                partBytes = partContent.getBytes();
                if (numBytes < partBytes.length) {
                    item.setPartValue(new String(partBytes, 0, numBytes));
                    item.setSize(numBytes);
                }
            }             
        }catch (Exception e){
            throw new EntityException ("Error cutting the email body " + e.getMessage());
        }
                
        return item;
    }
    
}
