/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.items.manager ;


import com.funambol.email.engine.cache.CrcSyncItemInfo;
import com.funambol.email.exception.EmailAccessException;
import com.funambol.email.exception.EntityException;
import com.funambol.email.exception.SendingException;
import com.funambol.email.items.dao.PopEntityDAO;
import com.funambol.email.items.model.EmailFilter;
import com.funambol.email.items.model.ItemFolder;
import com.funambol.email.items.model.ItemMessage;
import com.funambol.email.pdi.mail.Email;
import com.funambol.email.transport.IMailServerWrapper;
import com.funambol.email.transport.PopMailServerWrapper;
import com.funambol.email.pdi.folder.Folder;
import com.funambol.email.util.Def;
import com.funambol.email.util.Utility;
import com.funambol.foundation.pdi.common.Property;
import com.funambol.server.tools.IdSpaceGenerator;
import com.sun.mail.pop3.POP3Folder;


import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Session;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.logging.Level;
import javax.mail.Flags;
import javax.mail.internet.MimeMessage;


/**
 * @version $Id: PopEntityManager.java,v 1.49.2.1 2006/11/29 15:44:20 gbmiglia Exp $
 */
public class PopEntityManager extends EntityManagerFilter {

    // -------------------------------------------------------------- Properties
    
    /**     */
    public PopMailServerWrapper pmsw = null;

    /**     */
    public PopEntityDAO ped = null;

    // ------------------------------------------------------------ Costructors


    /**
     *
     */
    public PopEntityManager() {
    }

    /**
     * @param msw_e              IMailServerWrapper
     * @param jndiDataSourceName String
     * @throws EmailAccessException
     */
    public PopEntityManager(IMailServerWrapper msw_e, String jndiDataSourceName)
            throws EmailAccessException {
        pmsw = (PopMailServerWrapper) msw_e;
        ped = new PopEntityDAO(jndiDataSourceName);
    }

    // ---------------------------------------------------------- Public Methods


    /**
     *
     * In the pop3 protocol this method is used only for outbox mail
     * this mails will be add to the Local DB and then sent.
     *
     * @param FID  parent folder id
     * @param emailNew    Email
     * @param idSentSpace manager for the id creation IdSpaceGenerator
     * @param saveOnlyHeader true if all the email is saved in the local DB
     * @param source_uri  String
     * @param principalId long
     * @return Email
     * @throws EntityException
     */
    public Email addEmail(String FID,
                          Email emailNew,
                          IdSpaceGenerator idSentSpace,
                          boolean saveOnlyHeader,
                          String source_uri,
                          long principalId)
      throws EntityException, SendingException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("PopEmail insertEmail start");
        }

        Email emailOut = null;

        // when a message is sent it will be save as a sent email in the local DB
        // this method is used only for outbox folder
        FID = Def.FOLDER_SENT_ID;

        // get flags
        boolean[] flags = this.setFlags(emailNew);
        boolean read      = flags[0];
        boolean forwarded = flags[1];
        boolean replied   = flags[2];

        if (emailNew != null && emailNew.getEmailItem() != null &&
                Def.ENCODE_QUOTED_PRINTABLE.equals(emailNew.getEmailItem().getEncoding())) {
            try {
                emailNew.getEmailItem().setPropertyValue(
                        Utility.decode(emailNew.getEmailItem().getStringRFC2822(),
                                Def.ENCODE_QUOTED_PRINTABLE));
                emailNew.getEmailItem().setEncoding(null);
            }
            catch (MessagingException e) {
                throw new EntityException(e);
            }
        }

        try {

            Message msg = createMessage(this.pmsw.getSession(), emailNew,
                                        FID, false, read, forwarded, replied);

            if (msg != null){

               String messageID = Utility.getHeaderMessageID(msg);

               // send mail
               sendEmail(msg);

               msg.setHeader(Def.HEADER_LABEL_MESSAGE_ID,messageID);
                  
               /**
                * add email to local DB
                *
                * in the local db we save the header only (for privacy)
                * the parameter 'saveOnlyHeader' will set in the 
                * SyncConnectorConfigPanel
                *
                */ 
               Message msgForDB       = null;
               String GUID            = null;
               if (saveOnlyHeader){
                  msgForDB= createMessageForDB(msg); 
                  GUID = ped.addSentEmail(FID, messageID, msgForDB,  idSentSpace,
                                                 source_uri, principalId);
               } else {
                  GUID = ped.addSentEmail(FID, messageID, msg,  idSentSpace,
                                                 source_uri, principalId);
               }
               
               // 
               GUID = Utility.convertSentIDToOutboxID(GUID);

               // the returned id starts with "O" (for mapping purpose)
               ItemMessage im = setItemMessage(GUID, Def.FOLDER_OUTBOX_ID, msg,
                                               read, forwarded, replied,
                                               this.pmsw.getLocale(), null, null);

               emailOut = createFoundationMail(im);

            }

        } catch (MessagingException e) {
            throw new EntityException("Error settings Message-ID", e);
        } catch (EntityException e) {
            throw new EntityException(e);
        }  catch (SendingException e) {
            throw new SendingException(e);
        }

        return emailOut;

    }

    /**
     * clean all folder and email except the 5 main folders
     *
     * @param source_uri  String
     * @param principalId long
     * @throws EntityException
     */
    public void clearAllItems(String source_uri, long principalId)
            throws EntityException {

        POP3Folder f = null;
        if (log.isLoggable(Level.FINEST)) {
            log.finest("Pop3Email Delete all emails in Inbox Folder");
        }

        try {

            // we can remove only the inbox folder email
            f = this.pmsw.getInboxFolder();
            timeStart = System.currentTimeMillis();
            f.open(javax.mail.Folder.READ_WRITE);
            this.ped.removeAllEmail(f);

            // the local sent messages is already deleted.

            // we can't remove all folder

        } catch (MessagingException me) {
            throw new EntityException(me);
        } catch (EmailAccessException e) {
            throw new EntityException(e);
        } finally {
            if (f != null) {
                try {
                    if (f.isOpen()) {
                        f.close(true);
                        timeStop = System.currentTimeMillis();
                        if (log.isLoggable(Level.FINEST)) {
                            log.finest("insertItem Execution Time: " +
                                    (timeStop - timeStart) + " ms");
                        }
                    }
                } catch (MessagingException me) {
                    throw new EntityException(me);
                }
            }
        }
    }


    /**
     * deletes an email
     *
     * @param parentId  folder id
     * @param mailId  mail id in the folder.
     * @param GUID  GUID in the ds-server.
     * @param serverItems map with all CrcSyncItemInfo
     * @param source_uri  String
     * @param principalId long
     * @throws EntityException
     */
    public void removeEmail(EmailFilter filter,
                            String parentId,
                            String mailId,
                            String GUID,
                            HashMap serverItems,
                            String username,
                            String source_uri,
                            long principalId)
      throws EntityException {


        if (parentId.equalsIgnoreCase(Def.FOLDER_INBOX_ID)) {

            int threshold_1 = filter.getMaxInboxNum();
            int threshold_2 = (filter.getMaxInboxNum() + 40);
            int max = filter.getMaxInboxNum();
            
            try {

                timeStart = System.currentTimeMillis();                
                
                Message msg = getItem(mailId, max, threshold_1, threshold_2);
                
                if (msg != null) {
                    // remove from Mail Server               
                    ped.removeEmail(msg);
                    // remove from Inbox Cache Table
                    ped.removeEmailFromInbox(username, Def.PROTOCOL_POP3, GUID);
                    // remove serverItemsList
                    ped.removeEmailFromServerItems(GUID, serverItems);
                }

                timeStop = System.currentTimeMillis();                
                if (log.isLoggable(Level.FINEST)) {
                    log.finest("removeItem Execution Time: " + (timeStop - timeStart) + " ms");
                }
                
            } catch (EntityException e) {
                throw new EntityException(e);
            } 
            
        }


        if (parentId.equalsIgnoreCase(Def.FOLDER_SENT_ID)
            ||
            parentId.equalsIgnoreCase(Def.FOLDER_OUTBOX_ID)) {
            try {
                ped.removeEmail(GUID, source_uri, principalId);
            } catch (Exception me) {
                throw new EntityException(me);
            }
        }

    }
    
    /**
     * @param filter      EmailFilter
     * @param source_uri  String
     * @param principalId long
     * @return map with all the mail server CrcSyncItemInfo
     * @throws EntityException
     */
    public HashMap getAllEmailsInfo(EmailFilter filter,
                                              String username,
                                              String source_uri,
                                              long principalId)
            throws EntityException {
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("getAllSyncItemInfo Start");
        }

        HashMap syncItemInfos = null;
        HashMap iInfos        = null;
        HashMap sInfos        = null;

        try {
            
            iInfos = this.getAllEmailsInfoInbox(username);

            if (source_uri == null && principalId == 0){
                // used by the test suite                
            } else {
                sInfos = this.getAllEmailsInfoSent(filter, source_uri, principalId);
            }
            
        } catch (Exception e) {
            throw new EntityException("Error Getting all emails info (" +
                    e.getMessage() + ")");
        }

        syncItemInfos = Utility.mergeHashMap(iInfos, null, null, sInfos, null);

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getAllSyncItemInfo List: " + syncItemInfos.size() + " Document");
        }

        return syncItemInfos;

    }

    /**
     * the pop-like connector gets only the default folders
     * (inbox, outbox, sent)
     *
     * @return The info of the syncitems 
     * @throws EntityException
     */
    public HashMap getAllFoldersInfo() throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getAllFolderInfo Start");
        }

        HashMap syncItemInfos = new HashMap();
        CrcSyncItemInfo info  = null;

        info = new CrcSyncItemInfo(Def.FOLDER_INBOX_GUID, 0, 
                                   null,null,null,
                                   null,null, null, null);
        syncItemInfos.put(Def.FOLDER_INBOX_GUID, info);

        info = new CrcSyncItemInfo(Def.FOLDER_OUTBOX_GUID, 0,                 
                                   null,null,null,
                                   null,null, null, null);
        syncItemInfos.put(Def.FOLDER_OUTBOX_GUID, info);

        info = new CrcSyncItemInfo(Def.FOLDER_SENT_GUID, 0,                 
                                   null,null,null,
                                   null,null, null, null);
        syncItemInfos.put(Def.FOLDER_SENT_GUID, info);

        return syncItemInfos;

    }

   /**
    *
    * checks if an email is already inserted in the db
    * and returns the GUID
    * we use the header Message-ID as search criteria
    *
    * @param session mail server session
    * @param parentId parent if of the email
    * @param ctmp Email
    * @param source_uri String
    * @param principalId long
    * @return GUID String
    * @throws EntityException
    */
   public String getEmailFromClauseInDB(Session session,
                                        String parentId,
                                        Email ctmp,
                                        String source_uri,
                                        long principalId)
      throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getItemByWhereClause start ");
        }

        String      GUID         = null;
        ItemMessage im           = null;
        String      messageIdExt = null;
        Message     msgExt       = null;


        // create the Message
        try {
            boolean[] flags = this.setFlags(ctmp);
            boolean read      = flags[0];
            boolean forwarded = flags[1];
            boolean replied   = flags[2];
            msgExt = createMessage(this.pmsw.getSession(), ctmp,  parentId,
                                   false, read, forwarded, replied);
        } catch (Exception e) {
            throw new EntityException(e);
        }

        // get the Message-ID
        try {
            messageIdExt = Utility.getHeaderMessageID(msgExt);

            if (messageIdExt != null) {
                im = ped.getEmailFromMessageID(messageIdExt,
                                               this.pmsw.getSession(),
                                               source_uri,
                                               principalId);
                if (im != null){
                   GUID = im.getGUID();
                }
            }
        } catch (Exception e) {
            throw new EntityException(e);
        }

        return GUID;
    }
       
    /**
     *
     * checks if a mail match the filter
     * @param email  Email
     * @param filter EmailFilter
     * @param GUID String
     * @param FID String
     * @param FMID String
     * @param status
     * @param source_uri
     * @param principalId
     * @return true if the message match the filter
     * @throws EntityException
     */
    public boolean isEmailInFilter(Email email,
                                   EmailFilter filter,
                                   String GUID,
                                   String FID,
                                   String FMID,
                                   char status,
                                   String source_uri,
                                   long principalId)
            throws EntityException {
        
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("isEmailInFilter in folder " + FID + 
                       " GUID: " + GUID + " FMID " + FMID);
        }
        
        boolean isInFilter = false;
        
        try {
            
            // get email if null
            if (email == null){
                email = this.getEmailFromUID(filter, FID, FMID, status, 
                                             source_uri, principalId);
            } 
            
            isInFilter = super.isEmailInFilter(email, filter,
                                               GUID, FID, FMID,
                                               this.pmsw.getSession(),
                                               this.pmsw.getLocale(),
                                               source_uri,principalId);
            
            
        } catch (Exception e) {
            throw new EntityException(e);
        }
        
        return isInFilter;
    }
   
   
    /**
     * Get the email; called by the SyncSource
     *
     * @param filter      EmailFilter
     * @param parentId    id parent folder (folderid / mailid)
     * @param mailId      mail id (folderid / mailid)
     * @param status      status of the item; char
     * @param source_uri  String
     * @param principalId long
     * @return Email
     * @throws EntityException
     */
    public Email getEmailFromUID(EmailFilter filter,
                                 String parentId,
                                 String mailId,
                                 char status,
                                 String source_uri,
                                 long principalId)
       throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getItemByUID in folder " + parentId + " mailid " + mailId);
        }
 
        Email       emailOut = null;
        ItemMessage im       = null;

        // go in the Mail Server
        if (parentId.equalsIgnoreCase(Def.FOLDER_INBOX_ID)) {
            im = this.getEmailFromUIDInbox(parentId, mailId, filter, status,
                                           source_uri, principalId);
        }

        // go in the local DB
        if (parentId.equalsIgnoreCase(Def.FOLDER_SENT_ID) ||
            parentId.equalsIgnoreCase(Def.FOLDER_OUTBOX_ID)) {
            im = this.getEmailFromUIDSent(parentId, mailId, filter,
                                          source_uri, principalId);
        }

        if (im != null) {
            try {
                emailOut = createFoundationMail(im);
            } catch (Exception e) {
                throw new EntityException(e);
            }
        }

        return emailOut;

    }

    /**
     * the method doesn't update the item but get the item that must be return
     *
     * @param FID  folder id
     * @param FMID  folder id
     * @param emailNew  Email Object
     * @param source_uri  String
     * @param principalId long
     * @return Email
     * @throws EntityException
     */
    public Email updateEmail(String FID,
                             String FMID,
                             Email emailNew,
                             String source_uri,
                             long principalId)
     throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("updateEmail start FID " + FID + " FMID " + FMID);
        }

        Email emailOut = null;
        
        String GUID = Utility.createPOPGUID(FID, FMID);
        
        emailOut = emailNew;
        
        emailOut.setUID(new Property(GUID));
                
        return emailOut;

    }


    /**
     * updates a folder defined by folder id and parent id
     *
     * @param parentId  folder id String
     * @param folderId  mail id in folder String
     * @param folderNew Folder
     * @return Folder
     * @throws EntityException
     */
    public com.funambol.email.pdi.folder.Folder updateFolder(String parentId,
                                                             String folderId,
                                                             Folder folderNew)
      throws EntityException {

        com.funambol.email.pdi.folder.Folder folderOut = null;
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("updateFolder start for FID: " + folderId);
        }
        
        String name       = folderNew.getName().getPropertyValueAsString();
        String created    = folderNew.getCreated().getPropertyValueAsString();
        String modified   = folderNew.getModified().getPropertyValueAsString();
        String accessed   = folderNew.getAccessed().getPropertyValueAsString();
        String attributes = null;
        
        ItemFolder ifout = null;
        
        if        (folderId.equals(Def.FOLDER_INBOX_ID)){
            ifout =  new ItemFolder(Def.FOLDER_INBOX_GUID, 
                                    name, 
                                    Def.FOLDER_ROOT_ID, 
                                    created, 
                                    accessed, 
                                    modified, 
                                    Def.FOLDER_INBOX_ROLE, 
                                    attributes); 
        } else if (folderId.equals(Def.FOLDER_OUTBOX_ID)){
            ifout =  new ItemFolder(Def.FOLDER_OUTBOX_GUID, 
                                    name, 
                                    Def.FOLDER_ROOT_ID, 
                                    created, 
                                    accessed, 
                                    modified, 
                                    Def.FOLDER_OUTBOX_ROLE, 
                                    attributes) ;
        } else if (folderId.equals(Def.FOLDER_SENT_ID)){
            ifout =  new ItemFolder(Def.FOLDER_SENT_GUID, 
                                    name, 
                                    Def.FOLDER_ROOT_ID, 
                                    created, 
                                    accessed, 
                                    modified, 
                                    Def.FOLDER_SENT_ROLE, 
                                    attributes) ; 
        } else if (folderId.equals(Def.FOLDER_DRAFTS_ID)){
            ifout =  new ItemFolder(Def.FOLDER_DRAFTS_GUID, 
                                    name, 
                                    Def.FOLDER_ROOT_ID, 
                                    created, 
                                    accessed, 
                                    modified, 
                                    Def.FOLDER_DRAFTS_ROLE, 
                                    attributes) ; 
        } else if (folderId.equals(Def.FOLDER_TRASH_ID)){
            ifout =  new ItemFolder(Def.FOLDER_TRASH_GUID, 
                                    name, 
                                    Def.FOLDER_ROOT_ID, 
                                    created, 
                                    accessed, 
                                    modified, 
                                    Def.FOLDER_TRASH_ROLE, 
                                    attributes) ; 
            
        }
        
        // convert Folder in foundation Folder
        try {
            folderOut = createFoundationFolder(ifout);
        } catch (Exception e) {
            throw new EntityException(e);
        }
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("updateFolder end");
        }

        return folderOut;
    }


    /**
     * @param GUID        folder id
     * @param source_uri  String
     * @param principalId long
     * @return Folder
     * @throws EntityException
     */
    public com.funambol.email.pdi.folder.Folder getFolderFromUID(String GUID,
                                                                 String source_uri,
                                                                 long principalId)
            throws EntityException {


        if (log.isLoggable(Level.FINEST)) {
            log.finest("getFolderByUID start " + GUID);
        }

        com.funambol.email.pdi.folder.Folder folderOut;
        ItemFolder itemFolder = null;
        String creationDate;


        try {

            creationDate = Utility.getFolderCreationDate();

            if (GUID.equalsIgnoreCase(Def.FOLDER_INBOX_GUID)) {
                itemFolder = new ItemFolder(GUID,
                        this.pmsw.getDefaultFolder().getInboxName(),
                        Def.FOLDER_ROOT_ID,
                        creationDate,
                        null,
                        null,
                        Def.FOLDER_INBOX_ROLE,
                        null);
            } else if (GUID.equalsIgnoreCase(Def.FOLDER_OUTBOX_GUID)) {
                itemFolder = new ItemFolder(GUID,
                        this.pmsw.getDefaultFolder().getOutboxName(),
                        Def.FOLDER_ROOT_ID,
                        creationDate,
                        null,
                        null,
                        Def.FOLDER_OUTBOX_ROLE,
                        null);

            } else if (GUID.equalsIgnoreCase(Def.FOLDER_SENT_GUID)) {
                itemFolder = new ItemFolder(GUID,
                        this.pmsw.getDefaultFolder().getSentName(),
                        Def.FOLDER_ROOT_ID,
                        creationDate,
                        null,
                        null,
                        Def.FOLDER_SENT_ROLE,
                        null);
            }

            folderOut = createFoundationFolder(itemFolder);

        } catch (Exception e) {
            throw new EntityException(e);
        }

        return folderOut;

    }

    /**
     * @param name        folder name
     * @param parentId    parent folder id
     * @param source_uri  String
     * @param principalId long
     * @return Folder
     * @throws EntityException
     */
    public com.funambol.email.pdi.folder.Folder getFolderFromName(String name,
                                                                  String parentId,
                                                                  String source_uri,
                                                                  long principalId)
            throws EntityException {


        if (log.isLoggable(Level.FINEST)) {
            log.finest("getFolderByName start " + name + " FID " + parentId);
        }

        com.funambol.email.pdi.folder.Folder folderOut = null;
        ItemFolder itemFolder = null;
        String creationDate;

        /**
         * we should add some control.
         * We can use the device information
         */
        String nametmp = Utility.convertNokiaToDefault(name,
                this.pmsw.getDefaultFolder());
        // it is a nokia folder
        if (nametmp != null) {
            name = nametmp;
        }

        try {

            creationDate = Utility.getFolderCreationDate();

            if (name.equalsIgnoreCase(this.pmsw.getDefaultFolder().getInboxName())) {
                itemFolder = new ItemFolder(Def.FOLDER_INBOX_GUID,
                        this.pmsw.getDefaultFolder().getInboxName(),
                        Def.FOLDER_ROOT_ID,
                        creationDate,
                        null,
                        null,
                        Def.FOLDER_INBOX_ROLE,
                        null);
            } else if (name.equalsIgnoreCase(this.pmsw.getDefaultFolder().getOutboxName())) {
                itemFolder = new ItemFolder(Def.FOLDER_OUTBOX_GUID,
                        this.pmsw.getDefaultFolder().getOutboxName(),
                        Def.FOLDER_ROOT_ID,
                        creationDate,
                        null,
                        null,
                        Def.FOLDER_OUTBOX_ROLE,
                        null);

            } else if (name.equalsIgnoreCase(this.pmsw.getDefaultFolder().getSentName())) {
                itemFolder = new ItemFolder(Def.FOLDER_SENT_GUID,
                        this.pmsw.getDefaultFolder().getSentName(),
                        Def.FOLDER_ROOT_ID,
                        creationDate,
                        null,
                        null,
                        Def.FOLDER_SENT_ROLE,
                        null);
            }

            if (itemFolder != null) {
                try {
                    folderOut = createFoundationFolder(itemFolder);
                } catch (Exception e) {
                    throw new EntityException(e);
                }
            }

        } catch (Exception e) {
            throw new EntityException(e);
        }


        return folderOut;

    }

    /**
     *
     * Used in the stardard process (called by PopEntityManager.getEmailFormUID)
     *
     * @param FMID        mail id (in the folder, not unique)
     * @param max max number of email
     * @param threshold_1 max number for the first loop
     * @param threshold_2  max number for the second loop
     * @return ItemMessage
     * @throws EntityException
     */
    private Message getItem(String FMID, 
                            int max,
                            int threshold_1, 
                            int threshold_2)
    throws EntityException {
        
        Message msg     = null;
        int     counter = 1; 
        boolean loop    = true;
   
        if (log.isLoggable(Level.FINEST)) {
            log.finest("getItem Start");
        }
        
        timeStart = System.currentTimeMillis();                        
        
        while(loop){

            if (this.pmsw.messagesInboxOpened == null){
                 this.pmsw.messagesInboxOpened = getItemLoopManager(
                                                   this.pmsw.folderInboxOpened, 
                                                   max, counter, 
                                                   threshold_1, threshold_2);
            }

            msg = ped.getEmailInboxFromUID(this.pmsw.folderInboxOpened, 
                                           this.pmsw.messagesInboxOpened, 
                                           FMID);
            
            if (msg != null){
                // msg found
                loop = false;
            } else {
                // msg null; Not found
                if (counter == 4){
                    loop = false;
                } else {
                    counter++;
                    this.pmsw.messagesInboxOpened = null;
                }
            }
                        
        }
        
        timeStop = System.currentTimeMillis();                
        if (log.isLoggable(Level.FINEST)) {
            log.finest("getItem (with loop optimiztion) Execution Time: " + 
                    (timeStop - timeStart) + " ms");
        }
                                
        return msg;
        
    }

    /**
     * Used in the stardard process (called by PopEntityManager.getEmailFormUID)
     *
     * @param FID         parent folder id
     * @param FMID        mail id (in the folder, not unique)
     * @param filter      EmailFilter
     * @param status      status of the item; char
     * @param source_uri  String
     * @param principalId long
     * @return ItemMessage
     * @throws EntityException
     */
    private ItemMessage getEmailFromUIDInbox(String FID,
                                             String FMID,
                                             EmailFilter filter,
                                             char status,
                                             String source_uri,
                                             long principalId)
     throws EntityException {

        ItemMessage im = null;
        int threshold_1 = 20;
        int threshold_2 = 40;
        int max = filter.getMaxInboxNum();
        
        try {

            if (this.pmsw.folderInboxOpened == null){
                 this.pmsw.folderInboxOpened = 
                    (POP3Folder) this.pmsw.getMailDefaultFolder().getFolder(Def.FOLDER_INBOX_ENG);
            }

            long tStart = System.currentTimeMillis();
            if (!this.pmsw.folderInboxOpened.isOpen()){
               this.pmsw.folderInboxOpened.open(javax.mail.Folder.READ_WRITE);
            }
            long tStop = System.currentTimeMillis();
            if (log.isLoggable(Level.FINEST)) {
                log.finest("getEmailInboxFromUID opening pop3 folder Execution Time: " + (tStop - tStart) + " ms");
            }    

            
            // optimization
            tStart = System.currentTimeMillis();
            Message msg = getItem(FMID, max, threshold_1, threshold_2);
            tStop = System.currentTimeMillis();
            if (log.isLoggable(Level.FINEST)) {
                log.finest("getEmailInboxFromUID getting message Execution Time: " + (tStop - tStart) + " ms");
            }    
            
            // handling the filtering error in a different way
            // in order to avoid to analize the same mail in the next sync session
            try {
                tStart = System.currentTimeMillis();
                im = this.getPart(this.pmsw.folderInboxOpened, 
                                  FMID, filter, msg, status, 
                                  source_uri, principalId);
                tStop = System.currentTimeMillis();
                if (log.isLoggable(Level.FINEST)) {
                    log.finest("getEmailInboxFromUID filtering single mail Execution Time: " + (tStop - tStart) + " ms");
                }    
            } catch (EntityException e) {
                throw new EntityException(Def.ERROR_FILTERING_EMAIL + " " + e.getMessage());
            }
            
        } catch (Exception e) {
            throw new EntityException(e);
        } 
        
        return im;
        
    }

    /**
     * @param FID         String
     * @param FMID        String
     * @param filter      EmailFilter
     * @param source_uri  String
     * @param principalId long
     * @return ItemMessage
     * @throws EntityException
     */
    private ItemMessage getEmailFromUIDSent(String FID,
                                            String FMID,
                                            EmailFilter filter,
                                            String source_uri,
                                            long principalId)
            throws EntityException {

        ItemMessage im;
        String encodingType;
        boolean asciiOnly;

        try {

            String GUID = Utility.createPOPGUID(FID, FMID);

            Message msg = ped.getEmailSentFromUID(GUID, this.pmsw.getSession(),
                                                  source_uri, principalId);

            // save the flag
            boolean read      = msg.isSet(Flags.Flag.SEEN);
            boolean forwarded = false;
            boolean replied   = msg.isSet(Flags.Flag.ANSWERED);

            im = setItemMessage(GUID, FID, msg, read, forwarded, replied,
                                this.pmsw.getLocale(), null, null);

        } catch (Exception e) {
            throw new EntityException(e);
        }

        return im;
    }

    /**
     * the pop-like connector gets only the default folders
     * (inbox, outbox, sent)
     *
     * @return the array with the folder ids String[]
     * @throws EntityException
     */
    public String[] getAllFolders() throws EntityException {

        return new String[]{
                Def.FOLDER_INBOX_GUID,
                Def.FOLDER_OUTBOX_GUID,
                Def.FOLDER_SENT_GUID
        };

    }


    /**
     * get messages from the fnbl_email_inbox
     * this table it's a cache for the email in the inbox folder
     *
     * @param username  String
     * @return map with all the mail server CrcSyncItemInfo
     * @throws EntityException
     */
    private HashMap getAllEmailsInfoInbox(String username)
      throws EntityException {

        HashMap allSyncItemInfo = null;        
        try {
            allSyncItemInfo = ped.getAllEmailsInbox(username, Def.PROTOCOL_POP3);            
        } catch (EntityException e) {
            throw new EntityException(e);
        }

        return allSyncItemInfo;

    }
    
    
    /**
     * get messages from folders
     *
     * @param filter
     * @param source_uri  String
     * @param principalId long
     * @return map with all the mail server CrcSyncItemInfo
     * @throws EntityException
     */
    private HashMap getAllEmailsInfoSent(EmailFilter filter,
                                         String source_uri,
                                         long principalId)
      throws EntityException {

        HashMap allSyncItemInfo = new HashMap();

        try {

            // get message
            String[] ids = ped.getAllEmailsSent(filter, source_uri, principalId);

            String GUID      = null;
            String key       = null;
            int itemsNum     = 0;

            if (ids != null) {
                itemsNum = ids.length;
            }

            CrcSyncItemInfo syncItemInfo = null;
            
            for (int i = 0; i < itemsNum; i++) {
                syncItemInfo = new CrcSyncItemInfo(ids[i], 1, 
                                                   null, null, null, 
                                                   null, null, "Y", null);
                allSyncItemInfo.put(ids[i], syncItemInfo);
            }

        } catch (Exception e) {
            throw new EntityException(e);
        }

        return allSyncItemInfo;

    }

    /**
     *
     *
     * @param f           POP3Folder
     * @param FMID        mail id in the folder , not unique
     * @param filter      EmailFilter
     * @param msg         Message
     * @param status      status of the item; char
     * @param source_uri  String
     * @param principalId long
     * @return ItemMessage
     * @throws EntityException
     */
    private ItemMessage getPart(POP3Folder f,
                                String FMID,
                                EmailFilter filter,
                                Message msg,
                                char status,
                                String source_uri,
                                long principalId)
      throws EntityException {

        ItemMessage im = null;
        if (msg != null) {

            try {

                String parentfullpath = f.getFullName();
                if (parentfullpath.equals("INBOX")) {
                    parentfullpath = Def.FOLDER_INBOX_ENG;
                }

                String FID = this.ped.getGUIDFromFullPath(parentfullpath,
                                                          source_uri,
                                                          principalId);
                Locale loc = this.pmsw.getLocale();

                Session session = this.pmsw.getSession();

                if (filter.getSize() == Def.FILTER_SIZE_H) {
                    im = get_H(session, FID, FMID, msg, status, loc, filter);
                } else if (filter.getSize() == Def.FILTER_SIZE_H_B) {
                    im = get_HB(session, FID, FMID, msg, status, loc, -1, filter);
                } else if (filter.getSize() == Def.FILTER_SIZE_H_B_PERC) {
                    im = get_HB(session, FID, FMID, msg, status, loc, filter.getNumBytes(), filter);
                } else if (filter.getSize() == Def.FILTER_SIZE_H_B_A) {
                    im = get_HBA(session, FID, FMID, msg, status, loc, -1, filter);
                } else if (filter.getSize() == Def.FILTER_SIZE_H_B_A_PERC) {
                    im = get_HBA(session, FID, FMID, msg, status, loc, filter.getNumBytes(), filter);
                }
            } catch (EntityException me) {
                throw new EntityException(me);
            }

        }
        return im;
    }


    /**
     * This method creates an email with only header to save in
     * the local DB. 
     * 
     * @param session MAil Server session
     * @param msg complete message
     * @return msgForDB Message with only header
     * @throws EntityException
     */
    private Message createMessageForDB(Message msg)
      throws EntityException {

        Message msgForDB = null;

        try {
            
            msgForDB = new MimeMessage(this.pmsw.getSession());

            // header
            String[] h = Utility.setNewHeader(msgForDB, msg);
            String mid              = h[0];
            int    headersDimension = Integer.parseInt(h[1]);

            // default empty content
            msgForDB.setContent(Def.CONTENT_BODY, Def.CONTENT_CONTENTTYPE);

            msgForDB.saveChanges();

            // the save message command change the message-ID
            msgForDB.setHeader(Def.HEADER_LABEL_MESSAGE_ID, mid);

        } catch (MessagingException me) {
            throw new EntityException(me);
        }

        return msgForDB;

    } 
    
    
    /**
     *
     * return all the emails in the inbox
     * for getSyncItemFromUID:
     * first  cycle : get just 20 mails (if max > 20)
     * second cycle : get just 40 mails (if max > 40)
     * third  cycle : get Max emails
     * for removeSyncItem:
     * first  cycle : get just 20 mails (if max > 20)
     * second cycle : get just 40 mails (if max > 40)
     * third  cycle : get Max emails
     *
     * @param inbox folder
     * @param max number of email in the inbox folder
     * @param cycle
     * @param threshold 1
     * @param threshold 2
     * @return The all items fulfilling the filter requirements as an array of Strings.
     * @throws EntityException
     */
    private Message[] getItemLoopManager(POP3Folder inbox, 
                                         int max,
                                         int loop,
                                         int threshold_1, 
                                         int threshold_2) 
      throws EntityException {

        Message[] messages;
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("getItemLoopManager Start (loop: " + loop + "; max: "+max+
                       "; t_1: "+threshold_1+"; t_2 "+ threshold_2 +";)");
        }
        
        try {
                        
            int mNumber = inbox.getMessageCount();
                        
            if        (loop == 1){
                messages = this.getItemLoop1_2(inbox,mNumber,max,threshold_1);
            } else if (loop == 2){
                messages = this.getItemLoop1_2(inbox,mNumber,max,threshold_2);
            } else if (loop == 3){
                messages = this.getItemLoop3(inbox,mNumber,max);
            } else {
                messages = this.getItemLoop4(inbox);
            }            
                        
        } catch (Exception e) {
            throw new EntityException("Error Getting Max Number Inbox emails (" +
                    e.getMessage() + ")");
        }

        return messages;
        
    }        
    
    /**
     *
     *
     * @param inbox folder
     * @param max number of email in the inbox folder
     * @param threshold 1
     * @return The all items fulfilling the filter requirements as an array of Strings.
     * @throws EntityException
     */
    private Message[] getItemLoop1_2(POP3Folder inbox, 
                                     int mNumber,
                                     int max,
                                     int threshold) 
      throws EntityException {

        Message[] messages;
        
        try {
                                        
           if (mNumber > threshold){
               if (max > threshold){
                    messages = inbox.getMessages((mNumber-(threshold-1)),mNumber);            
               } else {
                    messages = inbox.getMessages((mNumber-(max-1)),mNumber); 
               }
           } else {
               messages = inbox.getMessages();
           }                                            
                        
        } catch (Exception e) {
            throw new EntityException("Error Getting Max Number Inbox emails (" +
                    e.getMessage() + ")");
        }


        return messages;
        
    }  
          
    
    
    /**
     *
     * @param inbox folder
     * @param max number of email in the inbox folder
     * @return The all items fulfilling the filter requirements as an array of Strings.
     * @throws EntityException
     */
    private Message[] getItemLoop3(POP3Folder inbox, 
                                    int mNumber,
                                    int max) 
      throws EntityException {

        Message[] messages;
        
        try {
                                                                
            if (mNumber > max) {
                messages = inbox.getMessages((mNumber-(max-1)),mNumber);            
            } else {
                messages = inbox.getMessages();
            }                                           
                        
        } catch (Exception e) {
            throw new EntityException("Error Getting Max Number Inbox emails (" +
                    e.getMessage() + ")");
        }

        return messages;
        
    }            
    
    /**
     *
     *
     * @param inbox folder
     * @param max number of email in the inbox folder
     * @param cycle
     * @param threshold 1
     * @param threshold 2
     * @return The all items fulfilling the filter requirements as an array of Strings.
     * @throws EntityException
     */
    private Message[] getItemLoop4(POP3Folder inbox) 
      throws EntityException {

        Message[] messages;
        
        try {
                                                
            messages = inbox.getMessages();                            
                        
        } catch (Exception e) {
            throw new EntityException("Error Getting Max Number Inbox emails (" +
                    e.getMessage() + ")");
        }

        return messages;
        
    }        
    

}
