/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.transport;

import java.util.Locale;
import javax.mail.Session;
import javax.mail.Store;
import javax.mail.Folder;

import com.funambol.email.exception.EmailAccessException;
import com.funambol.email.items.model.DefaultFolder;


/**
 *
 * <p>Interface for Mail Server Wrapper </p>
 *
 * @version $Id: IMailServerWrapper.java,v 1.12 2006/09/22 10:07:22 diogo_ajs Exp $
 */
public interface IMailServerWrapper {


    /**
     * opens the connection with the mail server
     *
     *
     * @param serverType String
     * @param outgoingServer String
     * @param outgoingPort String
     * @param incomingServer String
     * @param incomingPort String
     * @param defaultFolder DefaultFolder
     * @param user String
     * @param password String
     * @param isSSL boolean
     * @throws EmailAccessException
     */
    public void openConnection(String  serverType,
                               String  outgoingServer,
                               String  outgoingPort,
                               boolean smtpAuth,
                               String  incomingServer,
                               String  incomingPort,
                               DefaultFolder defaultFolder,
                               String  user,
                               String  password,
                               boolean isSSL,
                               String keystorePath,
                               String keystorePassphrase)
      throws EmailAccessException ;


    /**
     * closes the connection with the mail server
     *
     * @throws EmailAccessException
     */
    public void releaseConnection() throws EmailAccessException;


    /**
     *
     *
     * @param server The hostname of the incoming server.
     * @param incomingPort The port number of the incoming server.
     * @param user The username of the account to be used for the incoming server.
     * @param password The password of the account to be used for the incoming server.
     * @param isSSL Indication whether SSL is to be used.
     * @param keystorePath Path to the keystore file used in SSL certification
     * @param keystorePassphrase Passphrase of the keystore file
     * @throws EmailAccessException
     */
    public void checkUser(String server,
                          String incomingPort,
                          String user,
                          String password,
                          boolean isSSL,
                          String keystorePath,
                          String keystorePassphrase)
     throws EmailAccessException;


    /**
     *
     *
     * @return Session
     */
    public Session getSession();

    /**
     *
     * @return Store
     */
    public Store getStore();

    /**
     *
     * @return DefaultFolder
     */
    public DefaultFolder getDefaultFolder();


    /**
     *
     * @return Folder
     */
    public Folder getMailDefaultFolder();


    /**
     *
     *
     * @return Locale
     */
    public Locale getLocale();
    
    /**
     *
     *
     * @return Server type
     */
    public String getServerType();

}
