/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.transport  ;



/**
 *
 * <p>
 *    This class implemets Semaphore in order to permit
 *    to manager layer to retreive the UID of the added
 *    message.
 * </p>
 *
 * @version $Id: ImapSemaphore.java,v 1.1 2006/08/08 09:32:46 gibi Exp $
 */
public final class ImapSemaphore
{

    private boolean isGreen;

     /**
      * Create a new condition variable in a known state.
      *
      * @param _isGreen boolean
      */
     public ImapSemaphore(boolean _isGreen){
        isGreen = _isGreen;
     }

     /**
      * See if the condition variable is true (without releasing).
      *
      * @return boolean
      */
     public synchronized boolean isTrue()  {
        return isGreen;
     }

     /**
      * Set the condition to false. Waiting threads are not affected.
      */
     public synchronized void setFalse()   {
         isGreen = false;
     }

     /**
      * Set the condition to true. Waiting threads are not released.
      */
     public synchronized void setTrue() {
       isGreen = true;
       notifyAll();
     }

     /**
      * Release all waiting threads without setting the condition true
      */
     public synchronized void releaseAll(){
       notifyAll();
     }

     /**
      * Release one waiting thread without setting the condition true
      */
     public synchronized void releaseOne(){
        notify();
     }

     /**
      * Wait for the condition to become true.
      * @param timeout Timeout in milliseconds
      */
     public synchronized void waitForTrue( long timeout ) {
         try {
           if( !isGreen ){
               wait( timeout );
           }
         } catch (InterruptedException ie){
           ie.printStackTrace();
         }
     }

}
