/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.transport;

import java.util.logging.Level;
import java.util.Properties;
import java.security.Security;
import javax.mail.Session;
import com.sun.net.ssl.internal.ssl.Provider;
import com.sun.mail.pop3.POP3Folder;
import javax.mail.MessagingException;

import com.funambol.email.exception.EmailAccessException;
import com.funambol.email.items.model.DefaultFolder;
import com.funambol.email.util.Def;
import javax.mail.Message;


/**
 *
 * <p>Create a connection with a Mail Server </p>
 *
 * @version $Id: PopMailServerWrapper.java,v 1.22.2.1 2006/12/04 15:49:20 gbmiglia Exp $
 */
public class PopMailServerWrapper
        extends CommonMailServerWrapper
        implements IMailServerWrapper
{

    // -------------------------------------------------------------- Properties

    /**
     * used in the getMessageByUID method. in the POP protocol 
     * there is no the method "folder.getMessageByUID(String)
     * At the end of the sync session the server calls the getSyncItem
     * In this method we open and close the folder every time.
     * Using this propreties we can leave the folder open for alle the
     * calls to the getSyncItem method.
     * In the EndSync we close this folder if it's open
     *
     */
    public Message[]  messagesInboxOpened  = null;
    
    public POP3Folder folderInboxOpened    = null;
   
    // ---------------------------------------------------------- PUBLIC METHODS


    /**
     * opens the connection with the mail server
     *
     *
     * @param serverType Server Type
     * @param outgoingServer SMPT address i.e. 192.168.0.1
     * @param outgoingPort 25 def.
     * @param incomingServer POP3 address i.e. 192.168.0.1
     * @param incomingPort 110 / 995 POP3 def.
     * @param df inbox, outbox, sent items,...
     * @param user String
     * @param password String
     * @param isSSL boolean
     * @throws EmailAccessException
     */
    public void openConnection(String  serverType,
                               String  outgoingServer,
                               String  outgoingPort,
                               boolean smtpAuth,
                               String  incomingServer,
                               String  incomingPort,
                               DefaultFolder df,
                               String  user,
                               String  password,
                               boolean isSSL,
                               String keystorePath,
                               String keystorePassphrase)
      throws EmailAccessException {


      if (log.isLoggable(Level.FINEST)) {
          log.finest("Connection Parameters ["+
                     "(protocol: "+this.getClass().getName()+ ") "+
                     "(out: "+outgoingServer+") (port: "+outgoingPort+ ") "+
                     "(smtpauth: "+smtpAuth+") "+
                     "(in: "+incomingServer+") (port: " + incomingPort+") " +
                     "(user: "+user+ ") (SSL: " + isSSL+ ") "+
                     "(keystore: "+keystorePath+") (passphrase:"+keystorePassphrase+")]");
      }

      // set info
      this.userAddress = user;
      this.serverType  = serverType;
      this.serverOut   = outgoingServer;

      // containe for properties
      Properties props = new Properties();

      // provider for a SSL connection
      Provider provider = null;

      if (isSSL){
        this.setConnectionSSL(props, outgoingServer, outgoingPort,
                              smtpAuth, incomingServer, incomingPort,
                              provider, keystorePath, keystorePassphrase);
      } else {
        this.setConnectionNormal(props, outgoingServer, outgoingPort,
                                 smtpAuth, incomingServer, incomingPort);
      }

      try{

        if (smtpAuth){
           ConnectorAuthenticator auth = new ConnectorAuthenticator();
           auth.setUserName(user);
           auth.setPassword(password);
           this.session = Session.getInstance(props, auth);
        } else {
           this.session = Session.getInstance(props, null);
        }

      } catch (Exception e) {
         log.severe("Error Setting Authentication for Server: " + e);
         log.throwing(this.getClass().getName(), "openConnection", e);
	 throw new EmailAccessException("Error Setting Authentication for Server (" + e.getMessage() + ")");
      }

      boolean isOpen = multipleOpenConnection(incomingServer, user, password);

      if (isOpen){
          try {
              
              // 
              this.mailDefaultFolder = store.getDefaultFolder();
              this.defaultFolder     = df;
              
              // open the INBOX folder
              this.folderInboxOpened = (POP3Folder) 
                          this.mailDefaultFolder.getFolder(Def.FOLDER_INBOX_ENG);
              this.folderInboxOpened.open(javax.mail.Folder.READ_WRITE);                      
              
          } catch (Exception e) {
              log.severe("Error getting Folders: " + e);
              log.throwing(this.getClass().getName(), "openConnection", e);
              throw new EmailAccessException("Error getting Folders (" + e.getMessage() + ")");
          }
          log.severe("POP3 Connection opened");
      } else {
          log.severe("POP3 Error Opening Connection Store - Synchronization Interrupt");
      }

    }



    /**
     * closes the connection with the mail server
     *
     * @throws EmailAccessException
     */
    public void releaseConnection() throws EmailAccessException {
         log.info("POP3 Start Connection Closing");
         try {
            if (this.folderInboxOpened != null) {
                if (this.folderInboxOpened.isOpen()) {
                    this.folderInboxOpened.close(true);
                }
            }            
            if (this.store != null){
                if (this.store.isConnected()){
                   this.store.close();
                }
            }
            this.store = null;
            this.session = null;
        } catch (Exception e) {
     	    throw new EmailAccessException("Error Closing Store " + e.toString());
        }

    }


    /**
     *
     * returns Inbox Folder Instance
     *
     * @return POP3Folder
     * @throws EmailAccessException
     */
    public POP3Folder getInboxFolder() throws EmailAccessException {
       POP3Folder inbox;
       try {
           inbox = (POP3Folder)mailDefaultFolder.getFolder(
                                             defaultFolder.getInboxName());
       } catch (MessagingException me){
           throw new EmailAccessException("Error getting Inbox Folder " +
                                          me.toString());
       }
       return inbox;
    }

    //---------------------------------------------------------- PRIVATE METHODS

   /**
    *
    *
    *
    * @param props Properties
    * @param outgoingServer String
    * @param outgoingPort String
    * @param incomingServer String
    * @param incomingPort String
    * @param provider Provider
    */
   protected void setConnectionSSL(Properties props,
                                   String outgoingServer,
                                   String outgoingPort,
                                   boolean smtpAuth,
                                   String incomingServer,
                                   String incomingPort,
                                   Provider provider,
                                   String keystorePath,
                                   String keystorePassphrase)
   {
        if (log.isLoggable(Level.FINEST)){
           log.finest("POP3 set an SSL connection");
        }
        //
        props.setProperty("mail.smtp.host" , outgoingServer);
        props.setProperty("mail.smtp.port" , outgoingPort);
        props.setProperty("mail.transport.protocol" , "smtp");

        if (smtpAuth){
           props.setProperty("mail.smtp.auth", "true");
        }

        props.setProperty("mail.pop3.host" , incomingServer);
        props.setProperty("mail.pop3.port" , incomingPort);

        props.setProperty("mail.store.protocol", "pop3");        
        
        //Socket connection timeout value in milliseconds. Default is infinite timeout. 
        props.setProperty("mail.pop3.connectiontimeout", Def.TIMEOUT_STORE);
        //Socket I/O timeout value in milliseconds. Default is infinite timeout. 
        props.setProperty("mail.pop3.timeout"          , Def.TIMEOUT_STORE);
        
        System.setProperty("java.protocol.handler.pkgs",
                                      "com.sun.net.ssl.internal.www.protocol");
        System.setProperty("javax.net.ssl.trustStore", keystorePath);
        System.setProperty("javax.net.ssl.trustStorePassword", keystorePassphrase);

        provider = new Provider();
	    Security.addProvider(provider);

        // set SSL parameters
    	props.setProperty("mail.pop3.socketFactory.class",
                                     "javax.net.ssl.SSLSocketFactory");
    	props.setProperty("mail.pop3.socketFactory.fallback",
                                     "false");
    	props.setProperty("mail.pop3.socketFactory.port",
                                     incomingPort);

    }

    /**
     *
     *
     *
     * @param props Properties
     * @param outgoingServer String
     * @param outgoingPort String
     * @param incomingServer String
     * @param incomingPort String
     */
    protected void setConnectionNormal(Properties props,
                                       String outgoingServer,
                                       String outgoingPort,
                                       boolean smtpAuth,
                                       String incomingServer,
                                       String incomingPort){
        if (log.isLoggable(Level.FINEST)) {
           log.finest("POP3 set a normal connection");
        }
        props.setProperty("mail.smtp.host", outgoingServer);
        props.setProperty("mail.smtp.port", outgoingPort);
        props.setProperty("mail.transport.protocol", "smtp");

        if (smtpAuth){
           props.setProperty("mail.smtp.auth", "true");
        }

        props.setProperty("mail.pop3.host"    , incomingServer);
        props.setProperty("mail.pop3.port"    , incomingPort);

        props.setProperty("mail.store.protocol", "pop3");

        //Socket connection timeout value in milliseconds. Default is infinite timeout. 
        props.setProperty("mail.imap.connectiontimeout", Def.TIMEOUT_STORE);
        //Socket I/O timeout value in milliseconds. Default is infinite timeout. 
        props.setProperty("mail.imap.timeout"          , Def.TIMEOUT_STORE);
        
    }

}
