/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.util;


import java.util.Date;
import java.util.Enumeration;
import java.util.ArrayList;
import java.util.Locale;
import java.util.TimeZone;
import java.util.zip.CRC32;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.sql.Timestamp;
import javax.mail.Flags;
import javax.mail.Header;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.internet.MimeUtility;
import javax.mail.search.ComparisonTerm;
import javax.mail.Session;
import javax.mail.internet.MimeMessage;

import com.funambol.email.engine.cache.CrcSyncItemInfo;
import com.funambol.email.exception.EntityException;
import com.funambol.email.items.model.DefaultFolder;
import com.funambol.email.items.model.EmailFilter;
import com.funambol.email.items.model.FlagProperties;
import com.funambol.email.items.model.ItemFolder;
import com.funambol.framework.engine.SyncItem;
import com.funambol.framework.engine.SyncItemKey;
import com.funambol.framework.engine.SyncItemState;
import com.funambol.framework.tools.Base64;

/**
 * <p>Utility class</p>
 *
 * @version $Id: Utility.java,v 1.37.2.4 2006/12/11 15:36:37 gbmiglia Exp $
 */

public class Utility {

    // ------------------------------------------------------------ Public Methods

    private static Logger  log = Logger.getLogger(Def.LOGGER_NAME);    
    
    // ------------------------------------------------------------ Public Methods

    /**
     * get the Messages from the Mail Server
     *
     * @param folder The mail folder name.
     * @param maxNum Maximum number of messages to retrieve.
     * @return Array of messages.
     */
    public static Message[] getMessages(javax.mail.Folder folder, int maxNum) 
     throws MessagingException{
        if (log.isLoggable(Level.FINEST)) {
            log.finest("Get Messages Utility");
        }            
        int max = 100;
        Message[] messages = null;
        int mNumber  = folder.getMessageCount();
        if (mNumber > max){
            if (log.isLoggable(Level.FINEST)) {
                log.finest("Get "+max+" messages");
            }            
            messages = folder.getMessages(mNumber-max,mNumber);
        } else {
            if (log.isLoggable(Level.FINEST)) {
                log.finest("Get all messages");
            }            
            messages = folder.getMessages();
        }
        return messages;
    }
    
    
    public static boolean getBoolean(String value){
        boolean bol = false;
        if (value != null){
            if (!value.equals("")){
                if (value.equals("Y")){
                    bol = true;
                }
            }
        }
        return bol;
    }
    
    
    /**
     * credantials format:  "username:passowrd"
     *
     * @param username    String
     * @param credentials String
     * @return password String
     */
    public static String getPassword(String username, String credentials) {
        String up = new String(Base64.decode(credentials));
        int l = username.length();
        return up.substring(l + 1, up.length());
    }


    /**
     * converts the Nokia folder name in the defaul folder name
     *
     * @param oldfolder folder name in the Server String
     * @param df        DefaultFolder
     * @return newfolder folder name in the Nokia device String
     */
    public static String convertNokiaToDefault(String oldfolder, DefaultFolder df) {
        String newfolder = null;
        if (oldfolder.equalsIgnoreCase(Def.NOKIA_INBOX_ENG)) {
            newfolder = df.getInboxName();
        } else if (oldfolder.equalsIgnoreCase(Def.NOKIA_OUTBOX_ENG)) {
            newfolder = df.getOutboxName();
        } else if (oldfolder.equalsIgnoreCase(Def.NOKIA_SENT_ENG)) {
            newfolder = df.getSentName();
        } else if (oldfolder.equalsIgnoreCase(Def.NOKIA_DRAFTS_ENG)) {
            newfolder = df.getDraftName();
        }
        return newfolder;
    }


    /**
     *
     * converts the id of an email in the sent folder with an ID
     * for and email in the outbox folder
     *
     * @param oldID id that starts with S
     * @return newID id that starts with O 
     */
    public static String convertSentIDToOutboxID(String oldID) {
        String newID = null;
        if (log.isLoggable(Level.FINEST)) {
            log.finest("convertSentIDToOutboxID old id " + oldID);
        }                            
        int i = oldID.indexOf(Def.SEPARATOR_FIRST);        
        newID = Def.FOLDER_OUTBOX_ID + oldID.substring(i, oldID.length());
        if (log.isLoggable(Level.FINEST)) {
            log.finest("convertSentIDToOutboxID new id " + newID);
        }                                    
        return newID;
    }

    /**
     *
     * converts the id of an email in the outbox folder with an ID
     * for and email in the sent folder
     *
     * @param oldID id that starts with O
     * @return newID id that starts with S
     */
    public static String convertOutboxIDToSentID(String oldID) {
        String newID = null;        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("convertOutboxIDToSentID old id " + oldID);
        }                    
        int i = oldID.indexOf(Def.SEPARATOR_FIRST);        
        newID = Def.FOLDER_SENT_ID + oldID.substring(i, oldID.length());        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("convertOutboxIDToSentID new id " + newID);
        }            
        return newID;
    }

    /**
     * get content from a SyncItem
     *
     * @param syncItem SyncItem
     * @return content of the SyncItem String
     * @throws EntityException
     */
    public static String getContentFromSyncItem(SyncItem syncItem)
            throws EntityException {
        String content;
        try {
            byte[] itemContent = syncItem.getContent();
            if (itemContent == null) {
                itemContent = new byte[0];
            }
            content = new String(itemContent);
        }
        catch (Exception e) {
            throw new EntityException("Error getting the item " + syncItem, e);
        }

        return content;
    }


    /**
     * @param message Message
     * @return String
     */
    public static String getHeaderMessageID(Message message) {
        try {
            String[] values = message.getHeader(Def.HEADER_LABEL_MESSAGE_ID);
            if ((values != null) && (values.length > 0)) {
                return values[0];
            }
        } catch (MessagingException exception) {
            // do nothing; we just want to return null
        }
        return null;
    }


    /**
     *
     * get the message id from the email that the client try to send
     * format: Message-ID: O/AAAAAK8wAAACAAAAggAAAA==@192.168.55.101
     *
     * @param msg String
     * @return String
     */
    public static String getHeaderMessageID(String msg) {
        String messageid = null;

        int i = msg.indexOf(Def.HEADER_LABEL_MESSAGE_ID+": ");
        int s = Def.HEADER_LABEL_MESSAGE_ID.length() + 2;
        int start = i + s;
        if (i != -1){
            messageid = msg.substring(start, msg.length());
            int end = messageid.indexOf("\n");
            messageid = messageid.substring(0, end);
        }

        return messageid;
    }


    /**
     * @param message Message
     * @return String
     */
    public static String getHeaderSyncLabel(Message message) {
        
        String syncLabel = null;
        
        try {
            String[] values = message.getHeader(Def.HEADER_LABEL_SYNCMESSAGE);
            if ((values != null) && (values.length > 0)) {
                syncLabel = values[0];
            }
        } catch (MessagingException exception) {
            // do nothing; we just want to return null
        }
        return syncLabel;
    }  
    
    /**
     * Returns the charset in the ContentType
     * It can be used when the standard method
     * <br>
     *  ContentType ct = new ContentType(part.getContentType());
     * <br>
     *  String specifiedCharset = ct.getParameter("charset");
     * <br>
     * doesn't work and return null 
     *
     *
     * @param message Message
     * @return charset String
     */
    public static String getHeaderCharset(Message message) 
      throws EntityException {


        String   charset = null;
        String[] values  = null;
        String   subj    = "";
        
        try {
            subj   = message.getSubject();
            values = message.getHeader(Def.HEADER_LABEL_CONTENT_TYPE);
            if ((values != null) && (values.length == 1)) {
               String s = values[0];
               int chstart = s.indexOf(Def.HEADER_LABEL_CHARSET);
               charset = s.substring(chstart+Def.HEADER_LABEL_CHARSET.length()+2, s.length()-1);
            }
        } catch (MessagingException me) {
            throw new EntityException(
                    "Error getting Header ContentType for email: [" + subj + "]", me); 
        }
        return charset;
    }    

    /**
     * Date : Thu Oct 06 09:46:47 CEST 2005
     * Returns a received date in UTC format
     *
     * @param message Message
     * @param loc     Locale
     * @return String
     */
    public static String getReceivedDate(Message message, Locale loc) {

        String out = null;

        try {
            Date d = message.getReceivedDate();

            // convert in UTC format
            DateFormat utcf = new SimpleDateFormat("yyyyMMdd'T'HHmmss'Z'", loc);
            utcf.setTimeZone(TimeZone.getTimeZone("UTC"));

            out = utcf.format(d);

        }
        catch (Exception e) {
            // do nothing; we just want to return null
        }
        return out;
    }


    /**
     * the method checks if the folder is a default folder
     * On Exchange server the Default folder are:
     * - /Inbox
     * - /Outbox
     * - /Sent
     * - /Draft
     * - /Trash
     * It's depend of the language (see configuration panel)
     *
     * @param df       DefaultFolder
     * @param fullPath String
     * @return boolean
     * @throws EntityException
     */
    public static boolean defaultFolderChecker(DefaultFolder df, String fullPath)
            throws EntityException {
        return fullPath.equals(Def.SEPARATOR_FIRST + df.getInboxName()) ||
                fullPath.equals(Def.SEPARATOR_FIRST + df.getOutboxName()) ||
                fullPath.equals(Def.SEPARATOR_FIRST + df.getSentName()) ||
                fullPath.equals(Def.SEPARATOR_FIRST + df.getDraftName()) ||
                fullPath.equals(Def.SEPARATOR_FIRST + df.getTrashName());
    }

    /**
     * the method checks if the folder is a default folder
     * On Exchange server the Default folder are:
     * - /Inbox
     * - /Outbox
     * - /Sent
     * - /Draft
     * - /Trash
     * It's depend of the language (see configuration panel)
     *
     * @param df       DefaultFolder
     * @param fullPath String
     * @return folder id String
     * @throws EntityException
     */
    public static String getDefaultFolderId(DefaultFolder df, String fullPath)
            throws EntityException {

        if (fullPath.equals(Def.SEPARATOR_FIRST + df.getInboxName())) {
            return Def.FOLDER_INBOX_GUID;
        }
        if (fullPath.equals(Def.SEPARATOR_FIRST + df.getOutboxName())) {
            return Def.FOLDER_OUTBOX_GUID;
        }
        if (fullPath.equals(Def.SEPARATOR_FIRST + df.getSentName())) {
            return Def.FOLDER_SENT_GUID;
        }
        if (fullPath.equals(Def.SEPARATOR_FIRST + df.getDraftName())) {
            return Def.FOLDER_DRAFTS_GUID;
        }
        if (fullPath.equals(Def.SEPARATOR_FIRST + df.getTrashName())) {
            return Def.FOLDER_TRASH_GUID;
        }

        throw new EntityException(fullPath + " is not a default folder.");
    }


    /**
     * @param fullpath String
     * @return String
     */
    public static String getFolderNameFromFullpath(String fullpath) {
        String folderName;
        int lastslash = fullpath.lastIndexOf(Def.SEPARATOR_FIRST);
        folderName = fullpath.substring(lastslash + 1, fullpath.length());
        return folderName;
    }


    /**
     * set folder role using the name
     *
     * @param folderName String
     * @return folder role String
     */
    public static String getFolderRole(String folderName, DefaultFolder df) {
        String folderRole = null;
        if (folderName.equalsIgnoreCase(df.getInboxName())) {
            folderRole = Def.FOLDER_INBOX_ROLE;
        } else if (folderName.equalsIgnoreCase(df.getOutboxName())) {
            folderRole = Def.FOLDER_OUTBOX_ROLE;
        } else if (folderName.equalsIgnoreCase(df.getSentName())) {
            folderRole = Def.FOLDER_SENT_ROLE;
        } else if (folderName.equalsIgnoreCase(df.getDraftName())) {
            folderRole = Def.FOLDER_DRAFTS_ROLE;
        } else if (folderName.equalsIgnoreCase(df.getTrashName())) {
            folderRole = Def.FOLDER_TRASH_ROLE;
        }
        return folderRole;
    }


    /**
     * set creation date of a folder
     *
     * @return UTC creation date String
     */
    public static String getFolderCreationDate() {
        String creationDate;
        creationDate = "20050101T000000Z";
        return creationDate;
    }

    /**
     * gets the parent path of a folder using the full folder path
     *
     * @param fullpath String
     * @return String
     */
    public static String getParentFullpathFromFullpath(String fullpath) {
        String parentFullpath;

        int lastslash = fullpath.lastIndexOf(Def.SEPARATOR_FIRST);

        // gell all before last slash
        parentFullpath = fullpath.substring(0, lastslash);

        // add initial separator
        parentFullpath = Def.SEPARATOR_FIRST + parentFullpath;

        if (parentFullpath.equals(Def.SEPARATOR_FIRST + "INBOX")) {
            parentFullpath = Def.SEPARATOR_FIRST + "Inbox";
        }

        return parentFullpath;
    }


    /**
     *
     * get the ID after the specified position
     * 1 it returns the FID 
     * 2 it returns the FMID
     * 3 it returns the UIDV
     *
     * @param key String
     * @return id (it depends on the separator type)
     */
    public static String getKeyPart(String key, int position) {
        
          String id    = null;
          int    len_1 = Def.SEPARATOR_FIRST.length();
          int    len_2 = Def.SEPARATOR_SECOND.length();

          if        (position == 1){ // get FID
                int endFolderId = key.indexOf(Def.SEPARATOR_FIRST);
                if (endFolderId > 0){
                    id = key.substring(0, endFolderId);
                }
          } else if (position == 2){ // get FMID
                int endFolderId     = key.indexOf(Def.SEPARATOR_FIRST);
                int startValidityId = key.indexOf(Def.SEPARATOR_SECOND);
                if (startValidityId > 0){
                    // there is the Validity ID
                    id = key.substring((endFolderId + len_1), startValidityId);
                } else {
                    // there is not the Validity ID
                    id = key.substring((endFolderId + len_1), key.length());                
                }            
          } else if (position == 3){ // get UIDV
                int startValidityId = key.indexOf(Def.SEPARATOR_SECOND);
                if (startValidityId > 0){
                    id = key.substring((startValidityId + len_2), key.length());
                }            
          }
          
          return id;
          
    }  
    

    /**
     * convert the UTC string in a Timestamp
     *
     * @param utc String
     * @return Timestamp
     */
    public static Timestamp UtcToTimestamp(String utc) {

        Timestamp t = null;
        if (utc != null) {
            try {
                DateFormat utcFormatter = new SimpleDateFormat("yyyyMMdd'T'HHmmss'Z'");
                Date d = utcFormatter.parse(utc);
                t = new Timestamp(d.getTime());
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return t;
    }


    /**
     * convert the UTC string in a Date
     *
     * @param utc String
     * @return Date
     */
    public static Date UtcToDate(String utc) {

        Date d = null;

        if (utc != null) {

            try {
                
                DateFormat utcf = new SimpleDateFormat("yyyyMMdd'T'HHmmss'Z'");
                
                utcf.setTimeZone(TimeZone.getTimeZone("UTC"));
                
                d = utcf.parse(utc);
                
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return d;
    }

    /**
     * convert Message in a String (RFC2822)
     *
     * @param msg Message
     * @return String
     */
    public static String StringFromMessage(Message msg, String encodingType)
      throws MessagingException {
        
        String streamContentAsString;
        boolean isSeen;
        OutputStream outStream;

        if (msg == null) {
            return null;
        }

        isSeen = msg.isSet(Flags.Flag.SEEN);

        outStream = new ByteArrayOutputStream();

        try {
            if (Def.ENCODE_QUOTED_PRINTABLE.equalsIgnoreCase(encodingType)) {
                msg.writeTo(MimeUtility.encode(outStream, Def.ENCODE_QUOTED_PRINTABLE));
            } else {
                if (Def.ENCODE_BASE64.equalsIgnoreCase(encodingType)) {
                    msg.writeTo(MimeUtility.encode(outStream, Def.ENCODE_BASE64));
                } else {
                    msg.writeTo(outStream);
                }
            }
        } catch (IOException e) {
            throw new MessagingException(e.getMessage(), e);
        }

        streamContentAsString = outStream.toString();

        msg.setFlag(Flags.Flag.SEEN, isSeen);
        
        return streamContentAsString;
    }

    /**
     * convert a String (RFC2822) in the Message
     *
     * @param session 
     * @param stream
     * @return Converted message
     */
    public static Message StringToMessage(Session session, String stream)
      throws MessagingException {
        

        if (stream == null) {
            return null;
        }
        
        byte[] b = stream.getBytes();
        
        InputStream is = new ByteArrayInputStream(b);
        
        Message msg = new MimeMessage(session,is);
        
        msg.saveChanges();
                
        return msg;
    }    
    
    /**
     * merge arrays
     *
     * @param i CrcSyncItemInfo[]
     * @param o CrcSyncItemInfo[]
     * @param d CrcSyncItemInfo[]
     * @param s CrcSyncItemInfo[]
     * @param t CrcSyncItemInfo[]
     * @return CrcSyncItemInfo[]
     */
    public static String[] mergeArray(String[] i,
                                      String[] o,
                                      String[] d,
                                      String[] s,
                                      String[] t) {

        String[] ids;

        int length = 0;
        int ilength = 0;
        int olength = 0;
        int dlength = 0;
        int slength = 0;
        int tlength = 0;

        if (i != null) {
            ilength = i.length;
            length = ilength;
        }
        if (o != null) {
            olength = o.length;
            length = length + olength;
        }
        if (d != null) {
            dlength = d.length;
            length = length + dlength;
        }
        if (s != null) {
            slength = s.length;
            length = length + slength;
        }
        if (t != null) {
            tlength = t.length;
            length = length + tlength;
        }

        ids = new String[length];

        int yi, yo, yd, ys, yt;

        for (yi = 0; yi < ilength; yi++) {
            ids[yi] = i[yi];
        }
        for (yo = 0; yo < olength; yo++) {
            ids[yi + yo] = o[yo];
        }
        for (yd = 0; yd < dlength; yd++) {
            ids[yi + yo + yd] = d[yd];
        }
        for (ys = 0; ys < slength; ys++) {
            ids[yi + yo + yd + ys] = s[ys];
        }
        for (yt = 0; yt < tlength; yt++) {
            ids[yi + yo + yd + ys + yt] = t[yt];
        }

        return ids;
    }

    public static HashMap mergeHashMap(HashMap hmi,
                                       HashMap hmo,
                                       HashMap hms,
                                       HashMap hmd,
                                       HashMap hmt){
        HashMap all = new HashMap();
        
        if (hmi != null){
            if (hmi.size()>0){
               all.putAll(hmi);
            }
        }
        if (hmo != null){
            if (hmo.size()>0){
               all.putAll(hmo);
            }
        }
        if (hms != null){
            if (hms.size()>0){
               all.putAll(hms);
            }
        }
        if (hmd != null){
            if (hmd.size()>0){
               all.putAll(hmd);
            }
        }
        if (hmt != null){
            if (hmt.size()>0){
               all.putAll(hmt);
            }
        }

        return all;
        
    }
    
    
    /**
     * merge arrays
     *
     * @param i CrcSyncItemInfo[]
     * @param o CrcSyncItemInfo[]
     * @param d CrcSyncItemInfo[]
     * @param s CrcSyncItemInfo[]
     * @param t CrcSyncItemInfo[]
     * @return CrcSyncItemInfo[]
     */
    public static CrcSyncItemInfo[] mergeArray(CrcSyncItemInfo[] i,
                                               CrcSyncItemInfo[] o,
                                               CrcSyncItemInfo[] d,
                                               CrcSyncItemInfo[] s,
                                               CrcSyncItemInfo[] t) {
        CrcSyncItemInfo[] syncItemInfo;

        int length = 0;
        int ilength = 0;
        int olength = 0;
        int dlength = 0;
        int slength = 0;
        int tlength = 0;

        if (i != null) {
            ilength = i.length;
            length = ilength;
        }
        if (o != null) {
            olength = o.length;
            length = length + olength;
        }
        if (d != null) {
            dlength = d.length;
            length = length + dlength;
        }
        if (s != null) {
            slength = s.length;
            length = length + slength;
        }
        if (t != null) {
            tlength = t.length;
            length = length + tlength;
        }

        syncItemInfo = new CrcSyncItemInfo[length];

        int yi;
        int yo;
        int yd;
        int ys;
        int yt;

        for (yi = 0; yi < ilength; yi++) {
            syncItemInfo[yi] = i[yi];
        }
        for (yo = 0; yo < olength; yo++) {
            syncItemInfo[yi + yo] = o[yo];
        }
        for (yd = 0; yd < dlength; yd++) {
            syncItemInfo[yi + yo + yd] = d[yd];
        }
        for (ys = 0; ys < slength; ys++) {
            syncItemInfo[yi + yo + yd + ys] = s[ys];
        }
        for (yt = 0; yt < tlength; yt++) {
            syncItemInfo[yi + yo + yd + ys + yt] = t[yt];
        }

        return syncItemInfo;
    }

    /**
     * merge arrays
     *
     * @param i CrcSyncItemInfo[]
     * @param o CrcSyncItemInfo[]
     * @param d CrcSyncItemInfo[]
     * @param s CrcSyncItemInfo[]
     * @param t CrcSyncItemInfo[]
     * @return CrcSyncItemInfo[]
     */
    public static ItemFolder[] mergeArray(ItemFolder[] i,
                                          ItemFolder[] o,
                                          ItemFolder[] d,
                                          ItemFolder[] s,
                                          ItemFolder[] t) {
        ItemFolder[] itemFolder;

        int length = 0;
        int ilength = 0;
        int olength = 0;
        int dlength = 0;
        int slength = 0;
        int tlength = 0;

        if (i != null) {
            ilength = i.length;
            length = ilength;
        }
        if (o != null) {
            olength = o.length;
            length = length + olength;
        }
        if (d != null) {
            dlength = d.length;
            length = length + dlength;
        }
        if (s != null) {
            slength = s.length;
            length = length + slength;
        }
        if (t != null) {
            tlength = t.length;
            length = length + tlength;
        }

        itemFolder = new ItemFolder[length];

        int yi, yo, yd, ys, yt;

        for (yi = 0; yi < ilength; yi++) {
            itemFolder[yi] = i[yi];
        }
        for (yo = 0; yo < olength; yo++) {
            itemFolder[yi + yo] = o[yo];
        }
        for (yd = 0; yd < dlength; yd++) {
            itemFolder[yi + yo + yd] = d[yd];
        }
        for (ys = 0; ys < slength; ys++) {
            itemFolder[yi + yo + yd + ys] = s[ys];
        }
        for (yt = 0; yt < tlength; yt++) {
            itemFolder[yi + yo + yd + ys + yt] = t[yt];
        }

        return itemFolder;
    }

    /**
     *
     * Compute CRC-32 checksum value for the specified Message
     *
     * @param msg Message
     * @return long
     */
    public static long getLastCrc(Message msg, String messageid, String date) {

        CRC32 checksumEngine = new CRC32();
        long checksumFlag    = 0;
        long checksumMessage = 0;

        try {

            boolean isSeen     = msg.isSet(Flags.Flag.SEEN);
            boolean isAnswered = msg.isSet(Flags.Flag.ANSWERED);
            boolean isDeleted  = msg.isSet(Flags.Flag.DELETED);
            boolean isDraft    = msg.isSet(Flags.Flag.DRAFT);
            boolean isRecent   = msg.isSet(Flags.Flag.RECENT);
            boolean isFlagged  = msg.isSet(Flags.Flag.FLAGGED);

            FlagProperties fg = new FlagProperties(isSeen, isAnswered,
                                                   isDeleted, isDraft,
                                                   isRecent, isFlagged);

            String fgS = fg.toString();
            byte[] bytesFlags = fgS.getBytes();
            checksumEngine.update(bytesFlags, 0, bytesFlags.length);
            checksumFlag = checksumEngine.getValue();
            checksumEngine.reset();

            String msgS = messageid + date;
            byte[] bytesMsg = msgS.getBytes();
            checksumEngine.update(bytesMsg, 0, bytesMsg.length);
            checksumMessage = checksumEngine.getValue();
            checksumEngine.reset();
            
        } catch (MessagingException me) {
            // do nothing; we just want to return 0
        }

        return checksumFlag + checksumMessage;

    }


    /**
     *
     * set a new header but keep the old Message-ID
     *
     * @param newMsg Message
     * @param oldMsg Message
     * @return header-id and header dimension
     * @throws EntityException
     */
    public static String[] setNewHeader(Message newMsg,
                                        Message oldMsg)
            throws EntityException {


        String   oldMessageID     = null;
        String   label            = "";
        int      headersDimension = 0;
        Header   h                = null;
        String   name             = null;
        String   value            = null;
        String[] out              = new String[2];
        ArrayList receivedList    = new ArrayList();

        try {

            String[] recs  = oldMsg.getHeader(Def.HEADER_LABEL_RECEIVED);
            Enumeration en = oldMsg.getAllHeaders();
            while (en.hasMoreElements()) {
                h = (Header) en.nextElement();
                name  = h.getName();
                value = h.getValue();

                // calculate the header dimension
                label = name + ": " + value;
                int headerlength = label.getBytes().length;
                headersDimension = headersDimension + headerlength;

                if (name.equalsIgnoreCase(Def.HEADER_LABEL_MESSAGE_ID)) {
                    oldMessageID = value;
                }
                if (!name.equals(Def.HEADER_LABEL_RECEIVED)){
                    // doesn't save the received
                    newMsg.addHeader(name, value);
                }                    
            }
            
            // save the received
            if (recs != null){
                for (int i=recs.length-1; i>=0; i--){
                   newMsg.addHeader(Def.HEADER_LABEL_RECEIVED, recs[i]);                
                }
            }

        } catch (MessagingException me) {
            throw new EntityException(me);
        }

        out[0] = oldMessageID;
        out[1] = String.valueOf(headersDimension);

        return out;

    }


    /**
     * the Funambol client send id in format: I/1234
     * the mobile phone can send the LUID not in the format : 1234
     *
     * @param syncItem SyncItem
     * @return FID, FMID and UIDV in an array String[]
     */
    
    public static String[] getKeyParts(SyncItem syncItem) {

        String[] keys = new String[3];

        String FID;
        String FMID;
        String UIDV;
        String folderGUID;
        SyncItemKey parentKey;

        String key = syncItem.getKey().getKeyAsString();

        if (key.indexOf(Def.SEPARATOR_FIRST) >= 0) {
            //  id format I/1234 (Funambol client ... get "1234")
            FMID = getKeyPart(key,2);
        } else {
            //  id format 1234 (mobile phone client ... get "1234")
            FMID = key;
        }

        parentKey = syncItem.getParentKey();

        if (parentKey == null) {
            // get parent from LUID (format I/1234 ... get "I")
            FID = getKeyPart(key,1);
        } else {
            folderGUID = parentKey.getKeyAsString();
            if (folderGUID.indexOf(Def.SEPARATOR_FIRST) >= 0){
               // get parent from GUID (format ROOT/I ... get "I")
               FID = getKeyPart(folderGUID,2);
            } else {
               // get parent from GUID (format I ... get "I")
               FID = folderGUID; 
            }
        }

        // get uid validity IMAP GUID : I/1234-646464
        UIDV = getKeyPart(key,3);        
        
        keys[0] = FID;
        keys[1] = FMID;
        keys[2] = UIDV;

        return keys;
    }
    

    /**
     * sets the EMAIL GUID =  FID + Def.SEPARATOR + FMID + Def.SEPARATOR + UIDV 
     *
     * @param FID  folder id String
     * @param FMID mail id in the folder String
     * @param UIDV folder validity String
     * @return String
     */
    public static String createIMAPGUID(String FID, String FMID, String UIDV) {
        return FID + Def.SEPARATOR_FIRST + FMID + Def.SEPARATOR_SECOND + UIDV;
    }

    /**
     * sets the EMAIL GUID =  FID + Def.SEPARATOR + FMID 
     *
     * @param FID  folder id String
     * @param FMID mail id in the folder String
     * @return String
     */
    public static String createPOPGUID(String FID, String FMID) {
        return FID + Def.SEPARATOR_FIRST + FMID ;
    }
    
    /**
     * sets the FOLDER GUID =  FID + Def.SEPARATOR + FMID 
     *
     * @param FID  folder id String
     * @param FMID mail id in the folder String
     * @return String
     */
    public static String createFolderGUID(String FID, String FMID) {
        return FID + Def.SEPARATOR_FIRST + FMID;
    }

    /**
     *
     * @param timeFilterClause String
     * @return comparison term for time filter int
     */
    public static int getComparisonTerm(String timeFilterClause) {

        int out = ComparisonTerm.GE;

        if (timeFilterClause.equalsIgnoreCase("EQ") ||
                timeFilterClause.equalsIgnoreCase("GT") ||
                timeFilterClause.equalsIgnoreCase("GE")) {
            out = ComparisonTerm.GE;
        } else if (timeFilterClause.equalsIgnoreCase("LT") ||
                timeFilterClause.equalsIgnoreCase("LE")) {
            out = ComparisonTerm.LE;
        }

        return out;
    }

    /**
     * @param origDate Date
     * @return String
     */
    public static String UtcFromDate(Date origDate) {

        String out = null;

        DateFormat utcf = new SimpleDateFormat("yyyyMMdd'T'HHmmss'Z'");

        utcf.setTimeZone(TimeZone.getTimeZone("UTC"));

        if (origDate != null) {
            out = utcf.format(origDate);
        }
        return out;
    }

    /**
     * <p>Decode the given <i>text</i> considering it encoded
     * with the given <i>encode type</i>.</p>
     *
     * @param text
     * @param encodeType
     * @return the decoded text
     * @throws MessagingException
     */
    public static String decode(String text, String encodeType)
            throws MessagingException {
        StringBuffer decodedText;
        InputStream inputStream;
        BufferedInputStream buffIn;
        byte[] tempBuff;
        int readLength;

        if (text == null) {
            return null;
        }

        inputStream = new ByteArrayInputStream(text.getBytes());

        if (Def.ENCODE_QUOTED_PRINTABLE.equals(encodeType)) {
            buffIn = new BufferedInputStream(MimeUtility.decode(inputStream, Def.ENCODE_QUOTED_PRINTABLE));
            tempBuff = new byte[1024];
            decodedText = new StringBuffer();
            try {
                while ((readLength = buffIn.read(tempBuff)) > -1) {
                    decodedText.append(new String(tempBuff, 0, readLength));
                }
            }
            catch (IOException e) {
                throw new MessagingException(e.getMessage(), e);
            }
        }

        return text;
    }


    /**
     * Checks if it's possible to optimize the syncml message
     * dimension.
     * For instance: an email in the inbox with only the 'read' flag updated
     * and already on the client
     * if the we have a filter with a set ID we have to returns false.
     * this should be the case that the client ask all the email.
     *
     *
     * @param status  status of the item; char
     * @param FID     parent id of the item: String
     * @return true if it's possible to optimize the syncml dimension
     *
     */
    public static boolean needsFlagOptimization(char status,
                                                String FID,
                                                EmailFilter filter){

        boolean opt = false;
        
        if (filter != null){
           String filterID = filter.getId();
           if (filterID != null){
               //do not need optimization
               opt = false;
           } else {
               // 
               if (
                    status == SyncItemState.UPDATED
                    &&
                    (FID.equalsIgnoreCase(Def.FOLDER_INBOX_GUID)  ||
                     FID.equalsIgnoreCase(Def.FOLDER_OUTBOX_GUID) ||
                     FID.equalsIgnoreCase(Def.FOLDER_SENT_GUID)   ||
                     FID.equalsIgnoreCase(Def.FOLDER_TRASH_GUID)  )
                   ){
                    opt = true;
               }
           }
        }
        
        return opt;
    }

  /**
   * compare 2 date
   * @param d1 
   * @param d2 
   * @return true if d1 >= d2
   */
  public static boolean d1Afterd2(Date d1, Date d2){
      long d1t = d1.getTime();
      long d2t = d2.getTime();
      boolean check = (d1t >= d2t);
      return  check ;
  }
  
  /**
   * compare 2 date
   * @param d1 
   * @param d2 
   * @return true if d1 < d2
   */
  public static boolean d1Befored2(Date d1, Date d2){
      long d1t = d1.getTime();
      long d2t = d2.getTime();
      boolean check = (d1t < d2t);
      return  check ;
  }
  
  //------------------------------------------------------------ Private Methods
  
  
}
