/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.util;


import java.util.Date;
import java.util.Locale;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import javax.mail.Message;
import javax.mail.MessagingException;
import com.funambol.email.exception.EntityException;
import java.util.logging.Logger;

/**
 * <p>Utility class</p>
 *
 * @version $Id: UtilityDate.java,v 1.1.2.2 2006/12/11 15:37:17 gbmiglia Exp $
 */

public class UtilityDate {

    // ------------------------------------------------------------ Public Methods

    private static Logger  log = Logger.getLogger(Def.LOGGER_NAME);
    
    
    private static DateFormat pat_1 = 
            new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss Z", Locale.US);
    private static DateFormat pat_2 = 
            new SimpleDateFormat("dd MMM yyyy HH:mm:ss Z", Locale.US);
    private static DateFormat pat_3 = 
            new SimpleDateFormat("EEE MMM dd HH:mm:ss Z yyyy", Locale.US);
    private static DateFormat pat_4 = 
            new SimpleDateFormat("EEE MMM dd HH:mm:ss yyyy", Locale.US);
    
    
    // ------------------------------------------------------------ Public Methods


    /**
     * Returns an origination date in UTC format
     *
     * Available Date Parser
     * pat_1: Exchange 2003 (eng): wed, 6 Oct 2005 09:46:47 +0200
     * pat_2: Exchange 2000 (eng): 6 Oct 2005 09:46:47 +0200
     * pat_3: BB client          : Wed Dec 28 20:05:53 GMT 2005
     * pat_4: ??                 : Wed Dec 28 20:05:53 2005
     *
     * before apply the pat_5 parser you have to use the
     * private method bb7290toRFC822()
     * pat_5: BB client 7290     : Wed Feb 01 10:49:21 Europe/Amsterdam 2006
     *
     * @param message Message
     * @param loc     Locale
     * @return date in the header Date
     */
    public static Date getHeaderDate(Message message, Locale loc) 
      throws EntityException {

        Date origDate = null;
        String[] values = null;

        String subj = "";
        
        try {
            values = message.getHeader(Def.HEADER_LABEL_DATE);
        } catch (MessagingException me) {
            try {
                subj = message.getSubject();
            } catch (MessagingException me2){
                // do nothing
            }
            throw new EntityException(
                    "Error getting Header Date for email: [" + subj + "]", me); 
        }

        if ((values != null) && (values.length > 0)) {
            origDate = DateStringToDate(values[0], message);             
        }

        return origDate;
    }   
    
    

  //------------------------------------------------------------ Private Methods
  
  
    /**
     * fix for date in the following format
     *  Tue, 10 Oct 2006 07:11:43 -0060
     */
    private static String getHeaderDatePatch1(String s)     
      throws EntityException {
         
          String timezone = "";
          
          timezone = s.substring((s.length()-5), s.length());
          
          if (timezone.equals("-0060")){
              s = s.substring(0,(s.length()-5)) + "-0100";
          }           
          if (timezone.equals("+0060")){
              s = s.substring(0,(s.length()-5)) + "+0100";
          }
          
          return s;
          
    }        
    
    /**
     * fix for date in the following format
     *  Thu, 19 Oct 2006 17:37:47 UT
     */
   private static String  getHeaderDatePatch2(String s){
      
       String timezone = "";

       timezone = s.substring((s.length()-2), s.length());
      
       if (timezone.equals("UT")){
           s = s.substring(0,(s.length()-2)) + "+0000";
       } 
      
       return s;
   }
    
    /**
     * fix for all strange date format
     * 
     */
   private static String  getHeaderDatePatch3(String s){      
       //return "Thu, 01 Jan 1970 01:00:00 +0000";
       java.util.Date d = new java.util.Date(System.currentTimeMillis());
       DateFormat utcf = new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss Z", Locale.US);
       String out = utcf.format(d);
       return out;
       
   }     
   
   
    /**
     *
     */
    public static Date DateStringToDate(String s, Message message) 
    throws EntityException {

        Date origDate = null;
        String subj   = "";
        
        try {
            origDate = formatter(s);
        } catch (Exception e1) {
            try {
                s = getHeaderDatePatch1(s);
                origDate = formatter(s);
            } catch (Exception e2) {
                try {
                    s = getHeaderDatePatch2(s);
                    origDate = formatter(s);
                } catch (Exception e3) {
                    try {
                        s = getHeaderDatePatch3(s);
                        origDate = formatter(s);
                        try {
                            subj = message.getSubject();
                        } catch (MessagingException me2){
                            // do nothing
                        }
                        log.info("Insert Default value for email: [" + subj + "]");
                    } catch (Exception e4) {
                        try {
                            subj = message.getSubject();
                        } catch (MessagingException me2){
                            // do nothing
                        }
                        throw new EntityException(
                                "Error parsing Header Date for email: [" + subj + "]", e4);                        
                    }
                }
            }                 
        }
        
        return origDate;
    }
  
    
    /**
     *
     */
    public static Date formatter(String s) 
    throws EntityException {

        Date origDate = null;
        try {
            origDate = pat_1.parse(s);
        } catch (Exception e1) {
            try {
                origDate = pat_2.parse(s);
            } catch (Exception e2) {
                try {
                    origDate = pat_3.parse(s);
                } catch (Exception e3) {
                    try {
                        origDate = pat_4.parse(s);
                    } catch (Exception e4) {
                        throw new EntityException("Error parsing Header Date", e4);                                    
                    }
                }
            }
        }
        return origDate;
    }    
    
}
