/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.util;


import com.funambol.email.engine.source.ImapEmailSyncSource;
import com.funambol.email.engine.source.PopEmailSyncSource;
import com.funambol.email.exception.InboxListenerConfigException;
import com.funambol.email.exception.InboxListenerException;
import com.funambol.email.inboxlistener.engine.WSTools;
import com.funambol.email.inboxlistener.model.MSAccount;
import com.funambol.email.inboxlistener.model.WSAccount;
import com.funambol.framework.tools.beans.BeanException;
import com.funambol.framework.tools.beans.BeanFactory;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.logging.Logger;

/**
 * <p>InboxListener Utility class</p>
 *
 * @version $Id: UtilityIL.java,v 1.2 2006/11/16 16:10:32 gbmiglia Exp $
 */

public class UtilityIL {

  //------------------------------------------------------------------ CONSTANTS
    
  private static Logger log = Logger.getLogger(Def.LOGGER_NAME);
    
  // ------------------------------------------------------------ Public Methods

  /**
   *
   */
  public static int readInteger() throws InboxListenerException {

    InputStreamReader iReader = new InputStreamReader(System.in);
    BufferedReader indata     = new BufferedReader(iReader);

    int    value = -1;
    String temp  = null;
    try {
       temp = indata.readLine();
       value = Integer.parseInt(temp);
    } catch (IOException ex) {
       throw new InboxListenerException("Error reading input value (" +
                                         ex.getMessage() + ")");
    } catch (NumberFormatException ex) {
       throw new InboxListenerException("Error converting input value (" +
                                         ex.getMessage() + ")");
    }

    return value;
  }
  
  /**
   *
   */
  public static String readValue() throws InboxListenerException {

    InputStreamReader iReader = new InputStreamReader(System.in);
    BufferedReader indata     = new BufferedReader(iReader);
    String value  = null;
    try {
       value = indata.readLine();
    } catch (IOException ex) {
       throw new InboxListenerException("Error reading input value (" +
                                         ex.getMessage() + ")");
    }
    return value;
  }  
  
  
  /**
   *
   */
  public static int setMilliSec(int minutes) {
      int msecs = 600000; // default 10 minutes
      msecs = ((minutes * 60) * 1000);
      return msecs;
  }  
  
  
  /** 
   * Calls getServerBean method of the WS     
   */
  public static MSAccount getServerBean(WSAccount wsa, int interval) 
   throws InboxListenerConfigException {
      
      MSAccount msa = null;
      
      try {   
          
         WSTools ws  = new WSTools(wsa.getWsEndPoint(),
                                   wsa.getWsUsername(),
                                   wsa.getWsPassword());            
         
         String path = "email/email/email-" + 
                       wsa.getWsSyncSourceType() + "/" + 
                       wsa.getWsSyncSource() + ".xml";
                  
         String req = (String)ws.invoke("getServerBean", new Object[] { path });
                  
         Object obj = BeanFactory.unmarshal(req);
         
         String  mailServer         = null;
         String  host               = null;
         int     port               = 0;
         String  protocol           = null;
         boolean isSSL              = false;
         String  keystorePath       = null;
         String  keystorePassphrase = null;
         
         if (wsa.getWsSyncSourceType().equals(Def.PROTOCOL_IMAP)){
             ImapEmailSyncSource ss = (ImapEmailSyncSource)obj;
             mailServer         = ss.getServerType();
             host               = ss.getInHost();
             port               = Integer.parseInt(ss.getInPort());
             protocol           = "imap";
             isSSL              = ss.isSSL();
             keystorePath       = ss.getKeystorePath();
             keystorePassphrase = ss.getKeystorePassphrase();
         } else {
             PopEmailSyncSource ss = (PopEmailSyncSource)obj;
             mailServer         = ss.getServerType();
             host               = ss.getInHost();
             port               = Integer.parseInt(ss.getInPort());
             protocol           = "pop3";
             isSSL              = ss.isSSL();
             keystorePath       = ss.getKeystorePath();
             keystorePassphrase = ss.getKeystorePassphrase();             
         }
         
         msa = new MSAccount(mailServer, host, isSSL, protocol, port,
                             interval, keystorePath, keystorePassphrase);         
         
     } catch (BeanException e) {
         throw new InboxListenerConfigException(e.getMessage());
     } catch (Exception e) {
         throw new InboxListenerConfigException(e.getMessage());
     }

     return msa;
  }  
    
}
